@@ -1,40 +1,33 @@
-##	advanced.py
+# -*- coding:utf-8 -*-
+## src/advanced.py
 ##
-## Contributors for this file:
-## - Yann Le Boulanger <asterix@lagaule.org>
-## - Nikos Kouremenos <kourem@gmail.com>
-## - Vincent Hanquez <tab@snarc.org>
+## Copyright (C) 2005 Travis Shirk <travis AT pobox.com>
+##                    Vincent Hanquez <tab AT snarc.org>
+## Copyright (C) 2005-2007 Yann Leboulanger <asterix AT lagaule.org>
+##                         Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
+## Copyright (C) 2006-2007 Jean-Marie Traissard <jim AT lapin.org>
 ##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
+## This file is part of Gajim.
 ##
-## This program is free software; you can redistribute it and/or modify
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 import gtk
-import gtk.glade
 import gtkgui_helpers
+import gobject
 
 from common import gajim
-from common import i18n
-
-_ = i18n._
-APP = i18n.APP
-gtk.glade.bindtextdomain(APP, i18n.DIR)
-gtk.glade.textdomain(APP)
 
 (
 OPT_TYPE,
@@ -49,10 +42,48 @@ C_TYPE
 
 GTKGUI_GLADE = 'manage_accounts_window.glade'
 
-class AdvancedConfigurationWindow:
+def rate_limit(rate):
+	''' call func at most *rate* times per second '''
+	def decorator(func):
+		timeout = [None]
+		def f(*args, **kwargs):
+			if timeout[0] is not None:
+				gobject.source_remove(timeout[0])
+				timeout[0] = None
+			def timeout_func():
+				func(*args, **kwargs)
+				timeout[0] = None
+			timeout[0] = gobject.timeout_add(int(1000.0 / rate), timeout_func)
+		return f
+	return decorator
+
+def tree_model_iter_children(model, treeiter):
+	it = model.iter_children(treeiter)
+	while it:
+		yield it
+		it = model.iter_next(it)
+
+def tree_model_pre_order(model, treeiter):
+	yield treeiter
+	for childiter in tree_model_iter_children(model, treeiter):
+		for it in tree_model_pre_order(model, childiter):
+			yield it
+
+try:
+	any(()) # builtin since python 2.5
+except Exception:
+	def any(iterable):
+		for element in iterable:
+			if element:
+				return True
+		return False
+
+class AdvancedConfigurationWindow(object):
 	def __init__(self):
 		self.xml = gtkgui_helpers.get_glade('advanced_configuration_window.glade')
 		self.window = self.xml.get_widget('advanced_configuration_window')
+		self.window.set_transient_for(
+			gajim.interface.instances['preferences'].window)
 		self.entry = self.xml.get_widget('advanced_entry')
 		self.desc_label = self.xml.get_widget('advanced_desc_label')
 		self.restart_label = self.xml.get_widget('restart_label')
@@ -61,9 +92,19 @@ class AdvancedConfigurationWindow:
 		# key = option name (root/subopt/opt separated by \n then)
 		# value = array(oldval, newval)
 		self.changed_opts = {}
+		
+		# For i18n
+		self.right_true_dict = {True: _('Activated'), False: _('Deactivated')} 
+		self.types = {
+			'boolean': _('Boolean'),
+			'integer': _('Integer'),
+			'string': _('Text'),
+			'color': _('Color')} 
 
 		treeview = self.xml.get_widget('advanced_treeview')
+		self.treeview = treeview
 		self.model = gtk.TreeStore(str, str, str)
+		self.fill_model()
 		self.model.set_sort_column_id(0, gtk.SORT_ASCENDING)
 		self.modelfilter = self.model.filter_new()
 		self.modelfilter.set_visible_func(self.visible_func)
@@ -79,17 +120,13 @@ class AdvancedConfigurationWindow:
 			renderer_text, text = 1)
 		col.set_cell_data_func(renderer_text, self.cb_value_column_data)
 
-		if gtk.gtk_version >= (2, 8, 0) and gtk.pygtk_version >= (2, 8, 0):
-			col.set_resizable(True) # there is a bug in 2.6.x series
+		col.props.resizable = True
 		col.set_max_width(250)
 
 		renderer_text = gtk.CellRendererText()
 		treeview.insert_column_with_attributes(-1, _('Type'),
 			renderer_text, text = 2)
 
-		# add data to model
-		gajim.config.foreach(self.fill, self.model)
-
 		treeview.set_model(self.modelfilter)
 
 		# connect signal for selection change
@@ -101,24 +138,24 @@ class AdvancedConfigurationWindow:
 		self.restart_label.hide()
 		gajim.interface.instances['advanced_config'] = self
 
-	def cb_value_column_data(self, col, cell, model, iter):
+	def cb_value_column_data(self, col, cell, model, iter_):
 		'''check if it's boolen or holds password stuff and if yes
 		make the cellrenderertext not editable else it's editable'''
-		optname = model[iter][C_PREFNAME]
-		opttype = model[iter][C_TYPE]
-		if opttype == 'boolean' or optname in ('password', 'gpgpassword'):
+		optname = model[iter_][C_PREFNAME]
+		opttype = model[iter_][C_TYPE]
+		if opttype == self.types['boolean'] or optname == 'password':
 			cell.set_property('editable', False)
 		else:
 			cell.set_property('editable', True)
 
-	def get_option_path(self, model, iter):
+	def get_option_path(self, model, iter_):
 		# It looks like path made from reversed array
 		# path[0] is the true one optname
 		# path[1] is the key name
 		# path[2] is the root of tree
 		# last two is optional
-		path = [model[iter][0].decode('utf-8')]
-		parent = model.iter_parent(iter)
+		path = [model[iter_][0].decode('utf-8')]
+		parent = model.iter_parent(iter_)
 		while parent:
 			path.append(model[parent][0].decode('utf-8'))
 			parent = model.iter_parent(parent)
@@ -143,7 +180,7 @@ class AdvancedConfigurationWindow:
 				self.desc_label.set_text(_('(None)'))
 
 	def remember_option(self, option, oldval, newval):
-		if self.changed_opts.has_key(option):
+		if option in self.changed_opts:
 			self.changed_opts[option] = (self.changed_opts[option][0], newval)
 		else:
 			self.changed_opts[option] = (oldval, newval)
@@ -152,8 +189,11 @@ class AdvancedConfigurationWindow:
 		modelpath = self.modelfilter.convert_path_to_child_path(path)
 		modelrow = self.model[modelpath]
 		option = modelrow[0].decode('utf-8')
-		if modelrow[2] == 'boolean':
-			newval = {'False': 'True', 'True': 'False'}[modelrow[1]]
+		if modelrow[2] == self.types['boolean']:
+			for key in self.right_true_dict.keys():
+				if self.right_true_dict[key] == modelrow[1]:
+					modelrow[1] = key
+			newval = {'False': True, 'True': False}[modelrow[1]]
 			if len(modelpath) > 1:
 				optnamerow = self.model[modelpath[0]]
 				optname = optnamerow[0].decode('utf-8')
@@ -167,7 +207,7 @@ class AdvancedConfigurationWindow:
 				self.remember_option(option, modelrow[1], newval)
 				gajim.config.set(option, newval)
 			gajim.interface.save_config()
-			modelrow[1] = newval
+			modelrow[1] = self.right_true_dict[newval]
 			self.check_for_restart()
 
 	def check_for_restart(self):
@@ -211,60 +251,34 @@ class AdvancedConfigurationWindow:
 	def on_advanced_close_button_clicked(self, widget):
 		self.window.destroy()
 
-	def find_iter(self, model, parent_iter, name):
-		if not parent_iter:
-			iter = model.get_iter_root()
-		else:
-			iter = model.iter_children(parent_iter)
-		while iter:
-			if model[iter][C_PREFNAME].decode('utf-8') == name:
-				break
-			iter = model.iter_next(iter)
-		return iter
+	def fill_model(self, node=None, parent=None):
+		for item, option in gajim.config.get_children(node):
+			name = item[-1]
+			if option is None: # Node
+				newparent = self.model.append(parent, [name, '', ''])
+				self.fill_model(item, newparent)
+			else: # Leaf
+				type_ = self.types[option[OPT_TYPE][0]]
+				if name == 'password':
+					value = _('Hidden')
+				else:
+					value = self.right_true_dict.get(option[OPT_VAL],
+						option[OPT_VAL])
+				self.model.append(parent, [name, value, type_])
 
-	def fill(self, model, name, parents, val):
-		iter = None
-		if parents:
-			for p in parents:
-				iter2 = self.find_iter(model, iter, p)
-				if iter2:
-					iter = iter2
-
-		if not val:
-			model.append(iter, [name, '', ''])
-			return
-		type = ''
-		if val[OPT_TYPE]:
-			type = val[OPT_TYPE][0]
-		value = val[OPT_VAL]
-		if name in ('password', 'gpgpassword'):
-			#we talk about password
-			value = _('Hidden') # override passwords with this string
-		model.append(iter, [name, value, type])
-
-	def visible_func(self, model, iter):
-		str = self.entry.get_text().decode('utf-8')
-		if str in (None, ''):
-			return True # show all
-		name = model[iter][C_PREFNAME].decode('utf-8')
-		# If a child of the iter matches, we return True
-		if model.iter_has_child(iter):
-			iterC = model.iter_children(iter)
-			while iterC:
-				nameC = model[iterC][C_PREFNAME].decode('utf-8')
-				if model.iter_has_child(iterC):
-					iterCC = model.iter_children(iterC)
-					while iterCC:
-						nameCC = model[iterCC][C_PREFNAME].decode('utf-8')
-						if nameCC.find(str) != -1:
-							return True
-						iterCC = model.iter_next(iterCC)
-				elif nameC.find(str) != -1:
-					return True
-				iterC = model.iter_next(iterC)
-		elif name.find(str) != -1:
-			return True
-		return False
+	def visible_func(self, model, treeiter):
+		search_string  = self.entry.get_text()
+		return any(search_string in model[it][C_PREFNAME] for it in
+			tree_model_pre_order(model, treeiter) if model[it][C_TYPE] != '')
 
+	@rate_limit(3)
 	def on_advanced_entry_changed(self, widget):
 		self.modelfilter.refilter()
+		if not widget.get_text():
+			# Maybe the expanded rows should be remembered here ...
+			self.treeview.collapse_all()
+		else:
+			# ... and be restored correctly here
+			self.treeview.expand_all()	
+
+# vim: se ts=3:
