@@ -1,27 +1,26 @@
-##	cell_renderer_image.py
+# -*- coding:utf-8 -*-
+## src/cell_renderer_image.py
 ##
-## Contributors for this file:
-## - Yann Le Boulanger <asterix@lagaule.org>
-## - Nikos Kouremenos <kourem@gmail.com>
+## Copyright (C) 2003-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2005 Vincent Hanquez <tab AT snarc.org>
+## Copyright (C) 2005-2007 Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2006 Travis Shirk <travis AT pobox.com>
 ##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <kourem@gmail.com>
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
+## This file is part of Gajim.
 ##
-## This program is free software; you can redistribute it and/or modify
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
+
 
 import gtk
 import gobject
@@ -46,8 +45,9 @@ class CellRendererImage(gtk.GenericCellRenderer):
 	def do_get_property(self, pspec):
 		return getattr(self, pspec.name)
 
-	def func(self, model, path, iter, (image, tree)):
-		if model.get_value(iter, self.tv_index) != image:
+	def func(self, model, path, iter_, image_tree):
+		image, tree = image_tree	
+		if model.get_value(iter_, self.tv_index) != image:
 			return
 		self.redraw = 1
 		col = tree.get_column(self.col_index)
@@ -107,7 +107,7 @@ class CellRendererImage(gtk.GenericCellRenderer):
 		window.draw_pixbuf(widget.style.black_gc, pix,
 					draw_rect.x - pix_rect.x,
 					draw_rect.y - pix_rect.y,
-					draw_rect.x, draw_rect.y + 2,
+					draw_rect.x, draw_rect.y,
 					draw_rect.width, draw_rect.height,
 					gtk.gdk.RGB_DITHER_NONE, 0, 0)
 
@@ -136,5 +136,4 @@ class CellRendererImage(gtk.GenericCellRenderer):
 					self.get_property('ypad'))
 		return x_offset, y_offset, calc_width, calc_height
 
-if gtk.pygtk_version < (2, 8, 0):  
-	gobject.type_register(CellRendererImage)
+# vim: se ts=3:
