@@ -1,53 +1,47 @@
-##	common/GnuPG.py
+## src/common/GnuPG.py
 ##
-## Contributors for this file:
-##	- Yann Le Boulanger <asterix@lagaule.org>
-##	- Nikos Kouremenos <nkour@jabber.org>
+## Copyright (C) 2003-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2005 Alex Mauer <hawke AT hawkesnest.net>
+## Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2005-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2007 Stephan Erb <steve-e AT h3c.de>
+## Copyright (C) 2008 Jean-Marie Traissard <jim AT lapin.org>
+##                    Jonathan Schleifer <js-gajim AT webkeks.org>
 ##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
+## This file is part of Gajim.
 ##
-## This program is free software; you can redistribute it and/or modify
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
+import gajim
 from os import tmpfile
 from common import helpers
 
-USE_GPG = True
+if gajim.HAVE_GPG:
+	import GnuPGInterface
 
-try:
-	import GnuPGInterface # Debian package doesn't distribute 'our' file
-except ImportError:
-	try:
-		from common import GnuPGInterface # use 'our' file
-	except ImportError:
-		USE_GPG = False # user can't do OpenGPG only if he or she removed the file!
-	
-else:
 	class GnuPG(GnuPGInterface.GnuPG):
-		def __init__(self):
+		def __init__(self, use_agent = False):
 			GnuPGInterface.GnuPG.__init__(self)
+			self.use_agent = use_agent
 			self._setup_my_options()
 
 		def _setup_my_options(self):
 			self.options.armor = 1
 			self.options.meta_interactive = 0
 			self.options.extra_args.append('--no-secmem-warning')
-			# Nolith's patch - prevent crashs on non fully-trusted keys
-			self.options.extra_args.append('--always-trust')
+			if self.use_agent:
+				self.options.extra_args.append('--use-agent')
 
 		def _read_response(self, child_stdout):
 			# Internal method: reads all the output from GPG, taking notice
@@ -59,7 +53,7 @@ else:
 			# for that keyword.
 
 			resp = {}
-			while 1:
+			while True:
 				line = helpers.temp_failure_retry(child_stdout.readline)
 				if line == "": break
 				line = line.rstrip()
@@ -74,27 +68,44 @@ else:
 						resp[ keyword ] = ""
 			return resp
 
-		def encrypt(self, str, recipients):
-			if not USE_GPG:
-				return str
+		def encrypt(self, str_, recipients):
 			self.options.recipients = recipients   # a list!
 
-			proc = self.run(['--encrypt'], create_fhs=['stdin', 'stdout'])
-			proc.handles['stdin'].write(str)
-			proc.handles['stdin'].close()
+			proc = self.run(['--encrypt'], create_fhs=['stdin', 'stdout', 'status',
+				'stderr'])
+			proc.handles['stdin'].write(str_)
+			try:
+				proc.handles['stdin'].close()
+			except IOError:
+				pass
 
 			output = proc.handles['stdout'].read()
-			proc.handles['stdout'].close()
+			try:
+				proc.handles['stdout'].close()
+			except IOError:
+				pass
+
+			stat = proc.handles['status']
+			resp = self._read_response(stat)
+			try:
+				proc.handles['status'].close()
+			except IOError:
+				pass
+
+			error = proc.handles['stderr'].read()
+			proc.handles['stderr'].close()
 
 			try: proc.wait()
 			except IOError: pass
-			return self._stripHeaderFooter(output)
+			if 'BEGIN_ENCRYPTION' in resp and 'END_ENCRYPTION' in resp:
+				# Encryption succeeded, even if there is output on stderr. Maybe
+				# verbose is on
+				error = ''
+			return self._stripHeaderFooter(output), error
 
-		def decrypt(self, str, keyID):
-			if not USE_GPG:
-				return str
+		def decrypt(self, str_, keyID):
 			proc = self.run(['--decrypt', '-q', '-u %s'%keyID], create_fhs=['stdin', 'stdout'])
-			enc = self._addHeaderFooter(str, 'MESSAGE')
+			enc = self._addHeaderFooter(str_, 'MESSAGE')
 			proc.handles['stdin'].write(enc)
 			proc.handles['stdin'].close()
 	
@@ -105,11 +116,9 @@ else:
 			except IOError: pass
 			return output
 
-		def sign(self, str, keyID):
-			if not USE_GPG:
-				return str
+		def sign(self, str_, keyID):
 			proc = self.run(['-b', '-u %s'%keyID], create_fhs=['stdin', 'stdout', 'status', 'stderr'])
-			proc.handles['stdin'].write(str)
+			proc.handles['stdin'].write(str_)
 			try:
 				proc.handles['stdin'].close()
 			except IOError:
@@ -131,18 +140,16 @@ else:
 
 			try: proc.wait()
 			except IOError: pass
-			if resp.has_key('GOOD_PASSPHRASE') or resp.has_key('SIG_CREATED'):
+			if 'GOOD_PASSPHRASE' in resp or 'SIG_CREATED' in resp:
 				return self._stripHeaderFooter(output)
 			return 'BAD_PASSPHRASE'
 
-		def verify(self, str, sign):
-			if not USE_GPG:
-				return str
-			if not str:
+		def verify(self, str_, sign):
+			if str_ is None:
 				return ''
 			f = tmpfile()
 			fd = f.fileno()
-			f.write(str)
+			f.write(str_)
 			f.seek(0)
 
 			proc = self.run(['--verify', '--enable-special-filenames', '-', '-&%s'%fd], create_fhs=['stdin', 'status', 'stderr'])
@@ -159,15 +166,13 @@ else:
 
 			try: proc.wait()
 			except IOError: pass
-
+			
 			keyid = ''
-			if resp.has_key('GOODSIG'):
+			if 'GOODSIG' in resp:
 				keyid = resp['GOODSIG'].split()[0]
 			return keyid
 
 		def get_keys(self, secret = False):
-			if not USE_GPG:
-				return {}
 			if secret:
 				opt = '--list-secret-keys'
 			else:
@@ -183,8 +188,10 @@ else:
 				sline = line.split(':')
 				if (sline[0] == 'sec' and secret) or \
 						(sline[0] == 'pub' and not secret):
+					# decode escaped chars
+					name = eval('"' + sline[9].replace('"', '\\"') + '"')
 					# make it unicode instance
-					keys[sline[4][8:]] = helpers.decode_string(sline[9])
+					keys[sline[4][8:]] = helpers.decode_string(name)
 			return keys
 			try: proc.wait()
 			except IOError: pass
@@ -208,11 +215,13 @@ else:
 			line = '\n'.join(lines[0:i])
 			return line
 
-		def _addHeaderFooter(self, data, type):
+		def _addHeaderFooter(self, data, type_):
 			"""Add header and footer from data"""
-			out = "-----BEGIN PGP %s-----\n" % type
+			out = "-----BEGIN PGP %s-----\n" % type_
 			out = out + "Version: PGP\n"
 			out = out + "\n"
 			out = out + data + "\n"
-			out = out + "-----END PGP %s-----\n" % type
+			out = out + "-----END PGP %s-----\n" % type_
 			return out
+
+# vim: se ts=3:
