@@ -1,3 +1,26 @@
+# -*- coding:utf-8 -*-
+## src/common/GnuPGInterface.py
+##
+## Copyright (C) 2001 Frank J. Tobin <ftobin AT neverending.org>
+## Copyright (C) 2005 Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2006-2007 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2008 Jean-Marie Traissard <jim AT lapin.org>
+##
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published
+## by the Free Software Foundation; version 3 only.
+##
+## Gajim is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+## GNU General Public License for more details.
+##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
+
 """Interface to GNU Privacy Guard (GnuPG)
 
 GnuPGInterface is a Python module to interface with GnuPG.
@@ -196,28 +219,6 @@ Here is an example of generating a key:
 >>> proc.handles['logger'].close()
 >>> 
 >>> proc.wait()
-
-
-COPYRIGHT:
-
-Copyright (C) 2001  Frank J. Tobin, ftobin@neverending.org
-
-LICENSE:
-
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Lesser General Public
-License as published by the Free Software Foundation; either
-version 2.1 of the License, or (at your option) any later version.
-
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-or see http://www.gnu.org/copyleft/lesser.html
 """
 
 import os
@@ -336,19 +337,19 @@ class GnuPG:
         and can be written to.
         """
         
-        if args == None: args = []
-        if create_fhs == None: create_fhs = []
-        if attach_fhs == None: attach_fhs = {}
+        if args is None: args = []
+        if create_fhs is None: create_fhs = []
+        if attach_fhs is None: attach_fhs = {}
         
         for std in _stds:
-            if not attach_fhs.has_key(std) \
+            if std not in attach_fhs \
                and std not in create_fhs:
                 attach_fhs.setdefault(std, getattr(sys, std))
         
         handle_passphrase = 0
         
-        if self.passphrase != None \
-           and not attach_fhs.has_key('passphrase') \
+        if self.passphrase is not None \
+           and 'passphrase' not in attach_fhs \
            and 'passphrase' not in create_fhs:
             handle_passphrase = 1
             create_fhs.append('passphrase')
@@ -372,7 +373,7 @@ class GnuPG:
         process = Process()
         
         for fh_name in create_fhs + attach_fhs.keys():
-            if not _fd_modes.has_key(fh_name):
+            if fh_name not in _fd_modes:
                 raise KeyError, \
                       "unrecognized filehandle name '%s'; must be one of %s" \
                       % (fh_name, _fd_modes.keys())
@@ -380,7 +381,7 @@ class GnuPG:
         for fh_name in create_fhs:
             # make sure the user doesn't specify a filehandle
             # to be created *and* attached
-            if attach_fhs.has_key(fh_name):
+            if fh_name in attach_fhs:
                 raise ValueError, \
                       "cannot have filehandle '%s' in both create_fhs and attach_fhs" \
                       % fh_name
@@ -564,11 +565,16 @@ class Options:
     def get_standard_args( self ):
         """Generate a list of standard, non-meta or extra arguments"""
         args = []
-        if self.homedir != None: args.extend( [ '--homedir', self.homedir ] )
-        if self.options != None: args.extend( [ '--options', self.options ] )
-        if self.comment != None: args.extend( [ '--comment', self.comment ] )
-        if self.compress_algo != None: args.extend( [ '--compress-algo', self.compress_algo ] )
-        if self.default_key != None: args.extend( [ '--default-key', self.default_key ] )
+        if self.homedir is not None:
+            args.extend( [ '--homedir', self.homedir ] )
+        if self.options is not None:
+            args.extend( [ '--options', self.options ] )
+        if self.comment is not None:
+            args.extend( [ '--comment', self.comment ] )
+        if self.compress_algo is not None:
+            args.extend( [ '--compress-algo', self.compress_algo ] )
+        if self.default_key is not None:
+            args.extend( [ '--default-key', self.default_key ] )
         
         if self.no_options: args.append( '--no-options' )
         if self.armor: args.append( '--armor' )
@@ -647,3 +653,5 @@ GnuPGInterface = GnuPG
 
 if __name__ == '__main__':
     _run_doctests()
+
+# vim: se ts=3:
