@@ -1,4 +1,26 @@
 # this will go to src/common/xmpp later, for now it is in src/common
+# -*- coding:utf-8 -*-
+## src/common/dataforms.py
+##
+## Copyright (C) 2006-2007 Tomasz Melcer <liori AT exroot.org>
+## Copyright (C) 2006-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2007 Stephan Erb <steve-e AT h3c.de>
+##
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published
+## by the Free Software Foundation; version 3 only.
+##
+## Gajim is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+## GNU General Public License for more details.
+##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
+
 """ This module contains wrappers for different parts of data forms (JEP 0004). For information
 how to use them, read documentation. """
 
@@ -76,7 +98,9 @@ def ExtendForm(node):
 		return SimpleDataForm(extend=node)
 
 class DataField(ExtendedNode):
-	""" Keeps data about one field - var, field type, labels, instructions... """
+	""" Keeps data about one field - var, field type, labels, instructions...
+	Base class for different kinds of fields. Use Field() function to
+	construct one of these. """
 	def __init__(self, typ=None, var=None, value=None, label=None, desc=None, required=False,
 		options=None, extend=None):
 
@@ -127,7 +151,8 @@ class DataField(ExtendedNode):
 			assert isinstance(value, basestring)
 			self.setAttr('label', value)
 		def fdel(self):
-			self.delAttr('label')
+			if self.getAttr('label'):
+				self.delAttr('label')
 		return locals()
 
 	@nested_property
@@ -222,7 +247,10 @@ class ListField(DataField):
 		for element in self.iterTags('option'):
 			v = element.getTagData('value')
 			if v is None: raise WrongFieldValue
-			yield (v, element.getAttr('label'))
+			l = element.getAttr('label')
+			if not l:
+				l = v
+			yield (v, l)
 
 class ListSingleField(ListField, StringField):
 	'''Covers list-single and jid-single fields.'''
@@ -326,12 +354,12 @@ class DataRecord(ExtendedNode):
 		return self.vars[item]
 
 class DataForm(ExtendedNode):
-	def __init__(self, type=None, title=None, instructions=None, extend=None):
+	def __init__(self, type_=None, title=None, instructions=None, extend=None):
 		if extend is None:
 			# we have to build form from scratch
 			xmpp.Node.__init__(self, 'x', attrs={'xmlns': xmpp.NS_DATA})
 
-		if type is not None:		self.type=type
+		if type_ is not None:		self.type_=type_
 		if title is not None:		self.title=title
 		if instructions is not None:	self.instructions=instructions
 
@@ -342,9 +370,9 @@ class DataForm(ExtendedNode):
 			filledform = DataForm(replyto=thisform)...'''
 		def fget(self):
 			return self.getAttr('type')
-		def fset(self, type):
-			assert type in ('form', 'submit', 'cancel', 'result')
-			self.setAttr('type', type)
+		def fset(self, type_):
+			assert type_ in ('form', 'submit', 'cancel', 'result')
+			self.setAttr('type', type_)
 		return locals()
 
 	@nested_property
@@ -381,13 +409,32 @@ class DataForm(ExtendedNode):
 		return locals()
 
 class SimpleDataForm(DataForm, DataRecord):
-	def __init__(self, type=None, title=None, instructions=None, fields=None, extend=None):
-		DataForm.__init__(self, type=type, title=title, instructions=instructions, extend=extend)
+	def __init__(self, type_=None, title=None, instructions=None, fields=None, extend=None):
+		DataForm.__init__(self, type_=type_, title=title, instructions=instructions, extend=extend)
 		DataRecord.__init__(self, fields=fields, extend=self, associated=self)
+	
+	def get_purged(self):
+		c = SimpleDataForm(extend=self)
+		del c.title
+		c.instructions = ''
+		to_be_removed = []
+		for f in c.iter_fields():
+			if f.required:
+				# Keep all required fields
+				continue
+			if (hasattr(f, 'value') and not f.value) or (hasattr(f, 'values') and \
+			len(f.values) == 0):
+				to_be_removed.append(f)
+			else:
+				del f.label
+				del f.description
+		for f in to_be_removed:
+			c.delChild(f)
+		return c
 
 class MultipleDataForm(DataForm):
-	def __init__(self, type=None, title=None, instructions=None, items=None, extend=None):
-		DataForm.__init__(self, type=type, title=title, instructions=instructions, extend=extend)
+	def __init__(self, type_=None, title=None, instructions=None, items=None, extend=None):
+		DataForm.__init__(self, type_=type_, title=title, instructions=instructions, extend=extend)
 		# all records, recorded into DataRecords
 		if extend is None:
 
@@ -439,3 +486,5 @@ class MultipleDataForm(DataForm):
 #			self.addChild(node=record)
 #		return locals()
 
+
+# vim: se ts=3:
