@@ -1,19 +1,31 @@
-##	common/gajim.py
+# -*- coding:utf-8 -*-
+## src/common/gajim.py
 ##
-## Copyright (C) 2003-2006 Yann Le Boulanger <asterix@lagaule.org>
-## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
-## Copyright (C) 2005-2006 Dimitur Kirov <dkirov@gmail.com>
-## Copyright (C) 2005-2006 Travis Shirk <travis@pobox.com>
+## Copyright (C) 2003-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2005-2006 Dimitur Kirov <dkirov AT gmail.com>
+##                         Travis Shirk <travis AT pobox.com>
+##                         Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2006 Junglecow J <junglecow AT gmail.com>
+##                    Stefan Bethge <stefan AT lanpartei.de>
+## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+## Copyright (C) 2007-2008 Brendan Taylor <whateley AT gmail.com>
+##                         Stephan Erb <steve-e AT h3c.de>
+## Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
 ##
-## This program is free software; you can redistribute it and/or modify
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 import sys
 import logging
@@ -22,6 +34,7 @@ import locale
 import config
 from contacts import Contacts
 from events import Events
+import xmpp
 
 try:
 	import defs
@@ -54,12 +67,15 @@ config = config.Config()
 version = config.get('version')
 connections = {} # 'account name': 'account (connection.Connection) instance'
 verbose = False
+ipython_window = None
 
 h = logging.StreamHandler()
 f = logging.Formatter('%(asctime)s %(name)s: %(message)s', '%d %b %Y %H:%M:%S')
 h.setFormatter(f)
 log = logging.getLogger('Gajim')
 log.addHandler(h)
+del h
+del f
 
 import logger
 logger = logger.Logger() # init the logger
@@ -71,6 +87,10 @@ LOGPATH = gajimpaths['LOG'] # deprecated
 VCARD_PATH = gajimpaths['VCARD']
 AVATAR_PATH = gajimpaths['AVATAR']
 MY_EMOTS_PATH = gajimpaths['MY_EMOTS']
+MY_ICONSETS_PATH = gajimpaths['MY_ICONSETS']
+MY_MOOD_ICONSETS_PATH = gajimpaths['MY_MOOD_ICONSETS']
+MY_ACTIVITY_ICONSETS_PATH = gajimpaths['MY_ACTIVITY_ICONSETS']
+MY_CACERTS =  gajimpaths['MY_CACERTS']
 TMP = gajimpaths['TMP']
 DATA_DIR = gajimpaths['DATA']
 HOME_DIR = gajimpaths['HOME']
@@ -124,14 +144,50 @@ status_before_autoaway = {}
 # be online
 transport_avatar = {} # {transport_jid: [jid_list]}
 
+# Is Gnome configured to activate on single click ?
+single_click = False
 SHOW_LIST = ['offline', 'connecting', 'online', 'chat', 'away', 'xa', 'dnd',
-	'invisible']
+	'invisible', 'error']
 
 # zeroconf account name
 ZEROCONF_ACC_NAME = 'Local'
-priority_dict = {}
-for status in ('online', 'chat', 'away', 'xa', 'dnd', 'invisible'):
-	priority_dict[status] = config.get('autopriority' + status)
+
+HAVE_PYCRYPTO = True
+try:
+	import Crypto
+except ImportError:
+	HAVE_PYCRYPTO = False
+
+HAVE_PYSEXY = True
+try:
+	import sexy
+except ImportError:
+	HAVE_PYSEXY = False
+
+HAVE_GPG = True
+try:
+	import GnuPGInterface 
+except ImportError:
+	HAVE_GPG = False
+else:
+	from os import system
+	if system('gpg -h >/dev/null 2>&1'):
+		HAVE_GPG = False
+
+gajim_identity = {'type': 'pc', 'category': 'client', 'name': 'Gajim'}
+gajim_common_features = [xmpp.NS_BYTESTREAM, xmpp.NS_SI, xmpp.NS_FILE,
+	xmpp.NS_MUC, xmpp.NS_MUC_USER, xmpp.NS_MUC_ADMIN, xmpp.NS_MUC_OWNER,
+	xmpp.NS_MUC_CONFIG, xmpp.NS_COMMANDS, xmpp.NS_DISCO_INFO, 'ipv6',
+	'jabber:iq:gateway', xmpp.NS_LAST, xmpp.NS_PRIVACY, xmpp.NS_PRIVATE,
+	xmpp.NS_REGISTER, xmpp.NS_VERSION, xmpp.NS_DATA, xmpp.NS_ENCRYPTED, 'msglog',
+	'sslc2s', 'stringprep', xmpp.NS_PING, xmpp.NS_TIME_REVISED, xmpp.NS_SSN,
+	xmpp.NS_MOOD, xmpp.NS_ACTIVITY, xmpp.NS_NICK]
+
+# Optional features gajim supports per account
+gajim_optional_features = {}
+
+# Capabilities hash per account
+caps_hash = {}
 
 def get_nick_from_jid(jid):
 	pos = jid.find('@')
@@ -167,7 +223,7 @@ def get_real_jid_from_fjid(account, fjid):
 	if not nick: # It's not a fake_jid, it is a real jid
 		return fjid # we return the real jid
 	real_jid = fjid
-	if interface.msg_win_mgr.get_control(room_jid, account):
+	if interface.msg_win_mgr.get_gc_control(room_jid, account):
 		# It's a pm, so if we have real jid it's in contact.jid
 		gc_contact = contacts.get_gc_contact(account, room_jid, nick)
 		if not gc_contact:
@@ -236,16 +292,21 @@ def account_is_connected(account):
 def account_is_disconnected(account):
 	return not account_is_connected(account)
 
+def zeroconf_is_connected():
+	return account_is_connected(ZEROCONF_ACC_NAME) and \
+		config.get_per('accounts', ZEROCONF_ACC_NAME, 'is_zeroconf')
+
 def get_number_of_securely_connected_accounts():
 	'''returns the number of the accounts that are SSL/TLS connected'''
 	num_of_secured = 0
-	for account in connections:
+	for account in connections.keys():
 		if account_is_securely_connected(account):
 			num_of_secured += 1
 	return num_of_secured
 
 def account_is_securely_connected(account):
-	if account in con_types and con_types[account] in ('tls', 'ssl'):
+	if account_is_connected(account) and \
+	account in con_types and con_types[account] in ('tls', 'ssl'):
 		return True
 	else:
 		return False
@@ -269,7 +330,8 @@ def get_transport_name_from_jid(jid, use_config_setting = True):
 		# now we support both 'icq.' and 'icq' but not icqsucks.org
 		host = host_splitted[0]
 
-	if host in ('aim', 'irc', 'icq', 'msn', 'sms', 'tlen', 'weather', 'yahoo'):
+	if host in ('aim', 'irc', 'icq', 'msn', 'sms', 'tlen', 'weather', 'yahoo',
+	'mrim'):
 		return host
 	elif host == 'gg':
 		return 'gadu-gadu'
@@ -333,3 +395,5 @@ def get_priority(account, show):
 	config.get_per('accounts', account, 'adjust_priority_with_status'):
 		return config.get_per('accounts', account, 'autopriority_' + show)
 	return config.get_per('accounts', account, 'priority')
+
+# vim: se ts=3:
