@@ -1,20 +1,33 @@
-##	common/helpers.py
+# -*- coding:utf-8 -*-
+## src/common/helpers.py
 ##
-## Copyright (C) 2003-2006 Yann Le Boulanger <asterix@lagaule.org>
-## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
-## Copyright (C) 2005
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
+## Copyright (C) 2003-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2005-2006 Dimitur Kirov <dkirov AT gmail.com>
+##                         Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2006 Alex Mauer <hawke AT hawkesnest.net>
+## Copyright (C) 2006-2007 Travis Shirk <travis AT pobox.com>
+## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+## Copyright (C) 2007 Lukas Petrovicky <lukas AT petrovicky.net>
+##                    James Newton <redshodan AT gmail.com>
+##                    Julien Pivotto <roidelapluie AT gmail.com>
+## Copyright (C) 2007-2008 Stephan Erb <steve-e AT h3c.de>
+## Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
+##                    Jonathan Schleifer <js-gajim AT webkeks.org>
 ##
-## This program is free software; you can redistribute it and/or modify
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 import re
 import locale
@@ -24,23 +37,40 @@ import urllib
 import errno
 import select
 import sha
+import base64
+import sys
 from encodings.punycode import punycode_encode
-from encodings import idna
 
 import gajim
 from i18n import Q_
 from i18n import ngettext
-from xmpp_stringprep import nodeprep, resourceprep, nameprep
+import xmpp
 
+try:
+	# Python 2.5
+	import hashlib
+	hash_md5  = hashlib.md5
+	hash_sha1 = hashlib.sha1
+except ImportError:
+	# Python 2.4
+	import md5
+	import sha
+	hash_md5 = md5.new
+	hash_sha1 = sha.new
+
+try:
+	from osx import nsapp
+except ImportError:
+	pass
 
 try:
 	import winsound # windows-only built-in module for playing wav
 	import win32api
 	import win32con
-except:
+except Exception:
 	pass
 
-special_groups = (_('Transports'), _('Not in Roster'), _('Observers'))
+special_groups = (_('Transports'), _('Not in Roster'), _('Observers'), _('Groupchats'))
 
 class InvalidFormat(Exception):
 	pass
@@ -87,17 +117,30 @@ def parse_jid(jidstring):
 	return prep(*decompose_jid(jidstring))
 
 def idn_to_ascii(host):
-	'''convert IDN (Internationalized Domain Names) to ACE (ASCII-compatible encoding)'''
+	'''convert IDN (Internationalized Domain Names) to ACE
+	(ASCII-compatible encoding)'''
+	from encodings import idna
 	labels = idna.dots.split(host)
 	converted_labels = []
 	for label in labels:
 		converted_labels.append(idna.ToASCII(label))
 	return ".".join(converted_labels)
 
+def ascii_to_idn(host):
+	'''convert ACE (ASCII-compatible encoding) to IDN
+	(Internationalized Domain Names)'''
+	from encodings import idna
+	labels = idna.dots.split(host)
+	converted_labels = []
+	for label in labels:
+		converted_labels.append(idna.ToUnicode(label))
+	return ".".join(converted_labels)
+
 def parse_resource(resource):
 	'''Perform stringprep on resource and return it'''
 	if resource:
 		try:
+			from xmpp_stringprep import resourceprep
 			return resourceprep.prepare(unicode(resource))
 		except UnicodeError:
 			raise InvalidFormat, 'Invalid character in resource.'
@@ -109,6 +152,7 @@ def prep(user, server, resource):
 
 	if user:
 		try:
+			from xmpp_stringprep import nodeprep
 			user = nodeprep.prepare(unicode(user))
 		except UnicodeError:
 			raise InvalidFormat, _('Invalid character in username.')
@@ -119,12 +163,14 @@ def prep(user, server, resource):
 		raise InvalidFormat, _('Server address required.')
 	else:
 		try:
+			from xmpp_stringprep import nameprep
 			server = nameprep.prepare(unicode(server))
 		except UnicodeError:
 			raise InvalidFormat, _('Invalid character in hostname.')
 
 	if resource:
 		try:
+			from xmpp_stringprep import resourceprep
 			resource = resourceprep.prepare(unicode(resource))
 		except UnicodeError:
 			raise InvalidFormat, _('Invalid character in resource.')
@@ -180,8 +226,8 @@ def convert_bytes(string):
 					suffix = _('%s MB')
 		else:
 			if use_kib_mib:
-					#KiB means kibibyte
-					suffix = _('%s KiB')
+				#KiB means kibibyte
+				suffix = _('%s KiB')
 			else:
 				#KB means kilo bytes
 				suffix = _('%s KB')
@@ -200,7 +246,7 @@ def get_contact_dict_for_account(account):
 				jid)
 		contacts_dict[jid] = contact
 		name = contact.name
-		if contacts_dict.has_key(name):
+		if name in contacts_dict:
 			contact1 = contacts_dict[name]
 			del contacts_dict[name]
 			contacts_dict['%s (%s)' % (name, contact1.jid)] = contact1
@@ -237,7 +283,7 @@ def get_uf_show(show, use_mnemonic = False):
 		else:
 			uf_show = _('Available')
 	elif show == 'connecting':
-			uf_show = _('Connecting')
+		uf_show = _('Connecting')
 	elif show == 'away':
 		if use_mnemonic:
 			uf_show = _('A_way')
@@ -320,10 +366,8 @@ def get_uf_affiliation(affiliation):
 		affiliation_name = affiliation.capitalize()
 	return affiliation_name
 
-
 def get_sorted_keys(adict):
-	keys = adict.keys()
-	keys.sort()
+	keys = sorted(adict.keys())
 	return keys
 
 def to_one_line(msg):
@@ -372,7 +416,7 @@ def is_in_path(name_of_command, return_abs_path = False):
 	# on failures False is returned
 	is_in_dir = False
 	found_in_which_dir = None
-	path = os.getenv('PATH').split(':')
+	path = os.getenv('PATH').split(os.pathsep)
 	for path_to_directory in path:
 		try:
 			contents = os.listdir(path_to_directory)
@@ -406,7 +450,7 @@ def launch_browser_mailer(kind, uri):
 	if os.name == 'nt':
 		try:
 			os.startfile(uri) # if pywin32 is installed we open
-		except:
+		except Exception:
 			pass
 
 	else:
@@ -419,6 +463,9 @@ def launch_browser_mailer(kind, uri):
 			command = 'kfmclient exec'
 		elif gajim.config.get('openwith') == 'exo-open':
 			command = 'exo-open'
+		elif ((sys.platform == 'darwin') and\
+		(gajim.config.get('openwith') == 'open')):
+			command = 'open'
 		elif gajim.config.get('openwith') == 'custom':
 			if kind == 'url':
 				command = gajim.config.get('custombrowser')
@@ -430,14 +477,14 @@ def launch_browser_mailer(kind, uri):
 		command = build_command(command, uri)
 		try:
 			exec_command(command)
-		except:
+		except Exception:
 			pass
 
 def launch_file_manager(path_to_open):
 	if os.name == 'nt':
 		try:
 			os.startfile(path_to_open) # if pywin32 is installed we open
-		except:
+		except Exception:
 			pass
 	else:
 		if gajim.config.get('openwith') == 'gnome-open':
@@ -446,6 +493,9 @@ def launch_file_manager(path_to_open):
 			command = 'kfmclient exec'
 		elif gajim.config.get('openwith') == 'exo-open':
 			command = 'exo-open'
+		elif ((sys.platform == 'darwin') and\
+		(gajim.config.get('openwith') == 'open')):
+			command = 'open'
 		elif gajim.config.get('openwith') == 'custom':
 			command = gajim.config.get('custom_file_manager')
 		if command == '': # if no app is configured
@@ -453,7 +503,7 @@ def launch_file_manager(path_to_open):
 		command = build_command(command, path_to_open)
 		try:
 			exec_command(command)
-		except:
+		except Exception:
 			pass
 
 def play_sound(event):
@@ -468,11 +518,16 @@ def play_sound_file(path_to_soundfile):
 		return
 	if path_to_soundfile is None or not os.path.exists(path_to_soundfile):
 		return
-	if os.name == 'nt':
+	if sys.platform == 'darwin':
+		try:
+			nsapp.playFile(path_to_soundfile)
+		except NameError:
+			pass
+	elif os.name == 'nt':
 		try:
 			winsound.PlaySound(path_to_soundfile,
 				winsound.SND_FILENAME|winsound.SND_ASYNC)
-		except:
+		except Exception:
 			pass
 	elif os.name == 'posix':
 		if gajim.config.get('soundplayer') == '':
@@ -482,13 +537,17 @@ def play_sound_file(path_to_soundfile):
 		exec_command(command)
 
 def get_file_path_from_dnd_dropped_uri(uri):
-	path = urllib.url2pathname(uri) # escape special chars
+	path = urllib.unquote(uri) # escape special chars
 	path = path.strip('\r\n\x00') # remove \r\n and NULL
 	# get the path to file
-	if path.startswith('file:\\\\\\'): # windows
+	if re.match('^file:///[a-zA-Z]:/', path): # windows
 		path = path[8:] # 8 is len('file:///')
 	elif path.startswith('file://'): # nautilus, rox
-		path = path[7:] # 7 is len('file://')
+		if sys.platform == 'darwin':
+			# OS/X includes hostname in file:// URI
+			path = re.sub('file://[^/]*', '', path)
+		else:
+			path = path[7:] # 7 is len('file://')
 	elif path.startswith('file:'): # xffm
 		path = path[5:] # 5 is len('file:')
 	return path
@@ -543,13 +602,35 @@ def get_global_status():
 			status = gajim.connections[account].status
 	return status
 
+def statuses_unified(): 
+	'''testing if all statuses are the same.'''
+	reference = None
+	for account in gajim.connections:
+		if not gajim.config.get_per('accounts', account,
+		'sync_with_global_status'):
+			continue
+		if reference is None:
+			reference = gajim.connections[account].connected
+		elif reference != gajim.connections[account].connected:
+			return False
+	return True
+
 def get_icon_name_to_show(contact, account = None):
 	'''Get the icon name to show in online, away, requested, ...'''
 	if account and gajim.events.get_nb_roster_events(account, contact.jid):
-		return 'message'
+		return 'event'
 	if account and gajim.events.get_nb_roster_events(account,
 	contact.get_full_jid()):
-		return 'message'
+		return 'event'
+	if account and account in gajim.interface.minimized_controls and \
+	contact.jid in gajim.interface.minimized_controls[account] and gajim.interface.\
+		minimized_controls[account][contact.jid].get_nb_unread_pm() > 0:
+		return 'event'
+	if account and contact.jid in gajim.gc_connected[account]:
+		if gajim.gc_connected[account][contact.jid]:
+			return 'muc_active'
+		else:
+			return 'muc_inactive'
 	if contact.jid.find('@') <= 0: # if not '@' or '@' starts the jid ==> agent
 		return contact.show
 	if contact.sub in ('both', 'to'):
@@ -559,6 +640,8 @@ def get_icon_name_to_show(contact, account = None):
 	transport = gajim.get_transport_name_from_jid(contact.jid)
 	if transport:
 		return contact.show
+	if contact.show in gajim.SHOW_LIST:
+		return contact.show
 	return 'not in roster'
 
 def decode_string(string):
@@ -580,10 +663,15 @@ def ensure_utf8_string(string):
 	'''make sure string is in UTF-8'''
 	try:
 		string = decode_string(string).encode('utf-8')
-	except:
+	except Exception:
 		pass
 	return string
 
+def remove_invalid_xml_chars(string):
+	if string:
+		string = re.sub(gajim.interface.invalid_XML_chars_re, '', string)
+	return string
+
 def get_windows_reg_env(varname, default=''):
 	'''asks for paths commonly used but not exposed as ENVs
 	in english Windows 2003 those are:
@@ -617,7 +705,7 @@ r'Software\Microsoft\Windows\CurrentVersion\Explorer\User Shell Folders')
 		try:
 			val = str(win32api.RegQueryValueEx(rkey, varname)[0])
 			val = win32api.ExpandEnvironmentStrings(val) # expand using environ
-		except:
+		except Exception:
 			pass
 	finally:
 		win32api.RegCloseKey(rkey)
@@ -685,8 +773,7 @@ def get_random_string_16():
 	rng.extend(range(48, 57))
 	char_sequence = map(lambda e:chr(e), rng)
 	from random import sample
-	return reduce(lambda e1, e2: e1 + e2, 
-			sample(char_sequence, 16))
+	return ''.join(sample(char_sequence, 16))
 	
 def get_os_info():
 	if os.name == 'nt':
@@ -702,21 +789,20 @@ def get_os_info():
 			(2, 5, 2): '2003',
 			(2, 6, 0): 'Vista',
 		}
-		if win_version.has_key(ver_format):
+		if ver_format in win_version:
 			return 'Windows' + ' ' + win_version[ver_format]
 		else:
 			return 'Windows'
 	elif os.name == 'posix':
 		executable = 'lsb_release'
-		params = ' --id --codename --release --short'
+		params = ' --description --codename --release --short'
 		full_path_to_executable = is_in_path(executable, return_abs_path = True)
 		if full_path_to_executable:
 			command = executable + params
-			child_stdin, child_stdout = os.popen2(command)
-			output = temp_failure_retry(child_stdout.readline).strip()
-			child_stdout.close()
-			child_stdin.close()
-			os.wait()
+			p = subprocess.Popen([command], shell=True, stdin=subprocess.PIPE, 
+				stdout=subprocess.PIPE, close_fds=True) 
+			p.wait() 
+			output = temp_failure_retry(p.stdout.readline).strip()
 			# some distros put n/a in places, so remove those
 			output = output.replace('n/a', '').replace('N/A', '')
 			return output
@@ -740,36 +826,38 @@ def get_os_info():
 						'sourcemage') or not\
 						os.path.basename(path_to_file).startswith('slackware'):
 							text = distro_name + ' ' + text
-					elif path_to_file.endswith('aurox-release'):
+					elif path_to_file.endswith('aurox-release') or \
+					path_to_file.endswith('arch-release'):
 						# file doesn't have version
 						text = distro_name
 					elif path_to_file.endswith('lfs-release'): # file just has version
 						text = distro_name + ' ' + text
-				return text
+				return text.replace('\n', '')
 
 		# our last chance, ask uname and strip it
-		uname_output = get_output_of_command('uname -a | cut -d" " -f1,3')
+		uname_output = get_output_of_command('uname -sr')
 		if uname_output is not None:
 			return uname_output[0] # only first line
 	return 'N/A'
 
 def sanitize_filename(filename):
-	'''makes sure the filename we will write does contain only
-	acceptable and latin characters'''
+	'''makes sure the filename we will write does contain only acceptable and 
+	latin characters, and is not too long (in that case hash it)'''
+	# 48 is the limit
+	if len(filename) > 48:
+		hash = hash_md5(filename)
+		filename = base64.b64encode(hash.digest())
+
 	filename = punycode_encode(filename) # make it latin chars only
 	filename = filename.replace('/', '_')
 	if os.name == 'nt':
 		filename = filename.replace('?', '_').replace(':', '_')\
 			.replace('\\', '_').replace('"', "'").replace('|', '_')\
 			.replace('*', '_').replace('<', '_').replace('>', '_')
-		# 48 is the limit
-		if len(filename) > 48:
-			extension = filename.split('.')[-1]
-			filename = filename[0:48]
 	
 	return filename
 
-def allow_showing_notification(account, type = 'notify_on_new_message',
+def allow_showing_notification(account, type_ = 'notify_on_new_message',
 advanced_notif_num = None, is_first_message = True):
 	'''is it allowed to show nofication?
 	check OUR status and if we allow notifications for that status
@@ -782,7 +870,7 @@ advanced_notif_num = None, is_first_message = True):
 			return True
 		if popup == 'no':
 			return False
-	if type and (not gajim.config.get(type) or not is_first_message):
+	if type_ and (not gajim.config.get(type_) or not is_first_message):
 		return False
 	if gajim.config.get('autopopupaway'): # always show notification
 		return True
@@ -792,7 +880,7 @@ advanced_notif_num = None, is_first_message = True):
 
 def allow_popup_window(account, advanced_notif_num = None):
 	'''is it allowed to popup windows?'''
-	if advanced_notif_num != None:
+	if advanced_notif_num is not None:
 		popup = gajim.config.get_per('notifications', str(advanced_notif_num),
 			'auto_open')
 		if popup == 'yes':
@@ -824,19 +912,20 @@ def get_chat_control(account, contact):
 		full_jid_with_resource += '/' + contact.resource
 	highest_contact = gajim.contacts.get_contact_with_highest_priority(
 		account, contact.jid)
+
 	# Look for a chat control that has the given resource, or default to
 	# one without resource
 	ctrl = gajim.interface.msg_win_mgr.get_control(full_jid_with_resource,
 		account)
+
 	if ctrl:
 		return ctrl
-	elif not highest_contact or not highest_contact.resource:
-		# unknow contact or offline message
-		return gajim.interface.msg_win_mgr.get_control(contact.jid, account)
-	elif highest_contact and contact.resource != \
-	highest_contact.resource:
+	elif highest_contact and highest_contact.resource and \
+	contact.resource != highest_contact.resource:
 		return None
-	return gajim.interface.msg_win_mgr.get_control(contact.jid, account)
+	else:
+		# unknown contact or offline message
+		return gajim.interface.msg_win_mgr.get_control(contact.jid, account)
 
 def reduce_chars_newlines(text, max_chars = 0, max_lines = 0):
 	'''Cut the chars after 'max_chars' on each line
@@ -860,76 +949,134 @@ def reduce_chars_newlines(text, max_chars = 0, max_lines = 0):
 		if lines:
 			lines = map(lambda e: _cut_if_long(e), lines)
 	if lines:
-		reduced_text = reduce(lambda e, e1: e + '\n' + e1, lines)
+		reduced_text = '\n'.join(lines)
 		if reduced_text != text:
 			reduced_text += '...'
 	else:
 		reduced_text = ''
 	return reduced_text
 
-def get_notification_icon_tooltip_text():
-	text = None
-	unread_chat = gajim.events.get_nb_events(types = ['printed_chat',
-		'chat'])
-	unread_single_chat = gajim.events.get_nb_events(types = ['normal'])
-	unread_gc = gajim.events.get_nb_events(types = ['printed_gc_msg',
-		'gc_msg'])
-	unread_pm = gajim.events.get_nb_events(types = ['printed_pm', 'pm'])
+def get_account_status(account):
+	status = reduce_chars_newlines(account['status_line'], 100, 1)
+	return status
+
+def get_notification_icon_tooltip_dict():
+	'''returns a dict of the form {acct: {'show': show, 'message': message, 
+	'event_lines': [list of text lines to show in tooltip]}'''
+	# How many events must there be before they're shown summarized, not per-user
+	max_ungrouped_events = 10
 
 	accounts = get_accounts_info()
 
-	if unread_chat or unread_single_chat or unread_gc or unread_pm:
-		text = 'Gajim '
-		awaiting_events = unread_chat + unread_single_chat + unread_gc + unread_pm
-		if awaiting_events == unread_chat or awaiting_events == unread_single_chat \
-			or awaiting_events == unread_gc or awaiting_events == unread_pm:
-			# This condition is like previous if but with xor... 
-			# Print in one line
-			text += '-'
-		else:
-			# Print in multiple lines
-			text += '\n   '
-		if unread_chat:
-			text += ngettext(
-				' %d unread message',
-				' %d unread messages',
-				unread_chat, unread_chat, unread_chat)
-			text += '\n   '
-		if unread_single_chat:
-			text += ngettext(
-				' %d unread single message',
-				' %d unread single messages',
-				unread_single_chat, unread_single_chat, unread_single_chat)
-			text += '\n   '
-		if unread_gc:
-			text += ngettext(
-				' %d unread group chat message',
-				' %d unread group chat messages',
-				unread_gc, unread_gc, unread_gc)
-			text += '\n   '
-		if unread_pm:
-			text += ngettext(
-				' %d unread private message',
-				' %d unread private messages',
-				unread_pm, unread_pm, unread_pm)
-			text += '\n   '
-		text = text[:-4] # remove latest '\n   '
-	elif len(accounts) > 1:
-		text = _('Gajim')
-	elif len(accounts) == 1:
-		message = accounts[0]['status_line']
-		message = reduce_chars_newlines(message, 100, 1)
-		text = _('Gajim - %s') % message
-	else:
-		text = _('Gajim - %s') % get_uf_show('offline')
+	# Gather events. (With accounts, when there are more.)
+	for account in accounts:
+		account_name = account['name']
+		account['event_lines'] = []
+		# Gather events per-account
+		pending_events = gajim.events.get_events(account = account_name)
+		messages, non_messages, total_messages, total_non_messages = {}, {}, 0, 0
+		for jid in pending_events:
+			for event in pending_events[jid]:
+				if event.type_.count('file') > 0:
+					# This is a non-messagee event.
+					messages[jid] = non_messages.get(jid, 0) + 1
+					total_non_messages = total_non_messages + 1
+				else:
+					# This is a message.
+					messages[jid] = messages.get(jid, 0) + 1
+					total_messages = total_messages + 1
+		# Display unread messages numbers, if any
+		if total_messages > 0:
+			if total_messages > max_ungrouped_events:
+				text = ngettext(
+					'%d message pending',
+					'%d messages pending',
+					total_messages, total_messages, total_messages)
+				account['event_lines'].append(text)
+			else:
+				for jid in messages.keys():
+					text = ngettext(
+						'%d message pending',
+						'%d messages pending',
+						messages[jid], messages[jid], messages[jid])
+					contact = gajim.contacts.get_first_contact_from_jid(
+						account['name'], jid)
+					if jid in gajim.gc_connected[account['name']]:
+						text += _(' from room %s') % (jid)
+					elif contact:
+						name = contact.get_shown_name()
+						text += _(' from user %s') % (name)
+					else:
+						text += _(' from %s') % (jid)
+					account['event_lines'].append(text)
 		
+		# Display unseen events numbers, if any
+		if total_non_messages > 0:
+			if total_non_messages > max_ungrouped_events:
+				text = ngettext(
+					'%d event pending',
+					'%d events pending',
+					total_non_messages, total_non_messages, total_non_messages)
+				accounts[account]['event_lines'].append(text)
+			else:
+				for jid in non_messages.keys():
+					text = ngettext(
+						'%d event pending',
+						'%d events pending',
+						non_messages[jid], non_messages[jid], non_messages[jid])
+					text += _(' from user %s') % (jid)
+					accounts[account]['event_lines'].append(text)
+
+	return accounts
+
+def get_notification_icon_tooltip_text():
+	text = None
+	# How many events must there be before they're shown summarized, not per-user
+	max_ungrouped_events = 10
+	# Character which should be used to indent in the tooltip.
+	indent_with = ' '
+
+	accounts = get_notification_icon_tooltip_dict()
+
+	if len(accounts) == 0:
+		# No configured account
+		return _('Gajim')
+
+	# at least one account present
+
+	# Is there more that one account?
+	if len(accounts) == 1:
+		show_more_accounts = False
+	else:
+		show_more_accounts = True
+
+	# If there is only one account, its status is shown on the first line.
+	if show_more_accounts:
+		text = _('Gajim')
+	else:		
+		text = _('Gajim - %s') % (get_account_status(accounts[0]))
+
+	# Gather and display events. (With accounts, when there are more.)
+	for account in accounts:
+		account_name = account['name']
+		# Set account status, if not set above
+		if (show_more_accounts):
+			message = '\n' + indent_with + ' %s - %s'
+			text += message % (account_name, get_account_status(account))
+			# Account list shown, messages need to be indented more
+			indent_how = 2
+		else:
+			# If no account list is shown, messages could have default indenting.
+			indent_how = 1
+		for line in account['event_lines']:
+			text += '\n' + indent_with * indent_how + ' '
+			text += line
 	return text
 
 def get_accounts_info():
 	'''helper for notification icon tooltip'''
 	accounts = []
-	accounts_list = gajim.contacts.get_accounts()
-	accounts_list.sort()
+	accounts_list = sorted(gajim.contacts.get_accounts())
 	for account in accounts_list:
 		status_idx = gajim.connections[account].connected
 		# uncomment the following to hide offline accounts
@@ -947,14 +1094,200 @@ def get_accounts_info():
 				'show': status, 'message': message})
 	return accounts
 
+def get_avatar_path(prefix):
+	'''Returns the filename of the avatar, distinguishes between user- and
+	contact-provided one.  Returns None if no avatar was found at all.
+	prefix is the path to the requested avatar just before the ".png" or
+	".jpeg".'''
+	# First, scan for a local, user-set avatar
+	for type_ in ('jpeg', 'png'):
+		file_ = prefix + '_local.' + type_
+		if os.path.exists(file_):
+			return file_
+	# If none available, scan for a contact-provided avatar
+	for type_ in ('jpeg', 'png'):
+		file_ = prefix + '.' + type_
+		if os.path.exists(file_):
+			return file_
+	return None
+
 def datetime_tuple(timestamp):
 	'''Converts timestamp using strptime and the format: %Y%m%dT%H:%M:%S
 	Because of various datetime formats are used the following exceptions
 	are handled:
 		- Optional milliseconds appened to the string are removed
+		- Optional Z (that means UTC) appened to the string are removed
 		- XEP-082 datetime strings have all '-' cahrs removed to meet
 		  the above format.'''
 	timestamp = timestamp.split('.')[0]
 	timestamp = timestamp.replace('-', '')
+	timestamp = timestamp.replace('z', '')
+	timestamp = timestamp.replace('Z', '')
 	from time import strptime
 	return strptime(timestamp, '%Y%m%dT%H:%M:%S')
+
+def get_iconset_path(iconset):
+	if os.path.isdir(os.path.join(gajim.DATA_DIR, 'iconsets', iconset)):
+		return os.path.join(gajim.DATA_DIR, 'iconsets', iconset)
+	elif os.path.isdir(os.path.join(gajim.MY_ICONSETS_PATH, iconset)):
+		return os.path.join(gajim.MY_ICONSETS_PATH, iconset)
+
+def get_mood_iconset_path(iconset):
+	if os.path.isdir(os.path.join(gajim.DATA_DIR, 'moods', iconset)):
+		return os.path.join(gajim.DATA_DIR, 'moods', iconset)
+	elif os.path.isdir(os.path.join(gajim.MY_MOOD_ICONSETS_PATH, iconset)):
+		return os.path.join(gajim.MY_MOOD_ICONSETS_PATH, iconset)
+
+def get_activity_iconset_path(iconset):
+	if os.path.isdir(os.path.join(gajim.DATA_DIR, 'activities', iconset)):
+		return os.path.join(gajim.DATA_DIR, 'activities', iconset)
+	elif os.path.isdir(os.path.join(gajim.MY_ACTIVITY_ICONSETS_PATH,
+	iconset)):
+		return os.path.join(gajim.MY_ACTIVITY_ICONSETS_PATH, iconset)
+
+def get_transport_path(transport):
+	if os.path.isdir(os.path.join(gajim.DATA_DIR, 'iconsets', 'transports',
+	transport)):
+		return os.path.join(gajim.DATA_DIR, 'iconsets', 'transports', transport)
+	elif os.path.isdir(os.path.join(gajim.MY_ICONSETS_PATH, 'transports',
+	transport)):
+		return os.path.join(gajim.MY_ICONSETS_PATH, 'transports', transport)
+	# No transport folder found, use default jabber one
+	return get_iconset_path(gajim.config.get('iconset'))
+
+def prepare_and_validate_gpg_keyID(account, jid, keyID):
+	'''Returns an eight char long keyID that can be used with for GPG encryption with this contact.
+	If the given keyID is None, return UNKNOWN; if the key does not match the assigned key
+	XXXXXXXXMISMATCH is returned. If the key is trusted and not yet assigned, assign it'''
+	if gajim.connections[account].USE_GPG:	
+		if keyID and len(keyID) == 16:
+			keyID = keyID[8:]
+		
+		attached_keys = gajim.config.get_per('accounts', account,
+			'attached_gpg_keys').split()
+		
+		if jid in attached_keys and keyID:
+			attachedkeyID = attached_keys[attached_keys.index(jid) + 1]
+			if attachedkeyID != keyID:
+				# Mismatch! Another gpg key was expected
+				keyID += 'MISMATCH'
+		elif jid in attached_keys:
+			# An unsigned presence, just use the assigned key
+			keyID = attached_keys[attached_keys.index(jid) + 1]
+		elif keyID: 
+			public_keys = gajim.connections[account].ask_gpg_keys()
+			# Assign the corresponding key, if we have it in our keyring
+			if keyID in public_keys:
+				for u in gajim.contacts.get_contacts(account, jid):
+					u.keyID = keyID
+				keys_str = gajim.config.get_per('accounts', account, 'attached_gpg_keys')
+				keys_str += jid + ' ' + keyID + ' '
+				gajim.config.set_per('accounts', account, 'attached_gpg_keys', keys_str)
+		elif keyID is None:
+			keyID = 'UNKNOWN'
+	return keyID
+
+def sort_identities_func(i1, i2):
+	cat1 = i1['category']
+	cat2 = i2['category']
+	if cat1 < cat2:
+		return -1
+	if cat1 > cat2:
+		return 1
+	type1 = i1.get('type', '')
+	type2 = i2.get('type', '')
+	if type1 < type2:
+		return -1
+	if type1 > type2:
+		return 1
+	lang1 = i1.get('xml:lang', '')
+	lang2 = i2.get('xml:lang', '')
+	if lang1 < lang2:
+		return -1
+	if lang1 > lang2:
+		return 1
+	return 0
+
+def sort_dataforms_func(d1, d2):
+	f1 = d1.getField('FORM_TYPE')
+	f2 = d2.getField('FORM_TYPE')
+	if f1 and f2 and (f1.getValue() < f2.getValue()):
+		return -1
+	return 1
+
+def compute_caps_hash(identities, features, dataforms=[], hash_method='sha-1'):
+	'''Compute caps hash according to XEP-0115, V1.5
+	
+	dataforms are xmpp.DataForms objects as common.dataforms don't allow several
+	values without a field type list-multi'''
+	S = ''
+	identities.sort(cmp=sort_identities_func)
+	for i in identities:
+		c = i['category']
+		type_ = i.get('type', '')
+		lang = i.get('xml:lang', '')
+		name = i.get('name', '')
+		S += '%s/%s/%s/%s<' % (c, type_, lang, name)
+	features.sort()
+	for f in features:
+		S += '%s<' % f
+	dataforms.sort(cmp=sort_dataforms_func)
+	for dataform in dataforms:
+		# fields indexed by var
+		fields = {}
+		for f in dataform.getChildren():
+			fields[f.getVar()] = f
+		form_type = fields.get('FORM_TYPE')
+		if form_type:
+			S += form_type.getValue() + '<'
+			del fields['FORM_TYPE']
+		vars = sorted(fields.keys())
+		for var in vars:
+			S += '%s<' % var
+			values = sorted(fields[var].getValues())
+			for value in values:
+				S += '%s<' % value
+
+	if hash_method == 'sha-1':
+		hash = hash_sha1(S)
+	elif hash_method == 'md5':
+		hash = hash_md5(S)
+	else:
+		return ''
+	return base64.b64encode(hash.digest())
+
+def update_optional_features(account = None):
+	if account:
+		accounts = [account]
+	else:
+		accounts = [a for a in gajim.connections]
+	for a in accounts:
+		gajim.gajim_optional_features[a] = []
+		if gajim.config.get_per('accounts', a, 'subscribe_mood'):
+			gajim.gajim_optional_features[a].append(xmpp.NS_MOOD + '+notify')
+		if gajim.config.get_per('accounts', a, 'subscribe_activity'):
+			gajim.gajim_optional_features[a].append(xmpp.NS_ACTIVITY + '+notify')
+		if gajim.config.get_per('accounts', a, 'publish_tune'):
+			gajim.gajim_optional_features[a].append(xmpp.NS_TUNE)
+		if gajim.config.get_per('accounts', a, 'subscribe_tune'):
+			gajim.gajim_optional_features[a].append(xmpp.NS_TUNE + '+notify')
+		if gajim.config.get_per('accounts', a, 'subscribe_nick'):
+			gajim.gajim_optional_features[a].append(xmpp.NS_NICK + '+notify')
+		if gajim.config.get('outgoing_chat_state_notifactions') != 'disabled':
+			gajim.gajim_optional_features[a].append(xmpp.NS_CHATSTATES)
+		if not gajim.config.get('ignore_incoming_xhtml'):
+			gajim.gajim_optional_features[a].append(xmpp.NS_XHTML_IM)
+		if gajim.HAVE_PYCRYPTO \
+		and gajim.config.get_per('accounts', a, 'enable_esessions'):
+			gajim.gajim_optional_features[a].append(xmpp.NS_ESESSION)
+		if gajim.config.get_per('accounts', a, 'answer_receipts'):
+			gajim.gajim_optional_features[a].append(xmpp.NS_RECEIPTS)
+		gajim.caps_hash[a] = compute_caps_hash([gajim.gajim_identity],
+			gajim.gajim_common_features + gajim.gajim_optional_features[a])
+		# re-send presence with new hash
+		connected = gajim.connections[a].connected
+		if connected > 1 and gajim.SHOW_LIST[connected] != 'invisible':
+			gajim.connections[a].change_status(gajim.SHOW_LIST[connected],
+				gajim.connections[a].status)
+
+# vim: se ts=3:
