@@ -1,26 +1,28 @@
-## common/sleepy.py
+# -*- coding:utf-8 -*-
+## src/common/sleepy.py
 ##
-## Contributors for this file:
-##	  - Yann Le Boulanger <asterix@lagaule.org>
-##	  - Nikos Kouremenos <kourem@gmail.com>
+## Copyright (C) 2003-2007 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2007 Jean-Marie Traissard <jim AT lapin.org>
+## Copyright (C) 2008 Mateusz Biliński <mateusz AT bilinski.it>
 ##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##						 Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005-2006 Yann Le Boulanger <asterix@lagaule.org>
-##					Nikos Kouremenos <kourem@gmail.com>
+## This file is part of Gajim.
 ##
-## This program is free software; you can redistribute it and/or modify
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 from common import gajim
-import os
+import os, sys
 
 
 STATE_UNKNOWN  = 'OS probably not supported'
@@ -30,21 +32,23 @@ STATE_AWAKE	= 'awake'
 
 SUPPORTED = True
 try:
-		if os.name == 'nt':
-			import ctypes
+	if os.name == 'nt':
+		import ctypes
 
-			GetTickCount = ctypes.windll.kernel32.GetTickCount
-			GetLastInputInfo = ctypes.windll.user32.GetLastInputInfo
+		GetTickCount = ctypes.windll.kernel32.GetTickCount
+		GetLastInputInfo = ctypes.windll.user32.GetLastInputInfo
 
-			class LASTINPUTINFO(ctypes.Structure):
-				_fields_ = [('cbSize', ctypes.c_uint), ('dwTime', ctypes.c_uint)]
+		class LASTINPUTINFO(ctypes.Structure):
+			_fields_ = [('cbSize', ctypes.c_uint), ('dwTime', ctypes.c_uint)]
 
-			lastInputInfo = LASTINPUTINFO()
-			lastInputInfo.cbSize = ctypes.sizeof(lastInputInfo)
+		lastInputInfo = LASTINPUTINFO()
+		lastInputInfo.cbSize = ctypes.sizeof(lastInputInfo)
 
-		else: # unix
-			import idle
-except:
+	elif sys.platform == 'darwin':
+		import osx.idle as idle
+	else: # unix
+		import idle
+except Exception:
 	gajim.log.debug('Unable to load idle module')
 	SUPPORTED = False
 
@@ -88,7 +92,7 @@ class SleepyUnix:
 		self.state = STATE_AWAKE # assume we are awake
 		try:
 			idle.init()
-		except:
+		except Exception:
 			SUPPORTED = False
 			self.state = STATE_UNKNOWN
 
@@ -121,3 +125,5 @@ if os.name == 'nt':
 	Sleepy = SleepyWindows
 else:
 	Sleepy = SleepyUnix
+
+# vim: se ts=3:
