@@ -1,35 +1,32 @@
-
-##	common/xmpp/socks5.py
+# -*- coding:utf-8 -*-
+## src/common/socks5.py
 ##
-## Contributors for this file:
-##	- Yann Le Boulanger <asterix@lagaule.org>
-##	- Nikos Kouremenos <kourem@gmail.com>
-##	- Dimitur Kirov <dkirov@gmail.com>
+## Copyright (C) 2005-2006 Dimitur Kirov <dkirov AT gmail.com>
+##                         Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2005-2007 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+## Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
 ##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <kourem@gmail.com>
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
+## This file is part of Gajim.
 ##
-## This program is free software; you can redistribute it and/or modify
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 
 import socket
 import struct
 import sha
-from dialogs import BindPortError
+import os
 
 from errno import EWOULDBLOCK
 from errno import ENOBUFS
@@ -55,7 +52,8 @@ SEND_TIMEOUT = 180
 
 class SocksQueue:
 	''' queue for all file requests objects '''
-	def __init__(self, idlequeue, complete_transfer_cb = None, progress_transfer_cb = None):
+	def __init__(self, idlequeue, complete_transfer_cb=None,
+	progress_transfer_cb=None, error_cb=None):
 		self.connected = 0
 		self.readers = {}
 		self.files_props = {}
@@ -68,37 +66,39 @@ class SocksQueue:
 		self.idlequeue = idlequeue
 		self.complete_transfer_cb = complete_transfer_cb
 		self.progress_transfer_cb = progress_transfer_cb
+		self.error_cb = error_cb
 		self.on_success = None
 		self.on_failure = None
-	
+
 	def start_listener(self, port, sha_str, sha_handler, sid):
 		''' start waiting for incomming connections on (host, port)
 		and do a socks5 authentication using sid for generated sha
 		'''
 		self.sha_handlers[sha_str] = (sha_handler, sid)
-		if self.listener == None:
+		if self.listener is None:
 			self.listener = Socks5Listener(self.idlequeue, port)
 			self.listener.queue = self
 			self.listener.bind()
 			if self.listener.started is False:
 				self.listener = None
-				# We cannot bind port, call error 
-				# dialog from dialogs.py and fail
-				BindPortError(port)
+				# We cannot bind port, call error callback and fail
+				self.error_cb(_('Unable to bind to port %s.') % port,
+					 _('Maybe you have another running instance of Gajim. File '
+					 'Transfer will be cancelled.'))
 				return None
 			self.connected += 1
 		return self.listener
-	
+
 	def send_success_reply(self, file_props, streamhost):
-		if file_props.has_key('streamhost-used') and \
-			file_props['streamhost-used'] is True:
-				if file_props.has_key('proxyhosts'):
-					for proxy in file_props['proxyhosts']:
-						if proxy == streamhost:
-							self.on_success(streamhost)
-							return 2
-				return 0
-		if file_props.has_key('streamhosts'):
+		if 'streamhost-used' in file_props and \
+		file_props['streamhost-used'] is True:
+			if 'proxyhosts' in file_props:
+				for proxy in file_props['proxyhosts']:
+					if proxy == streamhost:
+						self.on_success(streamhost)
+						return 2
+			return 0
+		if 'streamhosts' in file_props:
 			for host in file_props['streamhosts']:
 				if streamhost['state'] == 1:
 					return 0
@@ -106,39 +106,38 @@ class SocksQueue:
 			self.on_success(streamhost)
 			return 1
 		return 0
-	
-	def connect_to_hosts(self, account, sid, on_success = None, 
-		on_failure = None):
+
+	def connect_to_hosts(self, account, sid, on_success=None, on_failure=None):
 		self.on_success = on_success
 		self.on_failure = on_failure
 		file_props = self.files_props[account][sid]
 		file_props['failure_cb'] = on_failure
-		
-		# add streamhosts to the queue 
+
+		# add streamhosts to the queue
 		for streamhost in file_props['streamhosts']:
 			receiver = Socks5Receiver(self.idlequeue, streamhost, sid, file_props)
 			self.add_receiver(account, receiver)
 			streamhost['idx'] = receiver.queue_idx
-	
+
 	def _socket_connected(self, streamhost, file_props):
-		''' called when there is a host connected to one of the 
+		''' called when there is a host connected to one of the
 		senders's streamhosts. Stop othere attempts for connections '''
 		for host in file_props['streamhosts']:
-			if host != streamhost and host.has_key('idx'):
+			if host != streamhost and 'idx' in host:
 				if host['state'] == 1:
 					# remove current
 					self.remove_receiver(streamhost['idx'])
 					return
 				# set state -2, meaning that this streamhost is stopped,
 				# but it may be connectected later
-				if host['state'] >=0:
+				if host['state'] >= 0:
 					self.remove_receiver(host['idx'])
 					host['idx'] = -1
 					host['state'] = -2
-	
+
 	def reconnect_receiver(self, receiver, streamhost):
-		''' Check the state of all streamhosts and if all has failed, then 
-		emit connection failure cb. If there are some which are still 
+		''' Check the state of all streamhosts and if all has failed, then
+		emit connection failure cb. If there are some which are still
 		not connected try to establish connection to one of them.
 		'''
 		self.idlequeue.remove_timeout(receiver.fd)
@@ -148,7 +147,7 @@ class SocksQueue:
 		# boolean, indicates that there are hosts, which are not tested yet
 		unused_hosts = False
 		for host in file_props['streamhosts']:
-			if host.has_key('idx'):
+			if 'idx' in host:
 				if host['state'] >= 0:
 					return
 				elif host['state'] == -2:
@@ -157,12 +156,13 @@ class SocksQueue:
 			for host in file_props['streamhosts']:
 				if host['state'] == -2:
 					host['state'] = 0
-					receiver = Socks5Receiver(self.idlequeue, host, host['sid'], file_props)
+					receiver = Socks5Receiver(self.idlequeue, host, host['sid'],
+						file_props)
 					self.add_receiver(receiver.account, receiver)
 					host['idx'] = receiver.queue_idx
 			# we still have chances to connect
 			return
-		if not file_props.has_key('received-len') or file_props['received-len'] == 0:
+		if 'received-len' not in file_props or file_props['received-len'] == 0:
 			# there are no other streamhosts and transfer hasn't started
 			self._connection_refused(streamhost, file_props, receiver.queue_idx)
 		else:
@@ -170,23 +170,23 @@ class SocksQueue:
 			receiver.disconnect()
 			file_props['error'] = -1
 			self.process_result(-1, receiver)
-	
+
 	def _connection_refused(self, streamhost, file_props, idx):
 		''' cb, called when we loose connection during transfer'''
 		if file_props is None:
 			return
 		streamhost['state'] = -1
 		self.remove_receiver(idx, False)
-		if file_props.has_key('streamhosts'):
+		if 'streamhosts' in file_props:
 			for host in file_props['streamhosts']:
 				if host['state'] != -1:
 					return
 		# failure_cb exists - this means that it has never been called
-		if file_props.has_key('failure_cb') and file_props['failure_cb']:
-			file_props['failure_cb'](streamhost['initiator'], streamhost['id'], 
+		if 'failure_cb' in file_props and file_props['failure_cb']:
+			file_props['failure_cb'](streamhost['initiator'], streamhost['id'],
 				file_props['sid'], code = 404)
 			del(file_props['failure_cb'])
-	
+
 	def add_receiver(self, account, sock5_receiver):
 		''' add new file request '''
 		self.readers[self.idx] = sock5_receiver
@@ -196,30 +196,28 @@ class SocksQueue:
 		self.idx += 1
 		result = sock5_receiver.connect()
 		self.connected += 1
-		if result != None:
+		if result is not None:
 			result = sock5_receiver.main()
 			self.process_result(result, sock5_receiver)
 			return 1
 		return None
-	
+
 	def get_file_from_sender(self, file_props, account):
 		if file_props is None:
 			return
-		if file_props.has_key('hash') and \
-			self.senders.has_key(file_props['hash']):
-			
+		if 'hash' in file_props and file_props['hash'] in self.senders:
 			sender = self.senders[file_props['hash']]
 			sender.account = account
 			result = self.get_file_contents(0)
 			self.process_result(result, sender)
-	
+
 	def result_sha(self, sha_str, idx):
-		if self.sha_handlers.has_key(sha_str):
+		if sha_str in self.sha_handlers:
 			props = self.sha_handlers[sha_str]
 			props[0](props[1], idx)
-	
+
 	def activate_proxy(self, idx):
-		if not self.readers.has_key(idx):
+		if idx not in self.readers:
 			return
 		reader = self.readers[idx]
 		if reader.file_props['type'] != 's':
@@ -243,15 +241,14 @@ class SocksQueue:
 			self.idlequeue.plug_idle(reader, True, False)
 			result = reader.write_next()
 			self.process_result(result, reader)
-	
+
 	def send_file(self, file_props, account):
-		if file_props.has_key('hash') and \
-			self.senders.has_key(file_props['hash']):
+		if 'hash' in file_props and file_props['hash'] in self.senders:
 			sender = self.senders[file_props['hash']]
 			file_props['streamhost-used'] = True
 			sender.account = account
 			if file_props['type'] == 's':
-				sender.file_props = file_props 
+				sender.file_props = file_props
 				result = sender.send_file()
 				self.process_result(result, sender)
 			else:
@@ -259,43 +256,42 @@ class SocksQueue:
 				file_props['last-time'] = self.idlequeue.current_time()
 				file_props['received-len'] = 0
 				sender.file_props = file_props
-	
+
 	def add_file_props(self, account, file_props):
 		''' file_prop to the dict of current file_props.
 		It is identified by account name and sid
 		'''
-		if file_props is None or \
-			file_props.has_key('sid') is False:
+		if file_props is None or ('sid' in file_props) is False:
 			return
 		_id = file_props['sid']
-		if not self.files_props.has_key(account):
+		if account not in self.files_props:
 			self.files_props[account] = {}
 		self.files_props[account][_id] = file_props
-	
+
 	def remove_file_props(self, account, sid):
-		if self.files_props.has_key(account):
+		if account in self.files_props:
 			fl_props = self.files_props[account]
-			if fl_props.has_key(sid):
+			if sid in fl_props:
 				del(fl_props[sid])
-		
+
 		if len(self.files_props) == 0:
 			self.connected = 0
-		
+
 	def get_file_props(self, account, sid):
 		''' get fil_prop by account name and session id '''
-		if self.files_props.has_key(account):
+		if account in self.files_props:
 			fl_props = self.files_props[account]
-			if fl_props.has_key(sid):
+			if sid in fl_props:
 				return fl_props[sid]
 		return None
-	
+
 	def on_connection_accepted(self, sock):
-		sock_hash =  sock.__hash__()
-		if not self.senders.has_key(sock_hash):
-			self.senders[sock_hash] = Socks5Sender(self.idlequeue, 
-				sock_hash, self, sock[0], sock[1][0], sock[1][1])
+		sock_hash = sock.__hash__()
+		if sock_hash not in self.senders:
+			self.senders[sock_hash] = Socks5Sender(self.idlequeue, sock_hash, self,
+				sock[0], sock[1][0], sock[1][1])
 			self.connected += 1
-	
+
 	def process_result(self, result, actor):
 		''' Take appropriate actions upon the result:
 		[ 0, - 1 ] complete/end transfer
@@ -306,17 +302,17 @@ class SocksQueue:
 			return
 		if result in (0, -1) and self.complete_transfer_cb is not None:
 			account = actor.account
-			if account is None and actor.file_props.has_key('tt_account'):
+			if account is None and 'tt_account' in actor.file_props:
 				account = actor.file_props['tt_account']
 			self.complete_transfer_cb(account, actor.file_props)
 		elif self.progress_transfer_cb is not None:
 			self.progress_transfer_cb(actor.account, actor.file_props)
-	
-	def remove_receiver(self, idx, do_disconnect = True):
-		''' Remove reciver from the list and decrease 
+
+	def remove_receiver(self, idx, do_disconnect=True):
+		''' Remove reciver from the list and decrease
 		the number of active connections with 1'''
 		if idx != -1:
-			if self.readers.has_key(idx):
+			if idx in self.readers:
 				reader = self.readers[idx]
 				self.idlequeue.unplug_idle(reader.fd)
 				self.idlequeue.remove_timeout(reader.fd)
@@ -324,14 +320,14 @@ class SocksQueue:
 					reader.disconnect()
 				else:
 					if reader.streamhost is not None:
-						reader.streamhost['state'] = -1 
+						reader.streamhost['state'] = -1
 					del(self.readers[idx])
-	
-	def remove_sender(self, idx, do_disconnect = True):
-		''' Remove sender from the list of senders and decrease the 
+
+	def remove_sender(self, idx, do_disconnect=True):
+		''' Remove sender from the list of senders and decrease the
 		number of active connections with 1'''
 		if idx != -1:
-			if self.senders.has_key(idx):
+			if idx in self.senders:
 				if do_disconnect:
 					self.senders[idx].disconnect()
 					return
@@ -343,14 +339,16 @@ class SocksQueue:
 				self.listener.disconnect()
 				self.listener = None
 				self.connected -= 1
-	
+
 class Socks5:
 	def __init__(self, idlequeue, host, port, initiator, target, sid):
 		if host is not None:
 			try:
-				self.host = socket.gethostbyname(host)
+				self.host = host
+				self.ais = socket.getaddrinfo(host, port, socket.AF_UNSPEC,
+					socket.SOCK_STREAM)
 			except socket.gaierror:
-				self.host = None
+				self.ais = None
 		self.idlequeue = idlequeue
 		self.fd = -1
 		self.port = port
@@ -364,38 +362,38 @@ class Socks5:
 		self.size = 0
 		self.remaining_buff = ''
 		self.file = None
-	
+
 	def open_file_for_reading(self):
-		if self.file == None:
+		if self.file is None:
 			try:
 				self.file = open(self.file_props['file-name'],'rb')
-				if self.file_props.has_key('offset') and self.file_props['offset']:
+				if 'offset' in self.file_props and self.file_props['offset']:
 					self.size = self.file_props['offset']
 					self.file.seek(self.size)
 					self.file_props['received-len'] = self.size
 			except IOError, e:
 				self.close_file()
 				raise IOError, e
-	
+
 	def close_file(self):
 		if self.file:
 			if not self.file.closed:
 				try:
 					self.file.close()
-				except:
+				except Exception:
 					pass
 			self.file = None
-	
+
 	def get_fd(self):
 		''' Test if file is already open and return its fd,
 		or just open the file and return the fd.
 		'''
-		if self.file_props.has_key('fd'):
+		if 'fd' in self.file_props:
 			fd = self.file_props['fd']
 		else:
 			offset = 0
 			opt = 'wb'
-			if self.file_props.has_key('offset') and self.file_props['offset']:
+			if 'offset' in self.file_props and self.file_props['offset']:
 				offset = self.file_props['offset']
 				opt = 'ab'
 			fd = open(self.file_props['file-name'], opt)
@@ -404,29 +402,28 @@ class Socks5:
 			self.file_props['last-time'] = self.idlequeue.current_time()
 			self.file_props['received-len'] = offset
 		return fd
-	
+
 	def rem_fd(self, fd):
-		if self.file_props.has_key('fd'):
+		if 'fd' in self.file_props:
 			del(self.file_props['fd'])
 		try:
 			fd.close()
-		except:
+		except Exception:
 			pass
-			
-	
+
 	def receive(self):
-		''' Reads small chunks of data. 
+		''' Reads small chunks of data.
 			Calls owner's disconnected() method if appropriate.'''
 		received = ''
-		try: 
+		try:
 			add = self._recv(64)
-		except Exception, e: 
-			add=''
-		received +=add
+		except Exception, e:
+			add = ''
+		received += add
 		if len(add) == 0:
 			self.disconnect()
 		return add
-	
+
 	def send_raw(self,raw_data):
 		''' Writes raw outgoing data. '''
 		try:
@@ -434,7 +431,7 @@ class Socks5:
 		except Exception, e:
 			self.disconnect()
 		return len(raw_data)
-	
+
 	def write_next(self):
 		if self.remaining_buff != '':
 			buff = self.remaining_buff
@@ -483,12 +480,10 @@ class Socks5:
 			self.state = 8 # end connection
 			self.disconnect()
 			return -1
-	
+
 	def get_file_contents(self, timeout):
-		''' read file contents from socket and write them to file ''', \
-			self.file_props['type'], self.file_props['sid']
-		if self.file_props is None or \
-			self.file_props.has_key('file-name') is False:
+		''' read file contents from socket and write them to file '''
+		if self.file_props is None or ('file-name' in self.file_props) is False:
 			self.file_props['error'] = -2
 			return None
 		fd = None
@@ -510,7 +505,7 @@ class Socks5:
 				return 0
 		else:
 			fd = self.get_fd()
-			try: 
+			try:
 				buff = self._recv(MAX_BUFF_LEN)
 			except Exception, e:
 				buff = ''
@@ -541,15 +536,15 @@ class Socks5:
 				self.file_props['completed'] = True
 				return 0
 			# return number of read bytes. It can be used in progressbar
-		if fd != None:
+		if fd is not None:
 			self.file_props['stalled'] = False
-		if fd == None and self.file_props['stalled'] is False:
+		if fd is None and self.file_props['stalled'] is False:
 			return None
-		if self.file_props.has_key('received-len'):
+		if 'received-len' in self.file_props:
 			if self.file_props['received-len'] != 0:
 				return self.file_props['received-len']
 		return None
-	
+
 	def disconnect(self):
 		''' Closes open descriptors and remover socket descr. from idleque '''
 		# be sure that we don't leave open file
@@ -559,18 +554,18 @@ class Socks5:
 		try:
 			self._sock.shutdown(socket.SHUT_RDWR)
 			self._sock.close()
-		except:
+		except Exception:
 			# socket is already closed
 			pass
 		self.connected = False
 		self.fd = -1
 		self.state = -1
-	
+
 	def _get_auth_buff(self):
-		''' Message, that we support 1 one auth mechanism: 
+		''' Message, that we support 1 one auth mechanism:
 		the 'no auth' mechanism. '''
 		return struct.pack('!BBB', 0x05, 0x01, 0x00)
-	
+
 	def _parse_auth_buff(self, buff):
 		''' Parse the initial message and create a list of auth
 		mechanisms '''
@@ -580,63 +575,64 @@ class Socks5:
 			for i in xrange(num_auth):
 				mechanism, = struct.unpack('!B', buff[1 + i])
 				auth_mechanisms.append(mechanism)
-		except:
+		except Exception:
 			return None
 		return auth_mechanisms
+
 	def _get_auth_response(self):
 		''' socks version(5), number of extra auth methods (we send
 		0x00 - no auth
 		) '''
 		return struct.pack('!BB', 0x05, 0x00)
-	
+
 	def _get_connect_buff(self):
 		''' Connect request by domain name '''
-		buff = struct.pack('!BBBBB%dsBB' % len(self.host), 
-			0x05, 0x01, 0x00, 0x03, len(self.host), self.host, 
+		buff = struct.pack('!BBBBB%dsBB' % len(self.host),
+			0x05, 0x01, 0x00, 0x03, len(self.host), self.host,
 			self.port >> 8, self.port & 0xff)
 		return buff
-	
+
 	def _get_request_buff(self, msg, command = 0x01):
-		''' Connect request by domain name, 
+		''' Connect request by domain name,
 		sid sha, instead of domain name (jep 0096) '''
-		buff = struct.pack('!BBBBB%dsBB' % len(msg), 
+		buff = struct.pack('!BBBBB%dsBB' % len(msg),
 			0x05, command, 0x00, 0x03, len(msg), msg, 0, 0)
 		return buff
-	
+
 	def _parse_request_buff(self, buff):
 		try: # don't trust on what comes from the outside
-			version, req_type, reserved, host_type,  = \
-				struct.unpack('!BBBB', buff[:4])
+			version, req_type, reserved, host_type, = struct.unpack('!BBBB',
+				buff[:4])
 			if host_type == 0x01:
 				host_arr = struct.unpack('!iiii', buff[4:8])
-				host, = reduce(lambda e1, e2: str(e1) + "." + str(e2), host_arr)
+				host, = '.'.join(str(s) for s in host_arr)
 				host_len = len(host)
 			elif host_type == 0x03:
 				host_len,  = struct.unpack('!B' , buff[4])
 				host, = struct.unpack('!%ds' % host_len, buff[5:5 + host_len])
 			portlen = len(buff[host_len + 5:])
-			if portlen == 1: 
+			if portlen == 1:
 				port, = struct.unpack('!B', buff[host_len + 5])
-			elif portlen == 2: 
+			elif portlen == 2:
 				port, = struct.unpack('!H', buff[host_len + 5:])
 			# file data, comes with auth message (Gaim bug)
-			else: 
+			else:
 				port, = struct.unpack('!H', buff[host_len + 5: host_len + 7])
 				self.remaining_buff = buff[host_len + 7:]
-		except:
+		except Exception:
 			return (None, None, None)
 		return (req_type, host, port)
-	
+
 	def read_connect(self):
 		''' connect responce: version, auth method '''
 		buff = self._recv()
 		try:
 			version, method = struct.unpack('!BB', buff)
-		except:
+		except Exception:
 			version, method = None, None
 		if version != 0x05 or method == 0xff:
 			self.disconnect()
-	
+
 	def continue_paused_transfer(self):
 		if self.state < 5:
 			return
@@ -644,18 +640,20 @@ class Socks5:
 			self.idlequeue.plug_idle(self, False, True)
 		else:
 			self.idlequeue.plug_idle(self, True, False)
-	
+
 	def _get_sha1_auth(self):
 		''' get sha of sid + Initiator jid + Target jid '''
-		if self.file_props.has_key('is_a_proxy'):
+		if 'is_a_proxy' in self.file_props:
 			del(self.file_props['is_a_proxy'])
-			return sha.new('%s%s%s' % (self.sid, self.file_props['proxy_sender'], 
+			return sha.new('%s%s%s' % (self.sid, self.file_props['proxy_sender'],
 				self.file_props['proxy_receiver'])).hexdigest()
-		return sha.new('%s%s%s' % (self.sid, self.initiator, self.target)).hexdigest()
-	
+		return sha.new('%s%s%s' % (self.sid, self.initiator, self.target)).\
+			hexdigest()
+
 class Socks5Sender(Socks5, IdleObject):
 	''' class for sending file to socket over socks5 '''
-	def __init__(self, idlequeue, sock_hash, parent, _sock, host = None, port = None):
+	def __init__(self, idlequeue, sock_hash, parent, _sock, host=None,
+	port=None):
 		self.queue_idx = sock_hash
 		self.queue = parent
 		Socks5.__init__(self, idlequeue, host, port, None, None, None)
@@ -669,7 +667,7 @@ class Socks5Sender(Socks5, IdleObject):
 		self.file_props = None
 		# start waiting for data
 		self.idlequeue.plug_idle(self, False, True)
-	
+
 	def read_timeout(self):
 		self.idlequeue.remove_timeout(self.fd)
 		if self.state > 5:
@@ -680,9 +678,9 @@ class Socks5Sender(Socks5, IdleObject):
 				if SEND_TIMEOUT > 0:
 					self.idlequeue.set_read_timeout(self.fd, SEND_TIMEOUT)
 			else:
-				# stop transfer, there is no error code for this 
+				# stop transfer, there is no error code for this
 				self.pollend()
-	
+
 	def pollout(self):
 		if not self.connected:
 			self.disconnect()
@@ -696,7 +694,7 @@ class Socks5Sender(Socks5, IdleObject):
 			if self.file_props['paused']:
 				self.file_props['continue_cb'] = self.continue_paused_transfer
 				self.idlequeue.plug_idle(self, False, False)
-				return 
+				return
 			result = self.write_next()
 			self.queue.process_result(result, self)
 			if result is None or result <= 0:
@@ -712,13 +710,13 @@ class Socks5Sender(Socks5, IdleObject):
 			self.state += 1
 			# unplug and plug this time for reading
 			self.idlequeue.plug_idle(self, False, True)
-	
+
 	def pollend(self):
 		self.state = 8 # end connection
 		self.disconnect()
 		self.file_props['error'] = -1
 		self.queue.process_result(-1, self)
-	
+
 	def pollin(self):
 		if self.connected:
 			if self.state < 5:
@@ -727,17 +725,16 @@ class Socks5Sender(Socks5, IdleObject):
 					self.queue.result_sha(self.sha_msg, self.queue_idx)
 				if result == -1:
 					self.disconnect()
-			
+
 			elif self.state == 5:
-				if self.file_props is not None and \
-				self.file_props['type'] == 'r':
+				if self.file_props is not None and self.file_props['type'] == 'r':
 					result = self.get_file_contents(0)
 					self.queue.process_result(result, self)
 		else:
 			self.disconnect()
-	
+
 	def send_file(self):
-		''' start sending the file over verified connection ''' 
+		''' start sending the file over verified connection '''
 		if self.file_props['started']:
 			return
 		self.file_props['error'] = 0
@@ -756,7 +753,7 @@ class Socks5Sender(Socks5, IdleObject):
 		# plug for writing
 		self.idlequeue.plug_idle(self, True, False)
 		return self.write_next() # initial for nl byte
-		
+
 	def main(self):
 		''' initial requests for verifying the connection '''
 		if self.state == 1: # initial read
@@ -775,8 +772,8 @@ class Socks5Sender(Socks5, IdleObject):
 		# unplug & plug for writing
 		self.idlequeue.plug_idle(self, True, False)
 		return None
-			
-	def disconnect(self, cb = True):
+
+	def disconnect(self, cb=True):
 		''' Closes the socket. '''
 		# close connection and remove us from the queue
 		Socks5.disconnect(self)
@@ -788,27 +785,43 @@ class Socks5Sender(Socks5, IdleObject):
 
 class Socks5Listener(IdleObject):
 	def __init__(self, idlequeue, port):
-		''' handle all incomming connections on (0.0.0.0, port) 
+		''' handle all incomming connections on (0.0.0.0, port)
 		This class implements IdleObject, but we will expect
 		only pollin events though
 		'''
 		self.port = port
-		self.queue_idx = -1	
+		self.ais = socket.getaddrinfo(None, port, socket.AF_UNSPEC,
+			socket.SOCK_STREAM, socket.SOL_TCP, socket.AI_PASSIVE)
+		self.queue_idx = -1
 		self.idlequeue = idlequeue
 		self.queue = None
 		self.started = False
 		self._sock = None
 		self.fd = -1
-		
+
 	def bind(self):
-		self._serv = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-		self._serv.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-		self._serv.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
-		self._serv.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
-		# will fail when port as busy, or we don't have rights to bind
-		try:
-			self._serv.bind(('0.0.0.0', self.port))
-		except Exception, e:
+		for ai in self.ais:
+			#try the different possibilities (ipv6, ipv4, etc.)
+			self._serv = socket.socket(*ai[:3])
+			self._serv.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+			self._serv.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
+			self._serv.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+			# Under windows Vista, we need that to listen on ipv6 AND ipv4
+			# Doesn't work under windows XP
+			if os.name == 'nt':
+				ver = os.sys.getwindowsversion()
+				if (ver[3], ver[0], ver[1]) == (2, 6, 0):
+					# 27 is socket.IPV6_V6ONLY under windows, but not defined ...
+					self._serv.setsockopt(socket.IPPROTO_IPV6, 27, 1)
+			# will fail when port as busy, or we don't have rights to bind
+			try:
+				self._serv.bind(ai[4])
+				self.ai = ai
+				break
+			except Exception:
+				self.ai = None
+				continue
+		if not self.ai:
 			# unable to bind, show error dialog
 			return None
 		self._serv.listen(socket.SOMAXCONN)
@@ -816,16 +829,16 @@ class Socks5Listener(IdleObject):
 		self.fd = self._serv.fileno()
 		self.idlequeue.plug_idle(self, False, True)
 		self.started = True
-	
+
 	def pollend(self):
 		''' called when we stop listening on (host, port) '''
 		self.disconnect()
-	
+
 	def pollin(self):
 		''' accept a new incomming connection and notify queue'''
 		sock = self.accept_conn()
 		self.queue.on_connection_accepted(sock)
-	
+
 	def disconnect(self):
 		''' free all resources, we are not listening anymore '''
 		self.idlequeue.remove_timeout(self.fd)
@@ -835,15 +848,15 @@ class Socks5Listener(IdleObject):
 		self.started = False
 		try:
 			self._serv.close()
-		except:
+		except Exception:
 			pass
-	
+
 	def accept_conn(self):
 		''' accepts a new incomming connection '''
 		_sock  = self._serv.accept()
 		_sock[0].setblocking(False)
 		return _sock
-	
+
 class Socks5Receiver(Socks5, IdleObject):
 	def __init__(self, idlequeue, streamhost, sid, file_props = None):
 		self.queue_idx = -1
@@ -862,42 +875,56 @@ class Socks5Receiver(Socks5, IdleObject):
 		self.file_props['paused'] = False
 		self.file_props['continue_cb'] = self.continue_paused_transfer
 		self.file_props['stalled'] = False
-		Socks5.__init__(self, idlequeue, streamhost['host'], int(streamhost['port']), 
-			streamhost['initiator'], streamhost['target'], sid)
-	
+		Socks5.__init__(self, idlequeue, streamhost['host'],
+			int(streamhost['port']), streamhost['initiator'], streamhost['target'],
+			sid)
+
 	def read_timeout(self):
 		self.idlequeue.remove_timeout(self.fd)
 		if self.state > 5:
 			# no activity for foo seconds
 			if self.file_props['stalled'] == False:
 				self.file_props['stalled'] = True
-				if not self.file_props.has_key('received-len'):
+				if 'received-len' not in self.file_props:
 					self.file_props['received-len'] = 0
 				self.queue.process_result(-1, self)
 				if READ_TIMEOUT > 0:
 					self.idlequeue.set_read_timeout(self.fd, READ_TIMEOUT)
 			else:
-				# stop transfer, there is no error code for this 
+				# stop transfer, there is no error code for this
 				self.pollend()
 		else:
 			self.queue.reconnect_receiver(self, self.streamhost)
-	
+
 	def connect(self):
 		''' create the socket and plug it to the idlequeue '''
-		self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-		# this will not block the GUI
-		self._sock.setblocking(False)
+		if self.ais is None:
+			return None
+
+		for ai in self.ais:
+			try:
+				self._sock = socket.socket(*ai[:3])
+				# this will not block the GUI
+				self._sock.setblocking(False)
+				self._server = ai[4]
+				break
+			except Exception:
+				if sys.exc_value[0] == errno.EINPROGRESS:
+					break
+				#for all errors, we try other addresses
+				continue
 		self.fd = self._sock.fileno()
 		self.state = 0 # about to be connected
 		self.idlequeue.plug_idle(self, True, False)
 		self.do_connect()
 		self.idlequeue.set_read_timeout(self.fd, CONNECT_TIMEOUT)
 		return None
-	
+
 	def _is_connected(self):
 		if self.state < 5:
 			return False
 		return True
+
 	def pollout(self):
 		self.idlequeue.remove_timeout(self.fd)
 		if self.state == 0:
@@ -908,7 +935,7 @@ class Socks5Receiver(Socks5, IdleObject):
 		elif self.state == 3: # send 'connect' request
 			self.send_raw(self._get_request_buff(self._get_sha1_auth()))
 		elif self.file_props['type'] != 'r':
-			if self.file_props['paused'] == True:
+			if self.file_props['paused']:
 				self.idlequeue.plug_idle(self, False, False)
 				return
 			result = self.write_next()
@@ -918,7 +945,7 @@ class Socks5Receiver(Socks5, IdleObject):
 		# unplug and plug for reading
 		self.idlequeue.plug_idle(self, False, True)
 		self.idlequeue.set_read_timeout(self.fd, CONNECT_TIMEOUT)
-	
+
 	def pollend(self):
 		if self.state >= 5:
 			# error during transfer
@@ -927,7 +954,7 @@ class Socks5Receiver(Socks5, IdleObject):
 			self.queue.process_result(-1, self)
 		else:
 			self.queue.reconnect_receiver(self, self.streamhost)
-	
+
 	def pollin(self):
 		self.idlequeue.remove_timeout(self.fd)
 		if self.connected:
@@ -944,13 +971,12 @@ class Socks5Receiver(Socks5, IdleObject):
 				self.idlequeue.set_read_timeout(self.fd, STALLED_TIMEOUT)
 				result = self.get_file_contents(0)
 				self.queue.process_result(result, self)
-				
 		else:
 			self.disconnect()
-	
+
 	def do_connect(self):
 		try:
-			self._sock.connect((self.host, self.port))
+			self._sock.connect(self._server)
 			self._sock.setblocking(False)
 			self._send=self._sock.send
 			self._recv=self._sock.recv
@@ -958,7 +984,7 @@ class Socks5Receiver(Socks5, IdleObject):
 			(errnum, errstr) = ee
 			self.connect_timeout += 1
 			if errnum == 111 or self.connect_timeout > 1000:
-				self.queue._connection_refused(self.streamhost, 
+				self.queue._connection_refused(self.streamhost,
 					self.file_props, self.queue_idx)
 				return None
 			# win32 needs this
@@ -973,13 +999,13 @@ class Socks5Receiver(Socks5, IdleObject):
 		self.file_props['connected'] = True
 		self.file_props['disconnect_cb'] = self.disconnect
 		self.state = 1 # connected
-		
+
 		# stop all others connections to sender's streamhosts
 		self.queue._socket_connected(self.streamhost, self.file_props)
 		self.idlequeue.plug_idle(self, True, False)
 		return 1 # we are connected
-		
-	def main(self, timeout = 0):
+
+	def main(self, timeout=0):
 		''' begin negotiation. on success 'address' != 0 '''
 		result = 1
 		buff = self.receive()
@@ -987,7 +1013,7 @@ class Socks5Receiver(Socks5, IdleObject):
 			# end connection
 			self.pollend()
 			return
-		
+
 		if self.state == 2: # read auth response
 			if buff is None or len(buff) != 2:
 				return None
@@ -995,7 +1021,7 @@ class Socks5Receiver(Socks5, IdleObject):
 			if version != 0x05 or method == 0xff:
 				self.disconnect()
 		elif self.state == 4: # get approve of our request
-			if buff == None:
+			if buff is None:
 				return None
 			sub_buff = buff[:4]
 			if len(sub_buff) < 4:
@@ -1006,7 +1032,7 @@ class Socks5Receiver(Socks5, IdleObject):
 				addrlen = ord(buff[4])
 				address = struct.unpack('!%ds' % addrlen, buff[5:addrlen + 5])
 				portlen = len(buff[addrlen + 5:])
-				if portlen == 1: 
+				if portlen == 1:
 					port, = struct.unpack('!B', buff[addrlen + 5])
 				elif portlen == 2:
 					port, = struct.unpack('!H', buff[addrlen + 5:])
@@ -1015,14 +1041,14 @@ class Socks5Receiver(Socks5, IdleObject):
 					self.remaining_buff = buff[addrlen + 7:]
 			self.state = 5 # for senders: init file_props and send '\n'
 			if self.queue.on_success:
-				result = self.queue.send_success_reply(self.file_props, 
+				result = self.queue.send_success_reply(self.file_props,
 					self.streamhost)
 				if result == 0:
 					self.state = 8
 					self.disconnect()
-		
-		# for senders: init file_props 
-		if result == 1 and self.state == 5: 
+
+		# for senders: init file_props
+		if result == 1 and self.state == 5:
 			if self.file_props['type'] == 's':
 				self.file_props['error'] = 0
 				self.file_props['disconnect_cb'] = self.disconnect
@@ -1042,17 +1068,19 @@ class Socks5Receiver(Socks5, IdleObject):
 				self.idlequeue.plug_idle(self, False, True)
 			self.file_props['continue_cb'] = self.continue_paused_transfer
 			# we have set up the connection, next - retrieve file
-			self.state = 6 
+			self.state = 6
 		if self.state < 5:
 			self.idlequeue.plug_idle(self, True, False)
 			self.state += 1
 			return None
-	
-	def disconnect(self, cb = True):
+
+	def disconnect(self, cb=True):
 		''' Closes the socket. Remove self from queue if cb is True'''
-		# close connection 
+		# close connection
 		Socks5.disconnect(self)
 		if cb is True:
 			self.file_props['disconnect_cb'] = None
 		if self.queue is not None:
 			self.queue.remove_receiver(self.queue_idx, False)
+
+# vim: se ts=3:
