@@ -96,7 +96,7 @@ class Browser(PlugIn):
         self._owner.UnregisterHandler('iq',self._DiscoveryHandler,typ='get',ns=NS_DISCO_INFO)
         self._owner.UnregisterHandler('iq',self._DiscoveryHandler,typ='get',ns=NS_DISCO_ITEMS)
 
-    def _traversePath(self,node,jid,set=0):
+    def _traversePath(self,node,jid,set_=0):
         """ Returns dictionary and key or None,None
             None - root node (w/o "node" attribute)
             /a/b/c - node
@@ -104,19 +104,19 @@ class Browser(PlugIn):
             Set returns '' or None as the key
             get returns '' or None as the key or None as the dict.
             Used internally."""
-        if self._handlers.has_key(jid): cur=self._handlers[jid]
-        elif set:
+        if jid in self._handlers: cur=self._handlers[jid]
+        elif set_:
             self._handlers[jid]={}
             cur=self._handlers[jid]
         else: cur=self._handlers['']
         if node is None: node=[None]
         else: node=node.replace('/',' /').split('/')
         for i in node:
-            if i<>'' and cur.has_key(i): cur=cur[i]
-            elif set and i<>'': cur[i]={dict:cur,str:i}; cur=cur[i]
-            elif set or cur.has_key(''): return cur,''
+            if i!='' and i in cur: cur=cur[i]
+            elif set_ and i!='': cur[i]={dict:cur,str:i}; cur=cur[i]
+            elif set_ or '' in cur: return cur,''
             else: return None,None
-        if cur.has_key(1) or set: return cur,1
+        if 1 in cur or set_: return cur,1
         raise "Corrupted data"
 
     def setDiscoHandler(self,handler,node='',jid=''):
@@ -195,22 +195,24 @@ class Browser(PlugIn):
         q=rep.getTag('query')
         if request.getQueryNS()==NS_DISCO_ITEMS:
             # handler must return list: [{jid,action,node,name}]
-            if type(handler)==dict: lst=handler['items']
+            if isinstance(handler, dict): lst=handler['items']
             else: lst=handler(conn,request,'items')
-            if lst==None:
+            if lst is None:
                 conn.send(Error(request,ERR_ITEM_NOT_FOUND))
                 raise NodeProcessed
             for item in lst: q.addChild('item',item)
         elif request.getQueryNS()==NS_DISCO_INFO:
-            if type(handler)==dict: dt=handler['info']
+            if isinstance(handler, dict): dt=handler['info']
             else: dt=handler(conn,request,'info')
-            if dt==None:
+            if dt is None:
                 conn.send(Error(request,ERR_ITEM_NOT_FOUND))
                 raise NodeProcessed
             # handler must return dictionary:
             # {'ids':[{},{},{},{}], 'features':[fe,at,ur,es], 'xdata':DataForm}
             for id in dt['ids']: q.addChild('identity',id)
             for feature in dt['features']: q.addChild('feature',{'var':feature})
-            if dt.has_key('xdata'): q.addChild(node=dt['xdata'])
+            if 'xdata' in dt: q.addChild(node=dt['xdata'])
         conn.send(rep)
         raise NodeProcessed
+
+# vim: se ts=3:
\ No newline at end of file
