@@ -49,7 +49,7 @@ class Dispatcher(PlugIn):
     def getAnID(self):
         global ID
         ID+=1
-        return `ID`
+        return repr(ID)
 
     def dumpHandlers(self):
         """ Return set of user-registered callbacks in it's internal format.
@@ -107,7 +107,7 @@ class Dispatcher(PlugIn):
         self._owner.send("<?xml version='1.0'?>%s>"%str(self._metastream)[:-2])
 
     def _check_stream_start(self,ns,tag,attrs):
-        if ns<>NS_STREAMS or tag<>'stream':
+        if ns!=NS_STREAMS or tag!='stream':
             raise ValueError('Incorrect stream start: (%s,%s). Terminating.'%(tag,ns))
 
     def Process(self, timeout=0):
@@ -175,9 +175,9 @@ class Dispatcher(PlugIn):
         if not xmlns: xmlns=self._owner.defaultNamespace
         self.DEBUG('Registering handler %s for "%s" type->%s ns->%s(%s)'%(handler,name,typ,ns,xmlns), 'info')
         if not typ and not ns: typ='default'
-        if not self.handlers.has_key(xmlns): self.RegisterNamespace(xmlns,'warn')
-        if not self.handlers[xmlns].has_key(name): self.RegisterProtocol(name,Protocol,xmlns,'warn')
-        if not self.handlers[xmlns][name].has_key(typ+ns): self.handlers[xmlns][name][typ+ns]=[]
+        if xmlns not in self.handlers: self.RegisterNamespace(xmlns,'warn')
+        if name not in self.handlers[xmlns]: self.RegisterProtocol(name,Protocol,xmlns,'warn')
+        if typ+ns not in self.handlers[xmlns][name]: self.handlers[xmlns][name][typ+ns]=[]
         if makefirst: self.handlers[xmlns][name][typ+ns].insert(0,{'func':handler,'system':system})
         else: self.handlers[xmlns][name][typ+ns].append({'func':handler,'system':system})
 
@@ -190,8 +190,8 @@ class Dispatcher(PlugIn):
         """ Unregister handler. "typ" and "ns" must be specified exactly the same as with registering."""
         if not xmlns: xmlns=self._owner.defaultNamespace
         if not typ and not ns: typ='default'
-        if not self.handlers[xmlns].has_key(name): return
-        if not self.handlers[xmlns][name].has_key(typ+ns): return
+        if name not in self.handlers[xmlns]: return
+        if typ+ns not in self.handlers[xmlns][name]: return
         for pack in self.handlers[xmlns][name][typ+ns]:
             if handler==pack['func']: break
         else: pack=None
@@ -209,7 +209,7 @@ class Dispatcher(PlugIn):
 
     def returnStanzaHandler(self,conn,stanza):
         """ Return stanza back to the sender with <feature-not-implemennted/> error set. """
-        if stanza.getType() in ['get','set']:
+        if stanza.getType() in ('get','set'):
             conn.send(Error(stanza,ERR_FEATURE_NOT_IMPLEMENTED))
 
     def streamErrorHandler(self,conn,error):
@@ -246,7 +246,7 @@ class Dispatcher(PlugIn):
 
         if not direct and self._owner._component:
             if name == 'route':
-                if stanza.getAttr('error') == None:
+                if stanza.getAttr('error') is None:
                     if len(stanza.getChildren()) == 1:
                         stanza = stanza.getChildren()[0]
                         name=stanza.getName()
@@ -264,10 +264,10 @@ class Dispatcher(PlugIn):
         if name=='features': session.Stream.features=stanza
 
         xmlns=stanza.getNamespace()
-        if not self.handlers.has_key(xmlns):
+        if xmlns not in self.handlers:
             self.DEBUG("Unknown namespace: " + xmlns,'warn')
             xmlns='unknown'
-        if not self.handlers[xmlns].has_key(name):
+        if name not in self.handlers[xmlns]:
             self.DEBUG("Unknown stanza: " + name,'warn')
             name='unknown'
         else:
@@ -283,24 +283,24 @@ class Dispatcher(PlugIn):
         session.DEBUG("Dispatching %s stanza with type->%s props->%s id->%s"%(name,typ,stanza.props,ID),'ok')
 
         list=['default']                                                     # we will use all handlers:
-        if self.handlers[xmlns][name].has_key(typ): list.append(typ)                # from very common...
+        if typ in self.handlers[xmlns][name]: list.append(typ)                # from very common...
         for prop in stanza.props:
-            if self.handlers[xmlns][name].has_key(prop): list.append(prop)
-            if typ and self.handlers[xmlns][name].has_key(typ+prop): list.append(typ+prop)  # ...to very particular
+            if prop in self.handlers[xmlns][name]: list.append(prop)
+            if typ and typ+prop in self.handlers[xmlns][name]: list.append(typ+prop)  # ...to very particular
 
         chain=self.handlers[xmlns]['default']['default']
         for key in list:
             if key: chain = chain + self.handlers[xmlns][name][key]
 
         output=''
-        if session._expected.has_key(ID):
+        if ID in session._expected:
             user=0
-            if type(session._expected[ID])==type(()):
+            if isinstance(session._expected[ID], tuple):
                 cb,args=session._expected[ID]
                 session.DEBUG("Expected stanza arrived. Callback %s(%s) found!"%(cb,args),'ok')
                 try: cb(session,stanza,**args)
                 except Exception, typ:
-                    if typ.__class__.__name__<>'NodeProcessed': raise
+                    if typ.__class__.__name__!='NodeProcessed': raise
             else:
                 session.DEBUG("Expected stanza arrived!",'ok')
                 session._expected[ID]=stanza
@@ -310,7 +310,7 @@ class Dispatcher(PlugIn):
                 try:
                     handler['func'](session,stanza)
                 except Exception, typ:
-                    if typ.__class__.__name__<>'NodeProcessed':
+                    if typ.__class__.__name__!='NodeProcessed':
                         self._pendingExceptions.insert(0, sys.exc_info())
                         return
                     user=0
@@ -354,12 +354,12 @@ class Dispatcher(PlugIn):
     def send(self,stanza):
         """ Serialise stanza and put it on the wire. Assign an unique ID to it before send.
             Returns assigned ID."""
-        if type(stanza) in [type(''), type(u'')]: return self._owner_send(stanza)
+        if isinstance(stanza, basestring): return self._owner_send(stanza)
         if not isinstance(stanza,Protocol): _ID=None
         elif not stanza.getID():
             global ID
             ID+=1
-            _ID=`ID`
+            _ID=repr(ID)
             stanza.setID(_ID)
         else: _ID=stanza.getID()
         if self._owner._registered_name and not stanza.getAttr('from'): stanza.setAttr('from',self._owner._registered_name)
@@ -381,3 +381,5 @@ class Dispatcher(PlugIn):
         """ Send a stream terminator and and handle all incoming stanzas before stream closure. """
         self._owner_send('</stream:stream>')
         while self.Process(1): pass
+
+# vim: se ts=3:
