@@ -23,7 +23,7 @@ Contains one tunable attribute: DefaultTimeout (25 seconds by default). It defin
 Dispatcher.SendAndWaitForResponce method will wait for reply stanza before giving up.
 '''
 
-import simplexml, sys
+import simplexml, sys, locale
 from xml.parsers.expat import ExpatError
 from protocol import *
 from client import PlugIn
@@ -53,7 +53,7 @@ class Dispatcher(PlugIn):
 	def getAnID(self):
 		global ID
 		ID += 1
-		return `ID`
+		return repr(ID)
 
 	def dumpHandlers(self):
 		''' Return set of user-registered callbacks in it's internal format.
@@ -111,10 +111,13 @@ class Dispatcher(PlugIn):
 		self._metastream.setAttr('version', '1.0')
 		self._metastream.setAttr('xmlns:stream', NS_STREAMS)
 		self._metastream.setAttr('to', self._owner.Server)
+		if locale.getdefaultlocale()[0]:
+			self._metastream.setAttr('xml:lang',
+				locale.getdefaultlocale()[0].split('_')[0])
 		self._owner.send("<?xml version='1.0'?>%s>" % str(self._metastream)[:-2])
 
 	def _check_stream_start(self, ns, tag, attrs):
-		if ns<>NS_STREAMS or tag<>'stream':
+		if ns!=NS_STREAMS or tag!='stream':
 			raise ValueError('Incorrect stream start: (%s,%s). Terminating.' % (tag, ns))
 
 	def ProcessNonBlocking(self, data=None):
@@ -135,10 +138,13 @@ class Dispatcher(PlugIn):
 				self._owner.Connection.disconnect()
 				return 0
 		except ExpatError:
-			sys.exc_clear()
 			self.DEBUG('Invalid XML received from server. Forcing disconnect.', 'error')
 			self._owner.Connection.pollend()
 			return 0
+		except ValueError, e:
+			self.DEBUG(str(e), 'error')
+			self._owner.Connection.pollend()
+			return 0
 		if len(self._pendingExceptions) > 0:
 			 _pendingException = self._pendingExceptions.pop()
 			 raise _pendingException[0], _pendingException[1], _pendingException[2]
@@ -191,11 +197,11 @@ class Dispatcher(PlugIn):
 								(handler, name, typ, ns, xmlns), 'info')
 		if not typ and not ns: 
 			typ='default'
-		if not self.handlers.has_key(xmlns): 
+		if xmlns not in self.handlers: 
 			self.RegisterNamespace(xmlns,'warn')
-		if not self.handlers[xmlns].has_key(name): 
+		if name not in self.handlers[xmlns]: 
 			self.RegisterProtocol(name,Protocol,xmlns,'warn')
-		if not self.handlers[xmlns][name].has_key(typ+ns): 
+		if typ+ns not in self.handlers[xmlns][name]: 
 			self.handlers[xmlns][name][typ+ns]=[]
 		if makefirst: 
 			self.handlers[xmlns][name][typ+ns].insert(0,{'func':handler,'system':system})
@@ -214,11 +220,11 @@ class Dispatcher(PlugIn):
 			xmlns=self._owner.defaultNamespace
 		if not typ and not ns: 
 			typ='default'
-		if not self.handlers.has_key(xmlns):
+		if xmlns not in self.handlers:
 			return
-		if not self.handlers[xmlns].has_key(name): 
+		if name not in self.handlers[xmlns]: 
 			return
-		if not self.handlers[xmlns][name].has_key(typ+ns): 
+		if typ+ns not in self.handlers[xmlns][name]: 
 			return
 		for pack in self.handlers[xmlns][name][typ+ns]:
 			if handler==pack['func']: 
@@ -241,7 +247,7 @@ class Dispatcher(PlugIn):
 
 	def returnStanzaHandler(self,conn,stanza):
 		''' Return stanza back to the sender with <feature-not-implemennted/> error set. '''
-		if stanza.getType() in ['get','set']:
+		if stanza.getType() in ('get','set'):
 			conn.send(Error(stanza,ERR_FEATURE_NOT_IMPLEMENTED))
 
 	def streamErrorHandler(self,conn,error):
@@ -285,7 +291,7 @@ class Dispatcher(PlugIn):
 
 		if not direct and self._owner._component:
 			if name == 'route':
-				if stanza.getAttr('error') == None:
+				if stanza.getAttr('error') is None:
 					if len(stanza.getChildren()) == 1:
 						stanza = stanza.getChildren()[0]
 						name=stanza.getName()
@@ -304,10 +310,10 @@ class Dispatcher(PlugIn):
 			session.Stream.features=stanza
 		
 		xmlns=stanza.getNamespace()
-		if not self.handlers.has_key(xmlns):
+		if xmlns not in self.handlers:
 			self.DEBUG("Unknown namespace: " + xmlns, 'warn')
 			xmlns='unknown'
-		if not self.handlers[xmlns].has_key(name):
+		if name not in self.handlers[xmlns]:
 			self.DEBUG("Unknown stanza: " + name, 'warn')
 			name='unknown'
 		else:
@@ -323,25 +329,25 @@ class Dispatcher(PlugIn):
 		
 		session.DEBUG("Dispatching %s stanza with type->%s props->%s id->%s"%(name,typ,stanza.props,ID),'ok')
 		list=['default']                                                     # we will use all handlers:
-		if self.handlers[xmlns][name].has_key(typ): list.append(typ)                # from very common...
+		if typ in self.handlers[xmlns][name]: list.append(typ)                # from very common...
 		for prop in stanza.props:
-			if self.handlers[xmlns][name].has_key(prop): list.append(prop)
-			if typ and self.handlers[xmlns][name].has_key(typ+prop): list.append(typ+prop)  # ...to very particular
+			if prop in self.handlers[xmlns][name]: list.append(prop)
+			if typ and typ+prop in self.handlers[xmlns][name]: list.append(typ+prop)  # ...to very particular
 		
 		chain=self.handlers[xmlns]['default']['default']
 		for key in list:
 			if key: chain = chain + self.handlers[xmlns][name][key]
 		
 		output=''
-		if session._expected.has_key(ID):
+		if ID in session._expected:
 			user=0
-			if type(session._expected[ID]) == type(()):
+			if isinstance(session._expected[ID], tuple):
 				cb,args = session._expected[ID]
 				session.DEBUG("Expected stanza arrived. Callback %s(%s) found!" % (cb, args), 'ok')
 				try: 
 					cb(session,stanza,**args)
 				except Exception, typ:
-					if typ.__class__.__name__ <>'NodeProcessed': raise
+					if typ.__class__.__name__ !='NodeProcessed': raise
 			else:
 				session.DEBUG("Expected stanza arrived!",'ok')
 				session._expected[ID]=stanza
@@ -352,7 +358,7 @@ class Dispatcher(PlugIn):
 				try:
 					handler['func'](session,stanza)
 				except Exception, typ:
-					if typ.__class__.__name__ <> 'NodeProcessed':
+					if typ.__class__.__name__ != 'NodeProcessed':
 						self._pendingExceptions.insert(0, sys.exc_info())
 						return
 					user=0
@@ -370,7 +376,7 @@ class Dispatcher(PlugIn):
 		self._owner.remove_timeout()
 		if self._expected[self._witid] is None:
 			return
-		if self.on_responses.has_key(self._witid):
+		if self._witid in self.on_responses:
 			i = self._witid # copy id cause it can change in resp() call
 			self._owner.onreceive(None)
 			resp, args = self.on_responses[self._witid]
@@ -402,14 +408,14 @@ class Dispatcher(PlugIn):
 	def send(self, stanza, is_message = False, now = False):
 		''' Serialise stanza and put it on the wire. Assign an unique ID to it before send.
 			Returns assigned ID.'''
-		if type(stanza) in [type(''), type(u'')]: 
+		if isinstance(stanza, basestring):
 			return self._owner.Connection.send(stanza, now = now)
 		if not isinstance(stanza, Protocol): 
 			_ID=None
 		elif not stanza.getID():
 			global ID
 			ID+=1
-			_ID=`ID`
+			_ID=repr(ID)
 			stanza.setID(_ID)
 		else: 
 			_ID=stanza.getID()
@@ -435,3 +441,5 @@ class Dispatcher(PlugIn):
 	def disconnect(self):
 		''' Send a stream terminator. '''
 		self._owner.Connection.send('</stream:stream>')
+
+# vim: se ts=3:
