@@ -27,11 +27,14 @@ NS_AGENTS       ='jabber:iq:agents'
 NS_AMP          ='http://jabber.org/protocol/amp'
 NS_AMP_ERRORS   =NS_AMP+'#errors'
 NS_AUTH         ='jabber:iq:auth'
+NS_AVATAR       ='http://www.xmpp.org/extensions/xep-0084.html#ns-metadata'
 NS_BIND         ='urn:ietf:params:xml:ns:xmpp-bind'
 NS_BROWSE       ='jabber:iq:browse'
-NS_BYTESTREAM   ='http://jabber.org/protocol/bytestreams'               # XEP-0065
-NS_CAPS         ='http://jabber.org/protocol/caps'                      # XEP-0115
-NS_CHATSTATES   ='http://jabber.org/protocol/chatstates'                # XEP-0085
+NS_BROWSING     ='http://jabber.org/protocol/browsing'                  # XEP-0195
+NS_BYTESTREAM   ='http://jabber.org/protocol/bytestreams'               # JEP-0065
+NS_CAPS         ='http://jabber.org/protocol/caps'                      # JEP-0115
+NS_CHATSTATES   ='http://jabber.org/protocol/chatstates'                # JEP-0085
+NS_CHATTING     ='http://jabber.org/protocol/chatting'                  # XEP-0194
 NS_CLIENT       ='jabber:client'
 NS_COMMANDS     ='http://jabber.org/protocol/commands'
 NS_COMPONENT_ACCEPT='jabber:component:accept'
@@ -40,15 +43,19 @@ NS_COMPRESS     ='http://jabber.org/protocol/compress'                  # XEP-01
 NS_CONFERENCE   ='jabber:x:conference'
 NS_DATA         ='jabber:x:data'                                        # XEP-0004
 NS_DELAY        ='jabber:x:delay'
+NS_DELAY2       ='urn:xmpp:delay'
 NS_DIALBACK     ='jabber:server:dialback'
 NS_DISCO        ='http://jabber.org/protocol/disco'
 NS_DISCO_INFO   =NS_DISCO+'#info'
 NS_DISCO_ITEMS  =NS_DISCO+'#items'
 NS_ENCRYPTED    ='jabber:x:encrypted'                                   # XEP-0027
+NS_ESESSION     ='http://www.xmpp.org/extensions/xep-0116.html#ns'
+NS_ESESSION_INIT='http://www.xmpp.org/extensions/xep-0116.html#ns-init' # XEP-0116
 NS_EVENT        ='jabber:x:event'                                       # XEP-0022
-NS_FEATURE      ='http://jabber.org/protocol/feature-neg'  
-NS_FILE         ='http://jabber.org/protocol/si/profile/file-transfer'  # XEP-0096
-NS_GEOLOC       ='http://jabber.org/protocol/geoloc'                    # XEP-0080
+NS_FEATURE      ='http://jabber.org/protocol/feature-neg'
+NS_FILE         ='http://jabber.org/protocol/si/profile/file-transfer'  # JEP-0096
+NS_GAMING       ='http://jabber.org/protocol/gaming'                    # XEP-0196
+NS_GEOLOC       ='http://jabber.org/protocol/geoloc'                    # JEP-0080
 NS_GROUPCHAT    ='gc-1.0'
 NS_HTTP_AUTH    ='http://jabber.org/protocol/http-auth'                 # XEP-0070
 NS_HTTP_BIND    ='http://jabber.org/protocol/httpbind'                  # XEP-0124
@@ -62,14 +69,18 @@ NS_MUC          ='http://jabber.org/protocol/muc'
 NS_MUC_USER     =NS_MUC+'#user'
 NS_MUC_ADMIN    =NS_MUC+'#admin'
 NS_MUC_OWNER    =NS_MUC+'#owner'
+NS_MUC_UNIQUE   =NS_MUC+'#unique'
+NS_MUC_CONFIG   =NS_MUC+'#roomconfig'
 NS_NICK         ='http://jabber.org/protocol/nick'                      # XEP-0172
 NS_OFFLINE      ='http://www.jabber.org/jeps/jep-0030.html'             # XEP-0013
 NS_PHYSLOC      ='http://jabber.org/protocol/physloc'                   # XEP-0112
+NS_PING         ='urn:xmpp:ping'                                        # SEP-0199
 NS_PRESENCE     ='presence'                                             # Jabberd2
 NS_PRIVACY      ='jabber:iq:privacy'
 NS_PRIVATE      ='jabber:iq:private'
 NS_PROFILE      ='http://jabber.org/protocol/profile'                   # XEP-0154
 NS_PUBSUB       ='http://jabber.org/protocol/pubsub'                    # XEP-0060
+NS_PUBSUB_OWNER ='http://jabber.org/protocol/pubsub#owner'              # JEP-0060
 NS_REGISTER     ='jabber:iq:register'
 NS_ROSTER       ='jabber:iq:roster'
 NS_ROSTERX      ='http://jabber.org/protocol/rosterx'                   # XEP-0144
@@ -81,25 +92,29 @@ NS_SESSION      ='urn:ietf:params:xml:ns:xmpp-session'
 NS_SI           ='http://jabber.org/protocol/si'                        # XEP-0096
 NS_SI_PUB       ='http://jabber.org/protocol/sipub'                     # XEP-0137
 NS_SIGNED       ='jabber:x:signed'                                      # XEP-0027
+NS_SSN          ='urn:xmpp:ssn'                                         # XEP-0155
+NS_STANZA_CRYPTO='http://www.xmpp.org/extensions/xep-0200.html#ns'      # XEP-0200
 NS_STANZAS      ='urn:ietf:params:xml:ns:xmpp-stanzas'
 NS_STREAM       ='http://affinix.com/jabber/stream'
 NS_STREAMS      ='http://etherx.jabber.org/streams'
 NS_TIME         ='jabber:iq:time'                                       # XEP-0900
 NS_TIME_REVISED ='urn:xmpp:time'                                        # XEP-0202
 NS_TLS          ='urn:ietf:params:xml:ns:xmpp-tls'
+NS_TUNE         ='http://jabber.org/protocol/tune'                      # XEP-0118
 NS_VACATION     ='http://jabber.org/protocol/vacation'
 NS_VCARD        ='vcard-temp'
 NS_GMAILNOTIFY  ='google:mail:notify'
 NS_GTALKSETTING ='google:setting'
 NS_VCARD_UPDATE =NS_VCARD+':x:update'
 NS_VERSION      ='jabber:iq:version'
-NS_PING         ='urn:xmpp:ping'                                        # XEP-0199
+NS_VIEWING      ='http://jabber.org/protocol/viewing'                   # XEP--197
 NS_WAITINGLIST  ='http://jabber.org/protocol/waitinglist'               # XEP-0130
 NS_XHTML_IM     ='http://jabber.org/protocol/xhtml-im'                  # XEP-0071
 NS_XHTML        = 'http://www.w3.org/1999/xhtml'                        #  "
 NS_DATA_LAYOUT  ='http://jabber.org/protocol/xdata-layout'              # XEP-0141
 NS_DATA_VALIDATE='http://jabber.org/protocol/xdata-validate'            # XEP-0122
 NS_XMPP_STREAMS ='urn:ietf:params:xml:ns:xmpp-streams'
+NS_RECEIPTS	='urn:xmpp:receipts'
 
 xmpp_stream_error_conditions="""
 bad-format --  --  -- The entity has sent XML that cannot be processed.
@@ -159,9 +174,9 @@ not-authorized --  --  -- The authentication failed because the initiating entit
 temporary-auth-failure --  --  -- The authentication failed because of a temporary error condition within the receiving entity; sent in reply to an <auth/> element or <response/> element."""
 
 ERRORS,_errorcodes={},{}
-for ns,errname,errpool in [(NS_XMPP_STREAMS,'STREAM',xmpp_stream_error_conditions),
+for ns,errname,errpool in ((NS_XMPP_STREAMS,'STREAM',xmpp_stream_error_conditions),
                            (NS_STANZAS     ,'ERR'   ,xmpp_stanza_error_conditions),
-                           (NS_SASL        ,'SASL'  ,sasl_error_conditions)]:
+                           (NS_SASL        ,'SASL'  ,sasl_error_conditions)):
     for err in errpool.split('\n')[1:]:
         cond,code,typ,text=err.split(' -- ')
         name=errname+'_'+cond.upper().replace('-','_')
@@ -240,7 +255,7 @@ class JID:
             JID(node='node',domain='domain.org')
         """
         if not jid and not domain: raise ValueError('JID must contain at least domain name')
-        elif type(jid)==type(self): self.node,self.domain,self.resource=jid.node,jid.domain,jid.resource
+        elif isinstance(jid, type(self)): self.node,self.domain,self.resource=jid.node,jid.domain,jid.resource
         elif domain: self.node,self.domain,self.resource=node,domain,resource
         else:
             if jid.find('@')+1: self.node,jid=jid.split('@',1)
@@ -307,12 +322,17 @@ class Protocol(Node):
         if not node and xmlns: self.setNamespace(xmlns)
         if self['to']: self.setTo(self['to'])
         if self['from']: self.setFrom(self['from'])
-        if node and type(self)==type(node) and self.__class__==node.__class__ and self.attrs.has_key('id'): del self.attrs['id']
+        if node and isinstance(self, type(node)) and self.__class__==node.__class__ and 'id' in self.attrs: del self.attrs['id']
         self.timestamp=None
-        for x in self.getTags('x',namespace=NS_DELAY):
+        for d in self.getTags('delay',namespace=NS_DELAY2):
             try:
-                if x.getAttr('stamp')<self.getTimestamp(): self.setTimestamp(x.getAttr('stamp'))
+                if d.getAttr('stamp')<self.getTimestamp2(): self.setTimestamp(d.getAttr('stamp'))
             except: pass
+        if not self.timestamp:
+            for x in self.getTags('x',namespace=NS_DELAY):
+                try:
+                    if x.getAttr('stamp')<self.getTimestamp(): self.setTimestamp(x.getAttr('stamp'))
+                except: pass
         if timestamp is not None: self.setTimestamp(timestamp)  # To auto-timestamp stanza just pass timestamp=''
     def getTo(self):
         """ Return value of the 'to' attribute. """
@@ -326,6 +346,10 @@ class Protocol(Node):
         """ Return the timestamp in the 'yyyymmddThhmmss' format. """
         if self.timestamp: return self.timestamp
         return time.strftime('%Y%m%dT%H:%M:%S', time.gmtime())
+    def getTimestamp2(self):
+        """ Return the timestamp in the 'yyyymmddThhmmss' format. """
+        if self.timestamp: return self.timestamp
+        return time.strftime('%Y-%m-%dT%H:%M:%SZ', time.gmtime())
     def getID(self):
         """ Return the value of the 'id' attribute. """
         return self.getAttr('id')
@@ -349,7 +373,7 @@ class Protocol(Node):
         errtag=self.getTag('error')
         if errtag:
             for tag in errtag.getChildren():
-                if tag.getName()<>'text': return tag.getName()
+                if tag.getName()!='text': return tag.getName()
             return errtag.getData()
     def getErrorMsg(self):
         """ Return the textual description of the error (if present) or the error condition """
@@ -366,7 +390,7 @@ class Protocol(Node):
         if code:
             if str(code) in _errorcodes.keys(): error=ErrorNode(_errorcodes[str(code)],text=error)
             else: error=ErrorNode(ERR_UNDEFINED_CONDITION,code=code,typ='cancel',text=error)
-        elif type(error) in [type(''),type(u'')]: error=ErrorNode(error)
+        elif isinstance(error, basestring): error=ErrorNode(error)
         self.setType('error')
         self.addChild(node=error)
     def setTimestamp(self,val=None):
@@ -395,7 +419,7 @@ class Message(Protocol):
         Protocol.__init__(self, 'message', to=to, typ=typ, attrs=attrs, frm=frm, payload=payload, timestamp=timestamp, xmlns=xmlns, node=node)
         if body: self.setBody(body)
         if xhtml: self.setXHTML(xhtml)
-        if subject != None: self.setSubject(subject)
+        if subject is not None: self.setSubject(subject)
     def getBody(self):
         """ Returns text of the message. """
         return self.getTagData('body')
@@ -449,6 +473,14 @@ class Message(Protocol):
         th=self.getThread()
         if th: m.setThread(th)
         return m
+    def getStatusCode(self):
+        """Returns the status code of the message (for groupchat config
+        change)"""
+        attrs = []
+        for xtag in self.getTags('x'):
+            for child in xtag.getTags('status'):
+                attrs.append(child.getAttr('code'))
+        return attrs
 
 class Presence(Protocol):
     """ XMPP Presence object."""
@@ -481,10 +513,14 @@ class Presence(Protocol):
 
     def _muc_getItemAttr(self,tag,attr):
         for xtag in self.getTags('x'):
+            if xtag.getNamespace() not in (NS_MUC_USER, NS_MUC_ADMIN):
+                continue
             for child in xtag.getTags(tag):
                 return child.getAttr(attr)
     def _muc_getSubTagDataAttr(self,tag,attr):
         for xtag in self.getTags('x'):
+            if xtag.getNamespace() not in (NS_MUC_USER, NS_MUC_ADMIN):
+                continue
             for child in xtag.getTags('item'):
                 for cchild in child.getTags(tag):
                     return cchild.getData(),cchild.getAttr(attr)
@@ -563,7 +599,7 @@ class ErrorNode(Node):
         """ Create new error node object.
             Mandatory parameter: name - name of error condition.
             Optional parameters: code, typ, text. Used for backwards compartibility with older jabber protocol."""
-        if ERRORS.has_key(name):
+        if name in ERRORS:
             cod,type,txt=ERRORS[name]
             ns=name.split()[0]
         else: cod,ns,type,txt='500',NS_STANZAS,'cancel',''
@@ -603,7 +639,7 @@ class DataField(Node):
             """
         Node.__init__(self,'field',node=node)
         if name: self.setVar(name)
-        if type(value) in [list,tuple]: self.setValues(value)
+        if isinstance(value, (list, tuple)): self.setValues(value)
         elif value: self.setValue(value)
         if typ: self.setType(typ)
         elif not typ and not node: self.setType('text-single')
@@ -654,7 +690,7 @@ class DataField(Node):
         for opt in lst: self.addOption(opt)
     def addOption(self,opt):
         """ Add one more label-option pair to this field."""
-        if type(opt) in [str,unicode]: self.addChild('option').setTagData('value',opt)
+        if isinstance(opt, basestring): self.addChild('option').setTagData('value',opt)
         else: self.addChild('option',{'label':opt[0]}).setTagData('value',opt[1])
     def getType(self):
         """ Get type of this field. """
@@ -697,12 +733,12 @@ class DataForm(Node):
         if typ: self.setType(typ)
         self.setNamespace(NS_DATA)
         if title: self.setTitle(title)
-        if type(data)==type({}):
+        if isinstance(data, dict):
             newdata=[]
             for name in data.keys(): newdata.append(DataField(name,data[name]))
             data=newdata
         for child in data:
-            if type(child) in [type(''),type(u'')]: self.addInstructions(child)
+            if isinstance(child, basestring): self.addInstructions(child)
             elif child.__class__.__name__=='DataField': self.kids.append(child)
             else: self.kids.append(DataField(node=child))
     def getType(self):
@@ -740,7 +776,7 @@ class DataForm(Node):
         for field in self.getTags('field'):
             name=field.getAttr('var')
             typ=field.getType()
-            if type(typ) in [type(''),type(u'')] and typ[-6:]=='-multi':
+            if isinstance(typ, basestring) and typ.endswith('-multi'):
                 val=[]
                 for i in field.getTags('value'): val.append(i.getData())
             else: val=field.getTagData('value')
@@ -755,3 +791,5 @@ class DataForm(Node):
     def __setitem__(self,name,val):
         """ Simple dictionary interface for setting datafields values by their names."""
         return self.setField(name).setValue(val)
+
+# vim: se ts=3:
