@@ -27,10 +27,10 @@ def XMLescape(txt):
 ENCODING='utf-8'
 def ustr(what):
 	"""Converts object "what" to unicode string using it's own __str__ method if accessible or unicode method otherwise."""
-	if type(what) == type(u''): return what
+	if isinstance(what, unicode): return what
 	try: r=what.__str__()
 	except AttributeError: r=str(what)
-	if type(r)<>type(u''): return unicode(r,ENCODING)
+	if not isinstance(r, unicode): return unicode(r,ENCODING)
 	return r
 
 class Node(object):
@@ -49,7 +49,7 @@ class Node(object):
 		replication (and using replication only to move upwards on the classes tree).
 	"""
 	FORCE_NODE_RECREATION=0
-	def __init__(self, tag=None, attrs={}, payload=[], parent=None, node=None):
+	def __init__(self, tag=None, attrs={}, payload=[], parent=None, nsp=None, node_built=False, node=None):
 		""" Takes "tag" argument as the name of node (prepended by namespace, if needed and separated from it
 			by a space), attrs dictionary as the set of arguments, payload list as the set of textual strings
 			and child nodes that this node carries within itself and "parent" argument that is another node
@@ -62,22 +62,50 @@ class Node(object):
 				node=str(node)
 			if not isinstance(node, Node): 
 				node=NodeBuilder(node,self)
+				node_built = True
 			else:
-				self.name,self.namespace,self.attrs,self.data,self.kids,self.parent = node.name,node.namespace,{},[],[],node.parent
+				self.name,self.namespace,self.attrs,self.data,self.kids,self.parent,self.nsd = node.name,node.namespace,{},[],[],node.parent,{}
 				for key  in node.attrs.keys(): self.attrs[key]=node.attrs[key]
 				for data in node.data: self.data.append(data)
 				for kid  in node.kids: self.kids.append(kid)
-		else: self.name,self.namespace,self.attrs,self.data,self.kids,self.parent = 'tag','',{},[],[],None
-
-		if tag: self.namespace, self.name = ([self.namespace]+tag.split())[-2:]
-		if parent: self.parent = parent
-		if self.parent and not self.namespace: self.namespace=self.parent.namespace
-		for attr in attrs.keys():
+				for k,v in node.nsd.items(): self.nsd[k] = v
+		else: self.name,self.namespace,self.attrs,self.data,self.kids,self.parent,self.nsd = 'tag','',{},[],[],None,{}
+		if parent:
+			self.parent = parent
+		self.nsp_cache = {}
+		if nsp:
+			for k,v in nsp.items(): self.nsp_cache[k] = v
+		for attr,val in attrs.items():
+			if attr == 'xmlns':
+				self.nsd[u''] = val
+			elif attr.startswith('xmlns:'):
+				self.nsd[attr[6:]] = val
 			self.attrs[attr]=attrs[attr]
+		if tag:
+			if node_built:
+				pfx,self.name = (['']+tag.split(':'))[-2:]
+				self.namespace = self.lookup_nsp(pfx)
+			else:
+				if ' ' in tag:
+					self.namespace,self.name = tag.split()
+				else:
+					self.name = tag
 		if isinstance(payload, basestring): payload=[payload]
 		for i in payload:
 			if isinstance(i, Node): self.addChild(node=i)
 			else: self.data.append(ustr(i))
+	
+	def lookup_nsp(self,pfx=''):
+		ns = self.nsd.get(pfx,None)
+		if ns is None:
+			ns = self.nsp_cache.get(pfx,None)
+		if ns is None:
+			if self.parent:
+				ns = self.parent.lookup_nsp(pfx)
+				self.nsp_cache[pfx] = ns
+			else:
+				return 'http://www.gajim.org/xmlns/undeclared'
+		return ns
 
 	def __str__(self,fancy=0):
 		""" Method used to dump node into textual representation.
@@ -85,7 +113,8 @@ class Node(object):
 		s = (fancy-1) * 2 * ' ' + "<" + self.name
 		if self.namespace:
 			if not self.parent or self.parent.namespace!=self.namespace:
-				s = s + ' xmlns="%s"'%self.namespace
+				if 'xmlns' not in self.attrs:
+					s = s + ' xmlns="%s"'%self.namespace
 		for key in self.attrs.keys():
 			val = ustr(self.attrs[key])
 			s = s + ' %s="%s"' % ( key, XMLescape(val) )
@@ -100,7 +129,7 @@ class Node(object):
 				cnt=cnt+1
 		if not fancy and (len(self.data)-1) >= cnt: s = s + XMLescape(self.data[cnt])
 		elif (len(self.data)-1) >= cnt: s = s + XMLescape(self.data[cnt].strip())
-		if not self.kids and s[-1:]=='>':
+		if not self.kids and s.endswith('>'):
 			s=s[:-1]+' />'
 			if fancy: s = s + "\n"
 		else:
@@ -111,11 +140,12 @@ class Node(object):
 	def addChild(self, name=None, attrs={}, payload=[], namespace=None, node=None):
 		""" If "node" argument is provided, adds it as child node. Else creates new node from
 			the other arguments' values and adds it as well."""
-		if namespace: name=namespace+' '+name
 		if node:
 			newnode=node
 			node.parent = self
 		else: newnode=Node(tag=name, parent=self, attrs=attrs, payload=payload)
+		if namespace:
+			newnode.setNamespace(namespace)
 		self.kids.append(newnode)
 		return newnode
 	def addData(self, data):
@@ -184,10 +214,10 @@ class Node(object):
 			Returns the list of nodes found. """
 		nodes=[]
 		for node in self.kids:
-			if namespace and namespace<>node.getNamespace(): continue
+			if namespace and namespace!=node.getNamespace(): continue
 			if node.getName() == name:
 				for key in attrs.keys():
-				   if not node.attrs.has_key(key) or node.attrs[key]<>attrs[key]: break
+				   if key not in node.attrs or node.attrs[key]!=attrs[key]: break
 				else: nodes.append(node)
 			if one and nodes: return nodes[0]
 		if not one: return nodes
@@ -198,7 +228,7 @@ class Node(object):
 			if namespace is not None and namespace!=node.getNamespace(): continue
 			if node.getName() == name:
 				for key in attrs.keys():
-					if not node.attrs.has_key(key) or \
+					if key not in node.attrs or \
 						node.attrs[key]!=attrs[key]: break
 				else:
 					yield node
@@ -222,7 +252,7 @@ class Node(object):
 	def setPayload(self,payload,add=0):
 		""" Sets node payload according to the list specified. WARNING: completely replaces all node's
 			previous content. If you wish just to add child or CDATA - use addData or addChild methods. """
-		if type(payload) in (type(''),type(u'')): payload=[payload]
+		if isinstance(payload, basestring): payload=[payload]
 		if add: self.kids+=payload
 		else: self.kids=payload
 	def setTag(self, name, attrs={}, namespace=None):
@@ -243,7 +273,7 @@ class Node(object):
 		except: self.addChild(tag,attrs,payload=[ustr(val)])
 	def has_attr(self,key):
 		""" Checks if node have attribute "key"."""
-		return self.attrs.has_key(key)
+		return key in self.attrs
 	def __getitem__(self,item):
 		""" Returns node's attribute "item" value. """
 		return self.getAttr(item)
@@ -294,26 +324,26 @@ class NodeBuilder:
 			"data" (if provided) feeded to parser immidiatedly after instance init.
 			"""
 		self.DEBUG(DBG_NODEBUILDER, "Preparing to handle incoming XML stream.", 'start')
-		self._parser = xml.parsers.expat.ParserCreate(namespace_separator=' ')
+	
+		self._parser = xml.parsers.expat.ParserCreate()
 		self._parser.StartElementHandler       = self.starttag
 		self._parser.EndElementHandler         = self.endtag
 		self._parser.StartNamespaceDeclHandler = self.handle_namespace_start
 		self._parser.CharacterDataHandler    = self.handle_cdata
+		self._parser.buffer_text = True
 		self.Parse = self._parser.Parse
-
+		
 		self.__depth = 0
 		self.__last_depth = 0
 		self.__max_depth = 0
 		self._dispatch_depth = 1
 		self._document_attrs = None
+		self._document_nsp = None
 		self._mini_dom=initial_node
 		self.last_is_data = 1
 		self._ptr=None
 		self.data_buffer = None
-		self.namespaces={"http://www.w3.org/XML/1998/namespace":'xml:'}
-		self.xmlns="http://www.w3.org/XML/1998/namespace"
-
-		if data: 
+		if data:
 			self._parser.Parse(data,1)
 	
 	def check_data_buffer(self):
@@ -333,28 +363,34 @@ class NodeBuilder:
 	def starttag(self, tag, attrs):
 		"""XML Parser callback. Used internally"""
 		self.check_data_buffer()
-		attlist=attrs.keys()       #
-		for attr in attlist:       # FIXME: Crude hack. And it also slows down the whole library considerably.
-			sp=attr.rfind(" ")     #
-			if sp==-1: continue    #
-			ns=attr[:sp]           #
-			attrs[self.namespaces[ns]+attr[sp+1:]]=attrs[attr]
-			del attrs[attr]        #
 		self._inc_depth()
-		self.DEBUG(DBG_NODEBUILDER, "DEPTH -> %i , tag -> %s, attrs -> %s" % (self.__depth, tag, `attrs`), 'down')
+		self.DEBUG(DBG_NODEBUILDER, "DEPTH -> %i , tag -> %s, attrs -> %s" % (self.__depth, tag, repr(attrs)), 'down')
 		if self.__depth == self._dispatch_depth:
 			if not self._mini_dom : 
-				self._mini_dom = Node(tag=tag, attrs=attrs)
+				self._mini_dom = Node(tag=tag, attrs=attrs, nsp = self._document_nsp, node_built=True)
 			else: 
-				Node.__init__(self._mini_dom,tag=tag, attrs=attrs)
+				Node.__init__(self._mini_dom,tag=tag, attrs=attrs, nsp = self._document_nsp, node_built=True)
 			self._ptr = self._mini_dom
 		elif self.__depth > self._dispatch_depth:
-			self._ptr.kids.append(Node(tag=tag,parent=self._ptr,attrs=attrs))
+			self._ptr.kids.append(Node(tag=tag,parent=self._ptr,attrs=attrs, node_built=True))
 			self._ptr = self._ptr.kids[-1]
 		if self.__depth == 1:
-			self._document_attrs = attrs
-			ns, name = (['']+tag.split())[-2:]
-			self.stream_header_received(ns, name, attrs)
+			self._document_attrs = {}
+			self._document_nsp = {}
+			nsp, name = (['']+tag.split(':'))[-2:]
+			for attr,val in attrs.items():
+				if attr == 'xmlns':
+					self._document_nsp[u''] = val
+				elif attr.startswith('xmlns:'):
+					self._document_nsp[attr[6:]] = val
+				else:
+					self._document_attrs[attr] = val
+			ns = self._document_nsp.get(nsp, 'http://www.gajim.org/xmlns/undeclared-root')
+			try:
+				self.stream_header_received(ns, name, attrs)
+			except ValueError, e:
+				self._document_attrs = None
+				raise ValueError(str(e))
 		if not self.last_is_data and self._ptr.parent: 
 			self._ptr.parent.data.append('')
 		self.last_is_data = 0
@@ -383,8 +419,6 @@ class NodeBuilder:
 	def handle_namespace_start(self, prefix, uri):
 		"""XML Parser callback. Used internally"""
 		self.check_data_buffer()
-		if prefix: self.namespaces[uri]=prefix+':'
-		else: self.xmlns=uri
 	def DEBUG(self, level, text, comment=None):
 		""" Gets all NodeBuilder walking events. Can be used for debugging if redefined."""
 	def getDom(self):
@@ -424,3 +458,5 @@ def BadXML2Node(xml):
 		I.e. "<html>some text <br>some more text". Will raise xml.parser.expat.parsererror on misplaced
 		tags though. F.e. "<b>some text <br>some more text</b>" will not work."""
 	return NodeBuilder(xml).getDom()
+
+# vim: se ts=3:
