@@ -3,15 +3,20 @@
 ## Copyright (C) 2006 Stefan Bethge <stefan@lanpartei.de>
 ## 				2006 Dimitur Kirov <dkirov@gmail.com>
 ##
-## This program is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## This file is part of Gajim.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published
+## by the Free Software Foundation; version 3 only.
+##
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+##
 from common import gajim
 import common.xmpp
 from common.xmpp.idlequeue import IdleObject
@@ -20,16 +25,19 @@ from common.xmpp.client import *
 from common.xmpp.simplexml import ustr
 from common.zeroconf import zeroconf
 
-from  common.xmpp.protocol import *
+from common.xmpp.protocol import *
 import socket
 import errno
 import sys
 
+import logging
+log = logging.getLogger('gajim.c.z.client_zeroconf')
+
 from common.zeroconf import roster_zeroconf
 
 MAX_BUFF_LEN = 65536
-DATA_RECEIVED='DATA RECEIVED'
-DATA_SENT='DATA SENT'
+DATA_RECEIVED = 'DATA RECEIVED'
+DATA_SENT = 'DATA SENT'
 TYPE_SERVER, TYPE_CLIENT = range(2)
 
 # wait XX sec to establish a connection
@@ -42,22 +50,27 @@ class ZeroconfListener(IdleObject):
 	def __init__(self, port, conn_holder):
 		''' handle all incomming connections on ('0.0.0.0', port)'''
 		self.port = port
-		self.queue_idx = -1	
+		self.queue_idx = -1
 		#~ self.queue = None
 		self.started = False
 		self._sock = None
 		self.fd = -1
 		self.caller = conn_holder.caller
 		self.conn_holder = conn_holder
-		
+
 	def bind(self):
-		self._serv = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+		flags = socket.AI_PASSIVE
+		if hasattr(socket, 'AI_ADDRCONFIG'):
+			flags |= socket.AI_ADDRCONFIG
+		ai = socket.getaddrinfo(None, self.port, socket.AF_UNSPEC,
+			socket.SOCK_STREAM, 0, flags)[0]
+		self._serv = socket.socket(ai[0], ai[1])
 		self._serv.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
 		self._serv.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
 		self._serv.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
 		# will fail when port is busy, or we don't have rights to bind
 		try:
-			self._serv.bind(('0.0.0.0', self.port))
+			self._serv.bind((ai[4][0], self.port))
 		except Exception, e:
 			# unable to bind, show error dialog
 			return None
@@ -66,25 +79,24 @@ class ZeroconfListener(IdleObject):
 		self.fd = self._serv.fileno()
 		gajim.idlequeue.plug_idle(self, False, True)
 		self.started = True
-	
+
 	def pollend(self):
 		''' called when we stop listening on (host, port) '''
 		self.disconnect()
-	
+
 	def pollin(self):
 		''' accept a new incomming connection and notify queue'''
 		sock = self.accept_conn()
 		''' loop through roster to find who has connected to us'''
 		from_jid = None
-		nameinfo = socket.getnameinfo(sock[1], 0)
-		ipaddr = socket.gethostbyname(nameinfo[0])
+		ipaddr = sock[1][0]
 		for jid in self.conn_holder.getRoster().keys():
 			entry = self.conn_holder.getRoster().getItem(jid)
 			if (entry['address'] == ipaddr):
 				from_jid = jid
 				break;
-		P2PClient(sock[0], sock[1][0], sock[1][1], self.conn_holder, [], from_jid)
-	
+		P2PClient(sock[0], ipaddr, sock[1][1], self.conn_holder, [], from_jid)
+
 	def disconnect(self):
 		''' free all resources, we are not listening anymore '''
 		gajim.idlequeue.remove_timeout(self.fd)
@@ -96,15 +108,16 @@ class ZeroconfListener(IdleObject):
 		except:
 			pass
 		self.conn_holder.kill_all_connections()
-	
+
 	def accept_conn(self):
 		''' accepts a new incoming connection '''
-		_sock  = self._serv.accept()
+		_sock = self._serv.accept()
 		_sock[0].setblocking(False)
 		return _sock
 
 class P2PClient(IdleObject):
-	def __init__(self, _sock, host, port, conn_holder, stanzaqueue = [], to = None):
+	def __init__(self, _sock, host, port, conn_holder, stanzaqueue=[], to=None,
+	on_ok=None, on_not_ok=None):
 		self._owner = self
 		self.Namespace = 'jabber:client'
 		self.defaultNamespace = self.Namespace
@@ -115,6 +128,8 @@ class P2PClient(IdleObject):
 		self.stanzaqueue = stanzaqueue
 		self.to = to
 		self.Server = host
+		self.on_ok = on_ok
+		self.on_not_ok = on_not_ok
 		self.DBG = 'client'
 		self.Connection = None
 		if gajim.verbose:
@@ -131,7 +146,13 @@ class P2PClient(IdleObject):
 		else:
 			self.sock_type = TYPE_CLIENT
 		self.fd = -1
-		conn = P2PConnection('', _sock, host, port, self._caller, self.on_connect, self)
+		conn = P2PConnection('', _sock, host, port, self._caller, self.on_connect,
+			self)
+		if not self.conn_holder:
+			# An error occured, disconnect() has been called
+			if on_not_ok:
+				on_not_ok('Connection to host could not be established.')
+			return
 		self.sock_hash = conn._sock.__hash__
 		self.fd = conn.fd
 		self.conn_holder.add_connection(self, self.Server, port, self.to)
@@ -140,15 +161,15 @@ class P2PClient(IdleObject):
 			stanza, is_message = val
 			if is_message:
 				if self.fd == -1:
-					self._caller.dispatch('MSGERROR',[unicode(self.to), -1, \
-                                        _('Connection to host could not be established'), None, None])
+					if on_not_ok:
+						on_not_ok('Connection to host could not be established.')
+					return
+				if self.conn_holder.number_of_awaiting_messages.has_key(self.fd):
+					self.conn_holder.number_of_awaiting_messages[self.fd] += 1
 				else:
-					if self.conn_holder.number_of_awaiting_messages.has_key(self.fd):
-						self.conn_holder.number_of_awaiting_messages[self.fd]+=1
-					else:
-						self.conn_holder.number_of_awaiting_messages[self.fd]=1
-	
-	def add_stanza(self, stanza, is_message = False):
+					self.conn_holder.number_of_awaiting_messages[self.fd] = 1
+
+	def add_stanza(self, stanza, is_message=False):
 		if self.Connection:
 			if self.Connection.state == -1:
 				return False
@@ -158,20 +179,22 @@ class P2PClient(IdleObject):
 
 		if is_message:
 			if self.conn_holder.number_of_awaiting_messages.has_key(self.fd):
-				self.conn_holder.number_of_awaiting_messages[self.fd]+=1
+				self.conn_holder.number_of_awaiting_messages[self.fd] += 1
 			else:
 				self.conn_holder.number_of_awaiting_messages[self.fd] = 1
 
 		return True
-	
+
 	def on_message_sent(self, connection_id):
-		self.conn_holder.number_of_awaiting_messages[connection_id]-=1
+		self.conn_holder.number_of_awaiting_messages[connection_id] -= 1
 
 	def on_connect(self, conn):
 		self.Connection = conn
 		self.Connection.PlugIn(self)
 		dispatcher_nb.Dispatcher().PlugIn(self)
 		self._register_handlers()
+		if self.on_ok:
+			self.on_ok()
 
 	def StreamInit(self):
 		''' Send an initial stream header. '''
@@ -184,7 +207,7 @@ class P2PClient(IdleObject):
 		self.Dispatcher.Stream.features = None
 		if self.sock_type == TYPE_CLIENT:
 			self.send_stream_header()
-	
+
 	def send_stream_header(self):
 		self.Dispatcher._metastream = Node('stream:stream')
 		self.Dispatcher._metastream.setNamespace(self.Namespace)
@@ -193,14 +216,16 @@ class P2PClient(IdleObject):
 		self.Dispatcher._metastream.setAttr('from', self.conn_holder.zeroconf.name)
 		if self.to:
 			self.Dispatcher._metastream.setAttr('to', self.to)
-		self.Dispatcher.send("<?xml version='1.0'?>%s>" % str(self.Dispatcher._metastream)[:-2])
-	
+		self.Dispatcher.send("<?xml version='1.0'?>%s>" % str(
+			self.Dispatcher._metastream)[:-2])
+
 	def _check_stream_start(self, ns, tag, attrs):
 		if ns<>NS_STREAMS or tag<>'stream':
-			self._caller.dispatch('MSGERROR',[unicode(self.to), -1, \
-				_('Connection to host could not be established: Incorrect answer from server.'), None, None])
-			self.Connection.DEBUG('Incorrect stream start: (%s,%s).Terminating! ' % (tag, ns), 'error')
+			self.Connection.DEBUG('Incorrect stream start: (%s,%s).Terminating! ' \
+				% (tag, ns), 'error')
 			self.Connection.disconnect()
+			if self.on_not_ok:
+				self.on_not_ok('Connection to host could not be established: Incorrect answer from server.')
 			return
 		if self.sock_type == TYPE_SERVER:
 			if attrs.has_key('from'):
@@ -218,15 +243,11 @@ class P2PClient(IdleObject):
 				stanza, is_message = self.stanzaqueue.pop(0)
 				self.send(stanza, is_message)
 
-	
 	def on_disconnect(self):
 		if self.conn_holder:
 			if self.conn_holder.number_of_awaiting_messages.has_key(self.fd):
-				if self.conn_holder.number_of_awaiting_messages[self.fd] > 0:
-					self._caller.dispatch('MSGERROR',[unicode(self.to), -1, \
-					_('Connection to host could not be established'), None, None])
 				del self.conn_holder.number_of_awaiting_messages[self.fd]
-			self.conn_holder.remove_connection(self.sock_hash) 
+			self.conn_holder.remove_connection(self.sock_hash)
 		if self.__dict__.has_key('Dispatcher'):
 			self.Dispatcher.PlugOut()
 		if self.__dict__.has_key('P2PConnection'):
@@ -234,13 +255,13 @@ class P2PClient(IdleObject):
 		self.Connection = None
 		self._caller = None
 		self.conn_holder = None
-	
+
 	def force_disconnect(self):
 		if self.Connection:
 			self.disconnect()
 		else:
 			self.on_disconnect()
-		
+
 	def _on_receive_document_attrs(self, data):
 		if data:
 			self.Dispatcher.ProcessNonBlocking(data)
@@ -249,15 +270,16 @@ class P2PClient(IdleObject):
 			return
 		self.onreceive(None)
 		if self.Dispatcher.Stream._document_attrs.has_key('version') and \
-			self.Dispatcher.Stream._document_attrs['version'] == '1.0':
+		self.Dispatcher.Stream._document_attrs['version'] == '1.0':
 				#~ self.onreceive(self._on_receive_stream_features)
 				#XXX continue with TLS
 				return
 		self.onreceive(None)
 		return True
-		
+
 	def _register_handlers(self):
-		self.RegisterHandler('message', lambda conn, data:self._caller._messageCB(self.Server, conn, data))
+		self.RegisterHandler('message', lambda conn, data:self._caller._messageCB(
+			self.Server, conn, data))
 		self.RegisterHandler('iq', self._caller._siSetCB, 'set',
 			common.xmpp.NS_SI)
 		self.RegisterHandler('iq', self._caller._siErrorCB, 'error',
@@ -270,13 +292,14 @@ class P2PClient(IdleObject):
 			common.xmpp.NS_BYTESTREAM)
 		self.RegisterHandler('iq', self._caller._bytestreamErrorCB, 'error',
 			common.xmpp.NS_BYTESTREAM)
-	
+
 class P2PConnection(IdleObject, PlugIn):
-	def __init__(self, sock_hash, _sock, host = None, port = None, caller = None, on_connect = None, client = None):
+	def __init__(self, sock_hash, _sock, host=None, port=None, caller=None,
+	on_connect=None, client=None):
 		IdleObject.__init__(self)
 		self._owner = client
 		PlugIn.__init__(self)
-		self.DBG_LINE='socket'
+		self.DBG_LINE = 'socket'
 		self.sendqueue = []
 		self.sendbuff = None
 		self.buff_is_message = False
@@ -287,23 +310,47 @@ class P2PConnection(IdleObject, PlugIn):
 		self.client = client
 		self.writable = False
 		self.readable = False
-		self._exported_methods=[self.send, self.disconnect, self.onreceive]
+		self._exported_methods = [self.send, self.disconnect, self.onreceive]
 		self.on_receive = None
 		if _sock:
 			self._sock = _sock
-			self.state = 1 
+			self.state = 1
 			self._sock.setblocking(False)
 			self.fd = self._sock.fileno()
 			self.on_connect(self)
 		else:
 			self.state = 0
-			self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+			try:
+				self.ais = socket.getaddrinfo(host, port, socket.AF_UNSPEC,
+					socket.SOCK_STREAM)
+			except socket.gaierror, e:
+				log.info('Lookup failure for %s: %s[%s]', host, e[1], repr(e[0]),
+					exc_info=True)
+			else:
+				self.connect_to_next_ip()
+
+	def connect_to_next_ip(self):
+		if len(self.ais) == 0:
+			log.error('Connection failure to %s', self.host, exc_info=True)
+			self.disconnect()
+			return
+		ai = self.ais.pop(0)
+		log.info('Trying to connect to %s through %s:%s', self.host, ai[4][0],
+			ai[4][1], exc_info=True)
+		try:
+			self._sock = socket.socket(*ai[:3])
 			self._sock.setblocking(False)
-			self.fd = self._sock.fileno()
-			gajim.idlequeue.plug_idle(self, True, False)
-			self.set_timeout(CONNECT_TIMEOUT_SECONDS)
-			self.do_connect()
-	
+			self._server = ai[4]
+		except:
+			if sys.exc_value[0] != errno.EINPROGRESS:
+				# for all errors, we try other addresses
+				self.connect_to_next_ip()
+				return
+		self.fd = self._sock.fileno()
+		gajim.idlequeue.plug_idle(self, True, False)
+		self.set_timeout(CONNECT_TIMEOUT_SECONDS)
+		self.do_connect()
+
 	def set_timeout(self, timeout):
 		gajim.idlequeue.remove_timeout(self.fd)
 		if self.state >= 0:
@@ -315,8 +362,8 @@ class P2PConnection(IdleObject, PlugIn):
 		return True
 
 	def plugout(self):
-		''' Disconnect from the remote server and unregister self.disconnected method from
-			the owner's dispatcher. '''
+		'''Disconnect from the remote server and unregister self.disconnected method from
+			the owner's dispatcher.'''
 		self.disconnect()
 		self._owner = None
 
@@ -331,10 +378,10 @@ class P2PConnection(IdleObject, PlugIn):
 		# make sure this cb is not overriden by recursive calls
 		if not recv_handler(None) and _tmp == self.on_receive:
 			self.on_receive = recv_handler
-		
-	def send(self, packet, is_message = False, now = False):
+
+	def send(self, packet, is_message=False, now=False):
 		'''Append stanza to the queue of messages to be send if now is
-		False, else send it instantly. 
+		False, else send it instantly.
 		If supplied data is unicode string, encode it to utf-8.
 		'''
 		if self.state <= 0:
@@ -342,9 +389,9 @@ class P2PConnection(IdleObject, PlugIn):
 
 		r = packet
 
-		if isinstance(r, unicode): 
+		if isinstance(r, unicode):
 			r = r.encode('utf-8')
-		elif not isinstance(r, str): 
+		elif not isinstance(r, str):
 			r = ustr(r).encode('utf-8')
 
 		if now:
@@ -353,19 +400,19 @@ class P2PConnection(IdleObject, PlugIn):
 		else:
 			self.sendqueue.append((r, is_message))
 		self._plug_idle()
-		
+
 	def read_timeout(self):
 		if self.client.conn_holder.number_of_awaiting_messages.has_key(self.fd) \
-				and self.client.conn_holder.number_of_awaiting_messages[self.fd] > 0:
-			self.client._caller.dispatch('MSGERROR',[unicode(self.client.to), -1, \
-					_('Connection to host could not be established: Timeout while sending data.'), None, None])
-			self.client.conn_holder.number_of_awaiting_messages[self.fd] = 0 
+		and self.client.conn_holder.number_of_awaiting_messages[self.fd] > 0:
+			self.client._caller.dispatch('MSGERROR',[unicode(self.client.to), -1,
+				_('Connection to host could not be established: Timeout while sending data.'), None, None])
+			self.client.conn_holder.number_of_awaiting_messages[self.fd] = 0
 		self.pollend()
 
 	def do_connect(self):
 		errnum = 0
 		try:
-			self._sock.connect((self.host, self.port))
+			self._sock.connect(self._server)
 			self._sock.setblocking(False)
 		except Exception, ee:
 			(errnum, errstr) = ee
@@ -373,38 +420,38 @@ class P2PConnection(IdleObject, PlugIn):
 			return
 		# win32 needs this
 		elif errnum not in (0, 10056, errno.EISCONN) or self.state != 0:
-			self.disconnect()
-			return None
+			log.error('Could not connect to %s: %s [%s]', self.host, errnum,
+				errstr)
+			self.connect_to_next_ip()
+			return
 		else: # socket is already connected
 			self._sock.setblocking(False)
 		self.state = 1 # connected
+		# we are connected
 		self.on_connect(self)
-		return 1 # we are connected
-	
-	
+
 	def pollout(self):
 		if self.state == 0:
 			self.do_connect()
 			return
 		gajim.idlequeue.remove_timeout(self.fd)
 		self._do_send()
-	
+
 	def pollend(self):
 		self.state = -1
 		self.disconnect()
-	
+
 	def pollin(self):
 		''' Reads all pending incoming data. Calls owner's disconnected() method if appropriate.'''
 		received = ''
 		errnum = 0
-		try: 
+		try:
 			# get as many bites, as possible, but not more than RECV_BUFSIZE
 			received = self._sock.recv(MAX_BUFF_LEN)
 		except Exception, e:
-			if len(e.args)  > 0 and isinstance(e.args[0], int):
+			if len(e.args) > 0 and isinstance(e.args[0], int):
 				errnum = e[0]
-			sys.exc_clear()
-			# "received" will be empty anyhow 
+			# "received" will be empty anyhow
 		if errnum == socket.SSL_ERROR_WANT_READ:
 			pass
 		elif errnum in [errno.ECONNRESET, errno.ENOTCONN, errno.ESHUTDOWN]:
@@ -412,13 +459,13 @@ class P2PConnection(IdleObject, PlugIn):
 			# don't proccess result, cas it will raise error
 			return
 		elif not received :
-			if errnum != socket.SSL_ERROR_EOF: 
+			if errnum != socket.SSL_ERROR_EOF:
 				# 8 EOF occurred in violation of protocol
 				self.pollend()
 			if self.state >= 0:
 				self.disconnect()
 			return
-		
+
 		if self.state < 0:
 			return
 		if self.on_receive:
@@ -434,7 +481,7 @@ class P2PConnection(IdleObject, PlugIn):
 			self.DEBUG('Unhandled data received: %s' % received,'error')
 			self.disconnect()
 		return True
-	
+
 	def onreceive(self, recv_handler):
 		if not recv_handler:
 			if hasattr(self._owner, 'Dispatcher'):
@@ -446,7 +493,7 @@ class P2PConnection(IdleObject, PlugIn):
 		# make sure this cb is not overriden by recursive calls
 		if not recv_handler(None) and _tmp == self.on_receive:
 			self.on_receive = recv_handler
-	
+
 	def disconnect(self):
 		''' Closes the socket. '''
 		gajim.idlequeue.remove_timeout(self.fd)
@@ -478,14 +525,13 @@ class P2PConnection(IdleObject, PlugIn):
 						self._on_send()
 						self.disconnect()
 						return
-					# we are not waiting for write 
+					# we are not waiting for write
 					self._plug_idle()
 				self._on_send()
 
 		except socket.error, e:
-			sys.exc_clear()
 			if e[0] == socket.SSL_ERROR_WANT_WRITE:
-				return True		
+				return True
 			if self.state < 0:
 				self.disconnect()
 				return
@@ -494,7 +540,7 @@ class P2PConnection(IdleObject, PlugIn):
 		if self._owner.sock_type == TYPE_CLIENT:
 			self.set_timeout(ACTIVITY_TIMEOUT_SECONDS)
 		return True
-	
+
 	def _plug_idle(self):
 		readable = self.state != 0
 		if self.sendqueue or self.sendbuff:
@@ -503,8 +549,8 @@ class P2PConnection(IdleObject, PlugIn):
 			writable = False
 		if self.writable != writable or self.readable != readable:
 			gajim.idlequeue.plug_idle(self, writable, readable)
-	
-	
+
+
 	def _on_send(self):
 		if self.sent_data and self.sent_data.strip():
 			self.DEBUG(self.sent_data,'sent')
@@ -520,7 +566,6 @@ class P2PConnection(IdleObject, PlugIn):
 		self._owner.disconnected()
 		self.sent_data = None
 
-
 class ClientZeroconf:
 	def __init__(self, caller):
 		self.caller = caller
@@ -534,13 +579,6 @@ class ClientZeroconf:
 		self.listener = None
 		self.number_of_awaiting_messages = {}
 
-	def test_avahi(self):
-		try:
-			import avahi
-		except ImportError:
-			return False
-		return True
-
 	def connect(self, show, msg):
 		self.port = self.start_listener(self.caller.port)
 		if not self.port:
@@ -555,21 +593,21 @@ class ClientZeroconf:
 	def remove_announce(self):
 		if self.zeroconf:
 			return self.zeroconf.remove_announce()
-	
+
 	def announce(self):
 		if self.zeroconf:
 			return self.zeroconf.announce()
-	
+
 	def set_show_msg(self, show, msg):
 		if self.zeroconf:
 			self.zeroconf.txt['msg'] = msg
 			self.last_msg = msg
 			return self.zeroconf.update_txt(show)
-	
+
 	def resolve_all(self):
 		if self.zeroconf:
 			self.zeroconf.resolve_all()
-	
+
 	def reannounce(self, txt):
 		self.remove_announce()
 		self.zeroconf.txt = txt
@@ -577,11 +615,11 @@ class ClientZeroconf:
 		self.zeroconf.username = self.caller.username
 		return self.announce()
 
-
 	def zeroconf_init(self, show, msg):
 		self.zeroconf = zeroconf.Zeroconf(self.caller._on_new_service,
-			self.caller._on_remove_service, self.caller._on_name_conflictCB, 
-			self.caller._on_disconnected, self.caller._on_error, self.caller.username, self.caller.host, self.port)
+			self.caller._on_remove_service, self.caller._on_name_conflictCB,
+			self.caller._on_disconnected, self.caller._on_error,
+			self.caller.username, self.caller.host, self.port)
 		self.zeroconf.txt['msg'] = msg
 		self.zeroconf.txt['status'] = show
 		self.zeroconf.txt['1st'] = self.caller.first
@@ -604,20 +642,20 @@ class ClientZeroconf:
 			self.roster.zeroconf = None
 			self.roster._data = None
 			self.roster = None
-		
+
 	def kill_all_connections(self):
 		for connection in self.connections.values():
 			connection.force_disconnect()
-	
+
 	def add_connection(self, connection, ip, port, recipient):
-		sock_hash = connection.sock_hash
+		sock_hash=connection.sock_hash
 		if sock_hash not in self.connections:
 			self.connections[sock_hash] = connection
 		self.ip_to_hash[ip] = sock_hash
 		self.hash_to_port[sock_hash] = port
 		if recipient:
 			self.recipient_to_hash[recipient] = sock_hash
-		
+
 	def remove_connection(self, sock_hash):
 		if sock_hash in self.connections:
 			del self.connections[sock_hash]
@@ -640,34 +678,49 @@ class ClientZeroconf:
 				return p
 		self.listener = None
 		return False
-	
+
 	def getRoster(self):
 		if self.roster:
 			return self.roster.getRoster()
 		return {}
 
-	def send(self, stanza, is_message = False, now = False):
+	def send(self, stanza, is_message=False, now=False, on_ok=None,
+	on_not_ok=None):
 		stanza.setFrom(self.roster.zeroconf.name)
 		to = stanza.getTo()
-		
+
 		try:
 			item = self.roster[to]
 		except KeyError:
-			self.caller.dispatch('MSGERROR', [unicode(to), '-1', _('Contact is offline. Your message could not be sent.'), None, None])
-			return False
-	
+			# Contact offline
+			return -1
+
 		# look for hashed connections
 		if to in self.recipient_to_hash:
 			conn = self.connections[self.recipient_to_hash[to]]
+			id = conn.Dispatcher.getAnID()
+			stanza.setID(id)
 			if conn.add_stanza(stanza, is_message):
-				return
+				if on_ok:
+					on_ok()
+				return id
 
 		if item['address'] in self.ip_to_hash:
 			hash = self.ip_to_hash[item['address']]
 			if self.hash_to_port[hash] == item['port']:
 				conn = self.connections[hash]
+				id = conn.Dispatcher.getAnID()
+				stanza.setID(id)
 				if conn.add_stanza(stanza, is_message):
-					return
+					if on_ok:
+						on_ok()
+					return id
 
 		# otherwise open new connection
-		P2PClient(None, item['address'], item['port'], self, [(stanza, is_message)], to)
+		stanza.setID('zero')
+		P2PClient(None, item['address'], item['port'], self,
+			[(stanza, is_message)], to, on_ok=on_ok, on_not_ok=on_not_ok)
+
+		return 'zero'
+
+# vim: se ts=3:
