@@ -1,21 +1,25 @@
-##	filetransfers_window.py
+# -*- coding:utf-8 -*-
+## src/filetransfers_window.py
 ##
-## Copyright (C) 2003-2006 Yann Le Boulanger <asterix@lagaule.org>
-## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
-## Copyright (C) 2005
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-## Copyright (C) 2004-2005 Vincent Hanquez <tab@snarc.org>
+## Copyright (C) 2003-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2005-2006 Dimitur Kirov <dkirov AT gmail.com>
+## Copyright (C) 2005-2007 Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2006 Travis Shirk <travis AT pobox.com>
 ##
-## This program is free software; you can redistribute it and/or modify
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 import gtk
 import gobject
@@ -107,7 +111,17 @@ class FileTransfersWindow:
 		col.set_expand(False)
 		self.tree.append_column(col)
 
-		self.set_images()
+		self.images = {}
+		self.icons = {
+			'upload': gtk.STOCK_GO_UP,
+			'download': gtk.STOCK_GO_DOWN,
+			'stop': gtk.STOCK_STOP,
+			'waiting': gtk.STOCK_REFRESH,
+			'pause': gtk.STOCK_MEDIA_PAUSE,
+			'continue': gtk.STOCK_MEDIA_PLAY,
+			'ok': gtk.STOCK_APPLY,
+		}
+
 		self.tree.get_selection().set_mode(gtk.SELECTION_SINGLE)
 		self.tree.get_selection().connect('changed', self.selection_changed)
 		self.tooltip = tooltips.FileTransfersTooltip()
@@ -116,8 +130,6 @@ class FileTransfersWindow:
 		self.cancel_menuitem = self.xml.get_widget('cancel_menuitem')
 		self.pause_menuitem = self.xml.get_widget('pause_menuitem')
 		self.continue_menuitem = self.xml.get_widget('continue_menuitem')
-		self.continue_menuitem.hide()
-		self.continue_menuitem.set_no_show_all(True)
 		self.remove_menuitem = self.xml.get_widget('remove_menuitem')
 		self.xml.signal_autoconnect(self)
 
@@ -130,7 +142,7 @@ class FileTransfersWindow:
 			if file_props['tt_account'] == account:
 				receiver_jid = unicode(file_props['receiver']).split('/')[0]
 				if jid == receiver_jid:
-					if not self.is_transfer_stoped(file_props):
+					if not self.is_transfer_stopped(file_props):
 						active_transfers[0].append(file_props)
 
 		# 'account' is the recipient
@@ -138,7 +150,7 @@ class FileTransfersWindow:
 			if file_props['tt_account'] == account:
 				sender_jid = unicode(file_props['sender']).split('/')[0]
 				if jid == sender_jid:
-					if not self.is_transfer_stoped(file_props):
+					if not self.is_transfer_stopped(file_props):
 						active_transfers[1].append(file_props)
 		return active_transfers
 
@@ -146,7 +158,7 @@ class FileTransfersWindow:
 		''' show a dialog saying that file (file_props) has been transferred'''
 		def on_open(widget, file_props):
 			dialog.destroy()
-			if not file_props.has_key('file-name'):
+			if 'file-name' not in file_props:
 				return
 			(path, file) = os.path.split(file_props['file-name'])
 			if os.path.exists(path) and os.path.isdir(path):
@@ -221,13 +233,17 @@ _('Connection with peer cannot be established.'))
 		self.tree.get_selection().unselect_all()
 
 	def show_file_send_request(self, account, contact):
+	
+		desc_entry = gtk.Entry()
+	
 		def on_ok(widget):
 			file_dir = None
 			files_path_list = dialog.get_filenames()
 			files_path_list = gtkgui_helpers.decode_filechooser_file_paths(
 				files_path_list)
+			desc = desc_entry.get_text()
 			for file_path in files_path_list:
-				if self.send_file(account, contact, file_path) and file_dir is None:
+				if self.send_file(account, contact, file_path, desc) and file_dir is None:
 					file_dir = os.path.dirname(file_path)
 			if file_dir:
 				gajim.config.set('last_send_dir', file_dir)
@@ -247,9 +263,17 @@ _('Connection with peer cannot be established.'))
 		# FIXME: add send icon to this button (JUMP_TO)
 		dialog.add_action_widget(btn, gtk.RESPONSE_OK)
 		dialog.set_default_response(gtk.RESPONSE_OK)
+		
+		desc_hbox = gtk.HBox(False, 5)
+		desc_hbox.pack_start(gtk.Label(_('Description: ')),False,False,0)
+		desc_hbox.pack_start(desc_entry,True,True,0)
+		
+		dialog.vbox.pack_start(desc_hbox, False, False, 0)
+		
 		btn.show()
+		desc_hbox.show_all()
 
-	def send_file(self, account, contact, file_path):
+	def send_file(self, account, contact, file_path, file_desc=''):
 		''' start the real transfer(upload) of the file '''
 		if gtkgui_helpers.file_is_locked(file_path):
 			pritext = _('Gajim cannot access this file')
@@ -261,11 +285,10 @@ _('Connection with peer cannot be established.'))
 			if contact.find('/') == -1:
 				return
 			(jid, resource) = contact.split('/', 1)
-			contact = gajim.contacts.create_contact(jid = jid,
-				resource = resource)
+			contact = gajim.contacts.create_contact(jid=jid, resource=resource)
 		(file_dir, file_name) = os.path.split(file_path)
 		file_props = self.get_send_file_props(account, contact, 
-				file_path, file_name)
+				file_path, file_name, file_desc)
 		if file_props is None:
 			return False
 		self.add_transfer(account, contact, file_props)
@@ -283,21 +306,20 @@ _('Connection with peer cannot be established.'))
 	def show_file_request(self, account, contact, file_props):
 		''' show dialog asking for comfirmation and store location of new
 		file requested by a contact'''
-		if file_props is None or not file_props.has_key('name'):
+		if file_props is None or 'name' not in file_props:
 			return
 		sec_text = '\t' + _('File: %s') % file_props['name']
-		if file_props.has_key('size'):
+		if 'size' in file_props:
 			sec_text += '\n\t' + _('Size: %s') % \
 				helpers.convert_bytes(file_props['size'])
-		if file_props.has_key('mime-type'):
+		if 'mime-type' in file_props:
 			sec_text += '\n\t' + _('Type: %s') % file_props['mime-type']
-		if file_props.has_key('desc'):
+		if 'desc' in file_props:
 			sec_text += '\n\t' + _('Description: %s') % file_props['desc']
 		prim_text = _('%s wants to send you a file:') % contact.jid
 		dialog, dialog2 = None, None
 
-		def on_response_ok(widget, account, contact, file_props):
-			dialog.destroy()
+		def on_response_ok(account, contact, file_props):
 
 			def on_ok(widget, account, contact, file_props):
 				file_path = dialog2.get_filename()
@@ -314,16 +336,21 @@ _('Connection with peer cannot be established.'))
 					dl_size = stat.st_size
 					file_size = file_props['size']
 					dl_finished = dl_size >= file_size
+
+					def on_response(response):
+						if response < 0:
+							return
+						elif response == 100:
+							file_props['offset'] = dl_size
+						dialog2.destroy()
+						self._start_receive(file_path, account, contact, file_props)
+
 					dialog = dialogs.FTOverwriteConfirmationDialog(
 						_('This file already exists'), _('What do you want to do?'),
-						not dl_finished)
+						propose_resume=not dl_finished, on_response=on_response)
 					dialog.set_transient_for(dialog2)
 					dialog.set_destroy_with_parent(True)
-					response = dialog.get_response()
-					if response < 0:
-						return
-					elif response == 100:
-						file_props['offset'] = dl_size
+					return
 				else:
 					dirname = os.path.dirname(file_path)
 					if not os.access(dirname, os.W_OK) and os.name != 'nt':
@@ -352,8 +379,7 @@ _('Connection with peer cannot be established.'))
 			dialog2.connect('delete-event', lambda widget, event:
 				on_cancel(widget, account, contact, file_props))
 
-		def on_response_cancel(widget, account, file_props):
-			dialog.destroy()
+		def on_response_cancel(account, file_props):
 			gajim.connections[account].send_file_rejection(file_props)
 
 		dialog = dialogs.NonModalConfirmationDialog(prim_text, sec_text,
@@ -363,23 +389,9 @@ _('Connection with peer cannot be established.'))
 			on_response_cancel(widget, account, file_props))
 		dialog.popup()
 
-	def set_images(self):
-		''' create pixbufs for status images in transfer rows'''
-		self.images = {}
-		self.images['upload'] = self.window.render_icon(gtk.STOCK_GO_UP, 
-			gtk.ICON_SIZE_MENU)
-		self.images['download'] = self.window.render_icon(gtk.STOCK_GO_DOWN, 
-			gtk.ICON_SIZE_MENU)
-		self.images['stop'] = self.window.render_icon(gtk.STOCK_STOP, 
-			gtk.ICON_SIZE_MENU)
-		self.images['waiting'] = self.window.render_icon(gtk.STOCK_REFRESH, 
-			gtk.ICON_SIZE_MENU)
-		self.images['pause'] = self.window.render_icon(gtk.STOCK_MEDIA_PAUSE, 
-			gtk.ICON_SIZE_MENU)
-		self.images['continue'] = self.window.render_icon(gtk.STOCK_MEDIA_PLAY, 
-			gtk.ICON_SIZE_MENU)
-		self.images['ok'] = self.window.render_icon(gtk.STOCK_APPLY, 
-			gtk.ICON_SIZE_MENU)
+	def get_icon(self, ident):
+		return self.images.setdefault(ident,
+			self.window.render_icon(self.icons[ident], gtk.ICON_SIZE_MENU))
 
 	def set_status(self, typ, sid, status):
 		''' change the status of a transfer to state 'status' '''
@@ -392,7 +404,7 @@ _('Connection with peer cannot be established.'))
 			file_props['stopped'] = True
 		elif status == 'ok':
 			file_props['completed'] = True
-		self.model.set(iter, C_IMAGE, self.images[status])
+		self.model.set(iter, C_IMAGE, self.get_icon(status))
 		path = self.model.get_path(iter)
 		self.select_func(path)
 
@@ -422,22 +434,32 @@ _('Connection with peer cannot be established.'))
 		#they are not translatable.
 		return _('%(hours)02.d:%(minutes)02.d:%(seconds)02.d')  % times
 
-	def _get_eta_and_speed(self, full_size, transfered_size, elapsed_time):
-		if elapsed_time == 0:
+	def _get_eta_and_speed(self, full_size, transfered_size, file_props):
+		if len(file_props['transfered_size']) == 0:
 			return 0., 0.
-		speed = round(float(transfered_size) / elapsed_time)
+		elif len(file_props['transfered_size']) == 1:
+			speed = round(float(transfered_size) / file_props['elapsed-time'])
+		else:
+			# first and last are (time, transfered_size)
+			first = file_props['transfered_size'][0]
+			last = file_props['transfered_size'][-1]
+			transfered = last[1] - first[1]
+			tim = last[0] - first[0]
+			if tim == 0:
+				return 0., 0.
+			speed = round(float(transfered) / tim)
 		if speed == 0.:
 			return 0., 0.
 		remaining_size = full_size - transfered_size
 		eta = remaining_size / speed
 		return eta, speed
 
-	def _remove_transfer(self, iter, sid, file_props):
-		self.model.remove(iter)
-		if  file_props.has_key('tt_account'):
+	def _remove_transfer(self, iter_, sid, file_props):
+		self.model.remove(iter_)
+		if  'tt_account' in file_props:
 			# file transfer is set
 			account = file_props['tt_account']
-			if gajim.connections.has_key(account):
+			if account in gajim.connections:
 				# there is a connection to the account
 				gajim.connections[account].remove_transfer(file_props)
 			if file_props['type'] == 'r': # we receive a file
@@ -458,21 +480,21 @@ _('Connection with peer cannot be established.'))
 		del(self.files_props[sid[0]][sid[1:]])
 		del(file_props)
 
-	def set_progress(self, typ, sid, transfered_size, iter = None):
+	def set_progress(self, typ, sid, transfered_size, iter_ = None):
 		''' change the progress of a transfer with new transfered size'''
-		if not self.files_props[typ].has_key(sid):
+		if sid not in self.files_props[typ]:
 			return
 		file_props = self.files_props[typ][sid]
 		full_size = int(file_props['size'])
 		if full_size == 0:
 			percent = 0
 		else:
-			percent = round(float(transfered_size) / full_size * 100)
-		if iter is None:
-			iter = self.get_iter_by_sid(typ, sid)
-		if iter is not None:
+			percent = round(float(transfered_size) / full_size * 100, 1)
+		if iter_ is None:
+			iter_ = self.get_iter_by_sid(typ, sid)
+		if iter_ is not None:
 			just_began = False
-			if self.model[iter][C_PERCENT] == 0 and int(percent > 0):
+			if self.model[iter_][C_PERCENT] == 0 and int(percent > 0):
 				just_began = True
 			text = self._format_percent(percent)
 			if transfered_size == 0:
@@ -483,14 +505,19 @@ _('Connection with peer cannot be established.'))
 			# Kb/s
 
 			# remaining time
-			if file_props.has_key('offset') and file_props['offset']:
+			if 'offset' in file_props and file_props['offset']:
 				transfered_size -= file_props['offset'] 
 				full_size -= file_props['offset']
-			eta, speed = self._get_eta_and_speed(full_size, transfered_size, 
-				file_props['elapsed-time'])
 
-			self.model.set(iter, C_PROGRESS, text)
-			self.model.set(iter, C_PERCENT, int(percent))
+			if file_props['elapsed-time'] > 0:
+				file_props['transfered_size'].append((file_props['last-time'], transfered_size))
+			if len(file_props['transfered_size']) > 6:
+				file_props['transfered_size'].pop(0)
+			eta, speed = self._get_eta_and_speed(full_size, transfered_size, 
+				file_props)
+
+			self.model.set(iter_, C_PROGRESS, text)
+			self.model.set(iter_, C_PERCENT, int(percent))
 			text = self._format_time(eta)
 			text += '\n'
 			#This should make the string Kb/s, 
@@ -498,24 +525,24 @@ _('Connection with peer cannot be established.'))
 			#Only the 's' after / (which means second) should be translated.
 			text += _('(%(filesize_unit)s/s)') % {'filesize_unit':
 				helpers.convert_bytes(speed)}
-			self.model.set(iter, C_TIME, text)
+			self.model.set(iter_, C_TIME, text)
 
 			# try to guess what should be the status image
 			if file_props['type'] == 'r':
 				status = 'download'
 			else:
 				status = 'upload'
-			if file_props.has_key('paused') and file_props['paused'] == True:
+			if 'paused' in file_props and file_props['paused'] == True:
 				status = 'pause'
-			elif file_props.has_key('stalled') and file_props['stalled'] == True:
+			elif 'stalled' in file_props and file_props['stalled'] == True:
 				status = 'waiting'
-			if file_props.has_key('connected') and file_props['connected'] == False:
+			if 'connected' in file_props and file_props['connected'] == False:
 				status = 'stop'
-			self.model.set(iter, 0, self.images[status])
+			self.model.set(iter_, 0, self.get_icon(status))
 			if transfered_size == full_size:
 				self.set_status(typ, sid, 'ok')
 			elif just_began:
-				path = self.model.get_path(iter)
+				path = self.model.get_path(iter_)
 				self.select_func(path)
 
 	def get_iter_by_sid(self, typ, sid):
@@ -527,11 +554,12 @@ _('Connection with peer cannot be established.'))
 				return iter
 			iter = self.model.iter_next(iter)
 
-	def get_send_file_props(self, account, contact, file_path, file_name):
+	def get_send_file_props(self, account, contact, file_path, file_name,
+	file_desc=''):
 		''' create new file_props dict and set initial file transfer 
 		properties in it'''
 		file_props = {'file-name' : file_path, 'name' : file_name, 
-			'type' : 's'}
+			'type' : 's', 'desc' : file_desc}
 		if os.path.isfile(file_path):
 			stat = os.stat(file_path)
 		else:
@@ -549,6 +577,8 @@ _('Connection with peer cannot be established.'))
 		file_props['sender'] = account
 		file_props['receiver'] = contact
 		file_props['tt_account'] = account
+		# keep the last time: transfered_size to compute transfer speed
+		file_props['transfered_size'] = []
 		return file_props
 
 	def add_transfer(self, account, contact, file_props):
@@ -569,12 +599,12 @@ _('Connection with peer cannot be established.'))
 			(file_path, file_name) = os.path.split(file_props['file-name'])
 		else:
 			file_name = file_props['name']
-		text_props = gtkgui_helpers.escape_for_pango_markup(file_name) + '\n'
+		text_props = gobject.markup_escape_text(file_name) + '\n'
 		text_props += contact.get_shown_name()
 		self.model.set(iter, 1, text_labels, 2, text_props, C_SID,
 			file_props['type'] + file_props['sid'])
 		self.set_progress(file_props['type'], file_props['sid'], 0, iter)
-		if file_props.has_key('started') and file_props['started'] is False:
+		if 'started' in file_props and file_props['started'] is False:
 			status = 'waiting'
 		elif file_props['type'] == 'r':
 			status = 'download'
@@ -598,7 +628,7 @@ _('Connection with peer cannot be established.'))
 			iter = None
 			try:
 				iter = self.model.get_iter(row)
-			except:
+			except Exception:
 				self.tooltip.hide_tooltip()
 				return
 			sid = self.model[iter][C_SID].decode('utf-8')
@@ -626,33 +656,33 @@ _('Connection with peer cannot be established.'))
 		self.on_open_folder_menuitem_activate(widget)
 
 	def is_transfer_paused(self, file_props):
-		if file_props.has_key('stopped') and file_props['stopped']:
+		if 'stopped' in file_props and file_props['stopped']:
 			return False
-		if file_props.has_key('completed') and file_props['completed']:
+		if 'completed' in file_props and file_props['completed']:
 			return False
-		if not file_props.has_key('disconnect_cb'):
+		if 'disconnect_cb' not in file_props:
 			return False
 		return file_props['paused']
 
 	def is_transfer_active(self, file_props):
-		if file_props.has_key('stopped') and file_props['stopped']:
+		if 'stopped' in file_props and file_props['stopped']:
 			return False
-		if file_props.has_key('completed') and file_props['completed']:
+		if 'completed' in file_props and file_props['completed']:
 			return False
-		if not file_props.has_key('started') or not file_props['started']:
+		if 'started' not in file_props or not file_props['started']:
 			return False
-		if not file_props.has_key('paused'):
+		if 'paused' not in file_props:
 			return True
 		return not file_props['paused']
 
-	def is_transfer_stoped(self, file_props):
-		if file_props.has_key('error') and file_props['error'] != 0:
+	def is_transfer_stopped(self, file_props):
+		if 'error' in file_props and file_props['error'] != 0:
 			return True
-		if file_props.has_key('completed') and file_props['completed']:
+		if 'completed' in file_props and file_props['completed']:
 			return True
-		if file_props.has_key('connected') and file_props['connected'] == False:
+		if 'connected' in file_props and file_props['connected'] == False:
 			return True
-		if not file_props.has_key('stopped') or not file_props['stopped']:
+		if 'stopped' not in file_props or not file_props['stopped']:
 			return False
 		return True
 
@@ -687,7 +717,7 @@ _('Connection with peer cannot be established.'))
 		self.remove_menuitem.set_sensitive(is_row_selected)
 		self.open_folder_menuitem.set_sensitive(is_row_selected)
 		is_stopped = False
-		if self.is_transfer_stoped(file_props):
+		if self.is_transfer_stopped(file_props):
 			is_stopped = True
 		self.cancel_button.set_sensitive(not is_stopped)
 		self.cancel_menuitem.set_sensitive(not is_stopped)
@@ -741,7 +771,7 @@ _('Connection with peer cannot be established.'))
 			iter = self.model.get_iter((i))
 			sid = self.model[iter][C_SID].decode('utf-8')
 			file_props = self.files_props[sid[0]][sid[1:]]
-			if self.is_transfer_stoped(file_props):
+			if self.is_transfer_stopped(file_props):
 				self._remove_transfer(iter, sid, file_props)
 			i -= 1
 		self.tree.get_selection().unselect_all()
@@ -786,6 +816,8 @@ _('Connection with peer cannot be established.'))
 		elif self.is_transfer_active(file_props):
 			file_props['paused'] = True
 			self.set_status(file_props['type'], file_props['sid'], 'pause')
+			# reset that to compute speed only when we resume
+			file_props['transfered_size'] = []
 			self.toggle_pause_continue(False)
 
 	def on_cancel_button_clicked(self, widget):
@@ -795,10 +827,10 @@ _('Connection with peer cannot be established.'))
 		s_iter = selected[1]
 		sid = self.model[s_iter][C_SID].decode('utf-8')
 		file_props = self.files_props[sid[0]][sid[1:]]
-		if not file_props.has_key('tt_account'):
+		if 'tt_account' not in file_props:
 			return 
 		account = file_props['tt_account']
-		if not gajim.connections.has_key(account):
+		if account not in gajim.connections:
 			return
 		gajim.connections[account].disconnect_transfer(file_props)
 		self.set_status(file_props['type'], file_props['sid'], 'stop')
@@ -837,11 +869,11 @@ _('Connection with peer cannot be established.'))
 	def on_close_button_clicked(self, widget):
 		self.window.hide()
 
-	def show_context_menu(self, event, iter):
+	def show_context_menu(self, event, iter_):
 		# change the sensitive propery of the buttons and menuitems
 		path = None
-		if iter is not None:
-			path = self.model.get_path(iter)
+		if iter_ is not None:
+			path = self.model.get_path(iter_)
 		self.set_buttons_sensitive(path, True)
 
 		event_button = gtkgui_helpers.get_possible_button_event(event)
@@ -905,7 +937,7 @@ _('Connection with peer cannot be established.'))
 		s_iter = selected[1]
 		sid = self.model[s_iter][C_SID].decode('utf-8')
 		file_props = self.files_props[sid[0]][sid[1:]]
-		if not file_props.has_key('file-name'):
+		if 'file-name' not in file_props:
 			return
 		(path, file) = os.path.split(file_props['file-name'])
 		if os.path.exists(path) and os.path.isdir(path):
@@ -934,3 +966,5 @@ _('Connection with peer cannot be established.'))
 		if event.keyval == gtk.keysyms.Escape: # ESCAPE
 			self.window.hide()
 
+
+# vim: se ts=3:
