@@ -1,23 +1,31 @@
-### Copyright (C) 2005 Gustavo J. A. M. Carneiro
-### Copyright (C) 2006 Santiago Gala
-###
-### This library is free software; you can redistribute it and/or
-### modify it under the terms of the GNU Lesser General Public
-### License as published by the Free Software Foundation; either
-### version 2 of the License, or (at your option) any later version.
-###
-### This library is distributed in the hope that it will be useful,
-### but WITHOUT ANY WARRANTY; without even the implied warranty of
-### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-### Lesser General Public License for more details.
-###
-### You should have received a copy of the GNU Lesser General Public
-### License along with this library; if not, write to the
-### Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-### Boston, MA 02111-1307, USA.
+# -*- coding:utf-8 -*-
+## src/htmltextview.py
+##
+## Copyright (C) 2005 Gustavo J. A. M. Carneiro
+## Copyright (C) 2006 Santiago Gala
+## Copyright (C) 2006-2007 Jean-Marie Traissard <jim AT lapin.org>
+## Copyright (C) 2006-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2007 Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
+##                    Julien Pivotto <roidelapluie AT gmail.com>
+##                    Stephan Erb <steve-e AT h3c.de>
+##
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published
+## by the Free Software Foundation; version 3 only.
+##
+## Gajim is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+## GNU General Public License for more details.
+##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
-
-"""
+'''
 A gtk.TextView-based renderer for XHTML-IM, as described in:
   http://www.jabber.org/jeps/jep-0071.html
 
@@ -26,7 +34,7 @@ I (Santiago Gala) am trying to make it more compatible
 with the markup that docutils generate, and also more
 modular.
 
-"""
+'''
 
 import gobject
 import pango
@@ -49,8 +57,8 @@ import tooltips
 
 __all__ = ['HtmlTextView']
 
-whitespace_rx = re.compile("\\s+")
-allwhitespace_rx = re.compile("^\\s*$")
+whitespace_rx = re.compile('\\s+')
+allwhitespace_rx = re.compile('^\\s*$')
 
 # pixels = points * display_resolution
 display_resolution = 0.3514598*(gtk.gdk.screen_height() /
@@ -83,7 +91,7 @@ element_styles['tt']  = element_styles['kbd']
 element_styles['i']   = element_styles['em']
 element_styles['b']   = element_styles['strong']
 
-"""
+'''
 ==========
   JEP-0071
 ==========
@@ -153,7 +161,7 @@ Common.extra
 #           ( structural     =  br | span )
 #Param/Legacy    param, font, basefont, center, s, strike, u, dir, menu, isindex
 #
-"""
+'''
 
 BLOCK_HEAD = set(( 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', ))
 BLOCK_PHRASAL = set(( 'address', 'blockquote', 'pre', ))
@@ -214,9 +222,10 @@ def build_patterns(view, config, interface):
 			'(?:(?<![\w.]' + emoticons_pattern_prematch[:-1]   + '))' + \
 			'(?:'       + emoticons_pattern[:-1]            + ')'  + \
 			'(?:(?![\w.]'  + emoticons_pattern_postmatch[:-1]  + '))'
-	except:
+	except Exception:
 		pass
 
+	view.emot_pattern_re = re.compile(emoticons_pattern, re.IGNORECASE)
 	# because emoticons match later (in the string) they need to be after
 	# basic matches that may occur earlier
 	emot_and_basic_pattern = basic_pattern + emoticons_pattern
@@ -225,7 +234,7 @@ def build_patterns(view, config, interface):
 
 def _parse_css_color(color):
 	'''_parse_css_color(css_color) -> gtk.gdk.Color'''
-	if color.startswith("rgb(") and color.endswith(')'):
+	if color.startswith('rgb(') and color.endswith(')'):
 		r, g, b = [int(c)*257 for c in color[4:-1].split(',')]
 		return gtk.gdk.Color(r, g, b)
 	else:
@@ -255,15 +264,15 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 
 	def _parse_style_color(self, tag, value):
 		color = _parse_css_color(value)
-		tag.set_property("foreground-gdk", color)
+		tag.set_property('foreground-gdk', color)
 
 	def _parse_style_background_color(self, tag, value):
 		color = _parse_css_color(value)
-		tag.set_property("background-gdk", color)
-		if gtk.gtk_version >= (2, 8):
-			tag.set_property("paragraph-background-gdk", color)
+		tag.set_property('background-gdk', color)
+		tag.set_property('paragraph-background-gdk', color)
 
 
+	#FIXME: when we migrate to 2.10 rm this
 	if gtk.gtk_version >= (2, 8, 5) or gobject.pygtk_version >= (2, 8, 1):
 
 		def _get_current_attributes(self):
@@ -279,7 +288,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 		def _get_current_style_attr(self, propname, comb_oper=None):
 			tags = [tag for tag in self.styles if tag is not None]
 			tags.reverse()
-			is_set_name = propname + "-set"
+			is_set_name = propname + '-set'
 			value = None
 			for tag in tags:
 				if tag.get_property(is_set_name):
@@ -292,15 +301,15 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			return value
 
 		class _FakeAttrs(object):
-			__slots__ = ("font", "font_scale")
+			__slots__ = ('font', 'font_scale')
 
 		def _get_current_attributes(self):
 			attrs = self._FakeAttrs()
-			attrs.font_scale = self._get_current_style_attr("scale",
+			attrs.font_scale = self._get_current_style_attr('scale',
 															operator.mul)
 			if attrs.font_scale is None:
 				attrs.font_scale = 1.0
-			attrs.font = self._get_current_style_attr("font-desc")
+			attrs.font = self._get_current_style_attr('font-desc')
 			if attrs.font is None:
 				attrs.font = self.textview.style.font_desc
 			return attrs
@@ -324,14 +333,14 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 				font_size = attrs.font.get_size() / pango.SCALE
 				callback(frac*display_resolution*font_size, *args)
 			elif block_relative:
-				# CSS says "Percentage values: refer to width of the closest
-				#           block-level ancestor"
+				# CSS says 'Percentage values: refer to width of the closest
+				#           block-level ancestor'
 				# This is difficult/impossible to implement, so we use
 				# textview width instead; a reasonable approximation..
 				alloc = self.textview.get_allocation()
 				self.__parse_length_frac_size_allocate(self.textview, alloc,
 													   frac, callback, args)
-				self.textview.connect("size-allocate",
+				self.textview.connect('size-allocate',
 									  self.__parse_length_frac_size_allocate,
 									  frac, callback, args)
 			else:
@@ -368,11 +377,11 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 				# validate length
 				val = sign*max(minl,min(abs(val),maxl))
 				callback(val, *args)
-			except:
+			except Exception:
 				warnings.warn('Unable to parse length value "%s"' % value)
-
+		
 	def __parse_font_size_cb(length, tag):
-		tag.set_property("size-points", length/display_resolution)
+		tag.set_property('size-points', length/display_resolution)
 	__parse_font_size_cb = staticmethod(__parse_font_size_cb)
 
 	def _parse_style_display(self, tag, value):
@@ -383,25 +392,25 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 	def _parse_style_font_size(self, tag, value):
 		try:
 			scale = {
-				"xx-small": pango.SCALE_XX_SMALL,
-				"x-small": pango.SCALE_X_SMALL,
-				"small": pango.SCALE_SMALL,
-				"medium": pango.SCALE_MEDIUM,
-				"large": pango.SCALE_LARGE,
-				"x-large": pango.SCALE_X_LARGE,
-				"xx-large": pango.SCALE_XX_LARGE,
+				'xx-small': pango.SCALE_XX_SMALL,
+				'x-small': pango.SCALE_X_SMALL,
+				'small': pango.SCALE_SMALL,
+				'medium': pango.SCALE_MEDIUM,
+				'large': pango.SCALE_LARGE,
+				'x-large': pango.SCALE_X_LARGE,
+				'xx-large': pango.SCALE_XX_LARGE,
 				} [value]
 		except KeyError:
 			pass
 		else:
 			attrs = self._get_current_attributes()
-			tag.set_property("scale", scale / attrs.font_scale)
+			tag.set_property('scale', scale / attrs.font_scale)
 			return
 		if value == 'smaller':
-			tag.set_property("scale", pango.SCALE_SMALL)
+			tag.set_property('scale', pango.SCALE_SMALL)
 			return
 		if value == 'larger':
-			tag.set_property("scale", pango.SCALE_LARGE)
+			tag.set_property('scale', pango.SCALE_LARGE)
 			return
 		# font relative (5 ~ 4pt, 110 ~ 72pt)
 		self._parse_length(value, True, False, 5, 110, self.__parse_font_size_cb, tag)
@@ -409,14 +418,14 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 	def _parse_style_font_style(self, tag, value):
 		try:
 			style = {
-				"normal": pango.STYLE_NORMAL,
-				"italic": pango.STYLE_ITALIC,
-				"oblique": pango.STYLE_OBLIQUE,
+				'normal': pango.STYLE_NORMAL,
+				'italic': pango.STYLE_ITALIC,
+				'oblique': pango.STYLE_OBLIQUE,
 				} [value]
 		except KeyError:
-			warnings.warn("unknown font-style %s" % value)
+			warnings.warn('unknown font-style %s' % value)
 		else:
-			tag.set_property("style", style)
+			tag.set_property('style', style)
 
 	def __frac_length_tag_cb(self,length, tag, propname):
 		styles = self._get_style_tags()
@@ -428,12 +437,12 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 	def _parse_style_margin_left(self, tag, value):
 		# block relative
 		self._parse_length(value, False, True, 1, 1000, self.__frac_length_tag_cb,
-						   tag, "left-margin")
+						   tag, 'left-margin')
 
 	def _parse_style_margin_right(self, tag, value):
 		# block relative
 		self._parse_length(value, False, True, 1, 1000, self.__frac_length_tag_cb,
-						   tag, "right-margin")
+						   tag, 'right-margin')
 
 	def _parse_style_font_weight(self, tag, value):
 		# TODO: missing 'bolder' and 'lighter'
@@ -452,12 +461,12 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 				'bold': pango.WEIGHT_BOLD,
 				} [value]
 		except KeyError:
-			warnings.warn("unknown font-style %s" % value)
+			warnings.warn('unknown font-style %s' % value)
 		else:
-			tag.set_property("weight", weight)
+			tag.set_property('weight', weight)
 
 	def _parse_style_font_family(self, tag, value):
-		tag.set_property("family", value)
+		tag.set_property('family', value)
 
 	def _parse_style_text_align(self, tag, value):
 		try:
@@ -468,41 +477,40 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 				'justify': gtk.JUSTIFY_FILL,
 				} [value]
 		except KeyError:
-			warnings.warn("Invalid text-align:%s requested" % value)
+			warnings.warn('Invalid text-align:%s requested' % value)
 		else:
-			tag.set_property("justification", align)
+			tag.set_property('justification', align)
 	
 	def _parse_style_text_decoration(self, tag, value):
-		if value == "none":
-			tag.set_property("underline", pango.UNDERLINE_NONE)
-			tag.set_property("strikethrough", False)
-		elif value == "underline":
-			tag.set_property("underline", pango.UNDERLINE_SINGLE)
-			tag.set_property("strikethrough", False)
-		elif value == "overline":
-			warnings.warn("text-decoration:overline not implemented")
-			tag.set_property("underline", pango.UNDERLINE_NONE)
-			tag.set_property("strikethrough", False)
-		elif value == "line-through":
-			tag.set_property("underline", pango.UNDERLINE_NONE)
-			tag.set_property("strikethrough", True)
-		elif value == "blink":
-			warnings.warn("text-decoration:blink not implemented")
+		values = value.split(' ')
+		if 'none' in values:
+			tag.set_property('underline', pango.UNDERLINE_NONE)
+			tag.set_property('strikethrough', False)
+		if 'underline' in values:
+			tag.set_property('underline', pango.UNDERLINE_SINGLE)
 		else:
-			warnings.warn("text-decoration:%s not implemented" % value)
+			tag.set_property('underline', pango.UNDERLINE_NONE)
+		if 'line-through' in values:
+			tag.set_property('strikethrough', True)
+		else:
+			tag.set_property('strikethrough', False)
+		if 'blink' in values:
+			warnings.warn('text-decoration:blink not implemented')
+		if 'overline' in values:
+			warnings.warn('text-decoration:overline not implemented')
 	
 	def _parse_style_white_space(self, tag, value):
 		if value == 'pre':
-			tag.set_property("wrap_mode", gtk.WRAP_NONE)
+			tag.set_property('wrap_mode', gtk.WRAP_NONE)
 		elif value == 'normal':
-			tag.set_property("wrap_mode", gtk.WRAP_WORD)
+			tag.set_property('wrap_mode', gtk.WRAP_WORD)
 		elif value == 'nowrap':
-			tag.set_property("wrap_mode", gtk.WRAP_NONE)
+			tag.set_property('wrap_mode', gtk.WRAP_NONE)
 
 	def __length_tag_cb(self, value, tag, propname):
 		try:
 			tag.set_property(propname, value)
-		except:
+		except Exception:
 			gajim.log.warn( "Error with prop: " + propname + " for tag: " + str(tag))
 		
 
@@ -520,14 +528,14 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 	
 	# build a dictionary mapping styles to methods, for greater speed
 	__style_methods = dict()
-	for style in ['background-color', 'color', 'font-family', 'font-size',
+	for style in ('background-color', 'color', 'font-family', 'font-size',
 				  'font-style', 'font-weight', 'margin-left', 'margin-right',
 				  'text-align', 'text-decoration', 'white-space', 'display',
-				  'width', 'height' ]:
+				  'width', 'height' ):
 		try:
-			method = locals()["_parse_style_%s" % style.replace('-', '_')]
+			method = locals()['_parse_style_%s' % style.replace('-', '_')]
 		except KeyError:
-			warnings.warn("Style attribute '%s' not yet implemented" % style)
+			warnings.warn('Style attribute "%s" not yet implemented' % style)
 		else:
 			__style_methods[style] = method
 	del style
@@ -560,14 +568,14 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			try: 
 				f = urllib2.urlopen(attrs['src']) 
 			except Exception, ex: 
-				gajim.log.debug(str('Error loading image %s ' % attrs['src'] + ex)) 
+				gajim.log.debug('Error loading image %s ' % attrs['src']  + str(ex))
 				pixbuf = None 
 				alt = attrs.get('alt', 'Broken image') 
 			else: 
 				# Wait 0.1s between each byte 
 				try: 
 					f.fp._sock.fp._sock.settimeout(0.5) 
-				except: 
+				except Exception: 
 					pass 
 			# Max image size = 2 MB (to try to prevent DoS)
 			mem = ''
@@ -633,11 +641,11 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 					pifbufs for every resize, gtk.gdk.Pixbuf.scale_simple
 					or similar.
 					'''
-					if type(dims[0]) == float:
+					if isinstance(dims[0], float):
 						dims[0] = int(dims[0]*w)
 					elif not dims[0]:
 						dims[0] = w
-					if type(dims[1]) == float:
+					if isinstance(dims[1], float):
 						dims[1] = int(dims[1]*h)
 					if not dims[1]:
 						dims[1] = h
@@ -667,7 +675,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			alt = attrs.get('alt', 'Broken image')
 			try:
 				loader.close()
-			except:
+			except Exception:
 				pass
 		return pixbuf
 
@@ -686,8 +694,8 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			try:
 				method = self.__style_methods[attr]
 			except KeyError:
-				warnings.warn("Style attribute '%s' requested "
-							  "but not yet implemented" % attr)
+				warnings.warn('Style attribute "%s" requested '
+							  'but not yet implemented' % attr)
 			else:
 				method(self, tag, val)
 		self.styles.append(tag)
@@ -718,21 +726,27 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			text = text.replace('\n', ' ')
 			self.handle_specials(whitespace_rx.sub(' ', text))
 		else:
-			self._insert_text(text.strip("\n"))
+			self._insert_text(text.strip('\n'))
 
-	def _anchor_event(self, tag, textview, event, iter, href, type_):
+	def _anchor_event(self, tag, textview, event, iter_, href, type_):
 		if event.type == gtk.gdk.BUTTON_PRESS:
-			self.textview.emit("url-clicked", href, type_)
+			self.textview.emit('url-clicked', href, type_)
 			return True
 		return False
 
 	def handle_specials(self, text):
 		index = 0
 		se = self.textview.config.get('show_ascii_formatting_chars')
+		af = gajim.config.get('ascii_formatting')
 		if self.textview.config.get('emoticons_theme'):
-			iterator = self.textview.emot_and_basic_re.finditer(text)
-		else:
+			if af:
+				iterator = self.textview.emot_and_basic_re.finditer(text)
+			else:
+				iterator = self.textview.emot_pattern_re.finditer(text)
+		elif af:
 			iterator = self.textview.basic_pattern_re.finditer(text)
+		else:
+			iterator = []
 		for match in iterator:
 			start, end = match.span()
 			special_text = text[start:end]
@@ -751,12 +765,12 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 				img.show()
 				# TODO: add alt/tooltip with the special_text (a11y) 
 				self.textview.add_child_at_anchor(img, anchor)
-			else:
+			elif af:
 				# now print it
 				if special_text.startswith('/'): # it's explicit italics
 					self.startElement('i', {})
 				elif special_text.startswith('_'): # it's explicit underline
-					self.startElement("u", {})
+					self.startElement('u', {})
 				if se: self._insert_text(special_text[0])
 				self.handle_specials(special_text[1:-1])
 				if se: self._insert_text(special_text[0])
@@ -845,7 +859,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 				li_head = unichr(0x2022)
 			else:
 				self.list_counters[-1] += 1
-				li_head = "%i." % self.list_counters[-1]
+				li_head = '%i.' % self.list_counters[-1]
 			self.text = ' '*len(self.list_counters)*4 + li_head + ' '
 			self._flush_text()
 			self.starting = True
@@ -859,7 +873,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 		elif name in INLINE:
 			pass
 		else:
-			warnings.warn("Unhandled element '%s'" % name)
+			warnings.warn('Unhandled element "%s"' % name)
 
 	def endElement(self, name):
 		endPreserving = False
@@ -871,10 +885,10 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			self._jump_line()
 			try:
 				self.textbuf.insert_pixbuf(self.iter, self.textview.focus_out_line_pixbuf)
-				#self._insert_text(u"\u2550"*40)
+				#self._insert_text(u'\u2550'*40)
 				self._jump_line()
 			except Exception, e:
-				gajim.log.debug(str("Error in hr"+e))
+				gajim.log.debug(str('Error in hr'+e))
 		elif name in LIST_ELEMS:
 			self.list_counters.pop()
 		elif name == 'li':
@@ -986,10 +1000,8 @@ class HtmlTextView(gtk.TextView):
 		
 		# too much space after :)
 		#if not eob.starts_line():
-		#    buffer.insert(eob, "\n")
+		#    buffer.insert(eob, '\n')
 
-if gobject.pygtk_version < (2, 8):
-	gobject.type_register(HtmlTextView)
 
 
 change_cursor = None
@@ -1056,7 +1068,7 @@ if __name__ == '__main__':
 
 	htmlview.connect('motion_notify_event', on_textview_motion_notify_event)
 
-	def handler(texttag, widget, event, iter, kind, href):
+	def handler(texttag, widget, event, iter_, kind, href):
 		if event.type == gtk.gdk.BUTTON_PRESS:
 			print href
 
@@ -1066,25 +1078,25 @@ if __name__ == '__main__':
 						  '  <img src="http://images.slashdot.org/topics/topicsoftware.gif"/><br/>\n'
 						  '  <span style="font-size: 500%; font-family: serif">World</span>\n'
 						  '</div>\n')
-	htmlview.display_html("<hr />")
-	htmlview.display_html("""
+	htmlview.display_html('<hr />')
+	htmlview.display_html('''
 	  <p style='font-size:large'>
 		<span style='font-style: italic'>O<span style='font-size:larger'>M</span>G</span>, 
 		I&apos;m <span style='color:green'>green</span>
 		with <span style='font-weight: bold'>envy</span>!
 	  </p>
-		""")
-	htmlview.display_html("<hr />")
-	htmlview.display_html("""
+		''')
+	htmlview.display_html('<hr />')
+	htmlview.display_html('''
 	<body xmlns='http://www.w3.org/1999/xhtml'>
 	  <p>As Emerson said in his essay <span style='font-style: italic; background-color:cyan'>Self-Reliance</span>:</p>
 	  <p style='margin-left: 5px; margin-right: 2%'>
 		&quot;A foolish consistency is the hobgoblin of little minds.&quot;
 	  </p>
 	</body>
-		""")
-	htmlview.display_html("<hr />")
-	htmlview.display_html("""
+		''')
+	htmlview.display_html('<hr />')
+	htmlview.display_html('''
 	<body xmlns='http://www.w3.org/1999/xhtml'>
 	  <p style='text-align:center'>Hey, are you licensed to <a href='http://www.jabber.org/'>Jabber</a>?</p>
 	  <p style='text-align:right'><img src='http://www.jabber.org/images/psa-license.jpg'
@@ -1092,9 +1104,9 @@ if __name__ == '__main__':
 			  width='50%' height='50%' 
 			  /></p>
 	</body>
-		""")
-	htmlview.display_html("<hr />")
-	htmlview.display_html("""
+		''')
+	htmlview.display_html('<hr />')
+	htmlview.display_html('''
 	<body xmlns='http://www.w3.org/1999/xhtml'>
 	  <ul style='background-color:rgb(120,140,100)'>
 	   <li> One </li>
@@ -1104,12 +1116,12 @@ if __name__ == '__main__':
   def faciter(n,acc): 
 	if n==0: return acc
 	return faciter(n-1, acc*n)
-  if n&lt;0: raise ValueError("Must be non-negative")
+  if n&lt;0: raise ValueError('Must be non-negative')
   return faciter(n,1)</pre>
 	</body>
-		""")
-	htmlview.display_html("<hr />")
-	htmlview.display_html("""
+		''')
+	htmlview.display_html('<hr />')
+	htmlview.display_html('''
 	<body xmlns='http://www.w3.org/1999/xhtml'>
 	 <ol style='background-color:rgb(120,140,100)'>
 	   <li> One </li>
@@ -1121,12 +1133,12 @@ if __name__ == '__main__':
 			</ul></li>
 	   <li> Three </li></ol>
 	</body>
-		""")
+		''')
 	htmlview.show()
 	sw = gtk.ScrolledWindow()
-	sw.set_property("hscrollbar-policy", gtk.POLICY_AUTOMATIC)
-	sw.set_property("vscrollbar-policy", gtk.POLICY_AUTOMATIC)
-	sw.set_property("border-width", 0)
+	sw.set_property('hscrollbar-policy', gtk.POLICY_AUTOMATIC)
+	sw.set_property('vscrollbar-policy', gtk.POLICY_AUTOMATIC)
+	sw.set_property('border-width', 0)
 	sw.add(htmlview)
 	sw.show()
 	frame = gtk.Frame()
@@ -1137,5 +1149,7 @@ if __name__ == '__main__':
 	w.add(frame)
 	w.set_default_size(400, 300)
 	w.show_all()
-	w.connect("destroy", lambda w: gtk.main_quit())
+	w.connect('destroy', lambda w: gtk.main_quit())
 	gtk.main()
+
+# vim: se ts=3:
