@@ -1,25 +1,36 @@
-##	notify.py
+# -*- coding:utf-8 -*-
+## src/notify.py
 ##
-## Copyright (C) 2005-2006 Yann Le Boulanger <asterix@lagaule.org>
-## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
-## Copyright (C) 2005-2006 Andrew Sayman <lorien420@myrealbox.com>
+## Copyright (C) 2005 Sebastian Estienne
+## Copyright (C) 2005-2006 Andrew Sayman <lorien420 AT myrealbox.com>
+## Copyright (C) 2005-2007 Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2005-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006 Travis Shirk <travis AT pobox.com>
+## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+## Copyright (C) 2007 Julien Pivotto <roidelapluie AT gmail.com>
+##                    Stephan Erb <steve-e AT h3c.de>
+## Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
+##                    Jonathan Schleifer <js-gajim AT webkeks.org>
 ##
-## Notification daemon connection via D-Bus code:
-## Copyright (C) 2005 by Sebastian Estienne
+## This file is part of Gajim.
 ##
-## This program is free software; you can redistribute it and/or modify
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 import os
 import time
 import dialogs
+import gobject
 import gtkgui_helpers
 
 from common import gajim
@@ -29,7 +40,6 @@ from common import dbus_support
 if dbus_support.supported:
 	import dbus
 	import dbus.glib
-	import dbus.service
 
 
 USER_HAS_PYNOTIFY = True # user has pynotify module
@@ -39,28 +49,40 @@ try:
 except ImportError:
 	USER_HAS_PYNOTIFY = False
 
-def get_show_in_roster(event, account, contact):
+USER_HAS_GROWL = True
+try:
+	import osx.growler
+	osx.growler.init()
+except Exception:
+	USER_HAS_GROWL = False
+
+def get_show_in_roster(event, account, contact, session=None):
 	'''Return True if this event must be shown in roster, else False'''
+	if event == 'gc_message_received':
+		return True
 	num = get_advanced_notification(event, account, contact)
-	if num != None:
+	if num is not None:
 		if gajim.config.get_per('notifications', str(num), 'roster') == 'yes':
 			return True
 		if gajim.config.get_per('notifications', str(num), 'roster') == 'no':
 			return False
 	if event == 'message_received':
-		chat_control = helpers.get_chat_control(account, contact)
-		if chat_control:
+		if session and session.control:
 			return False
 	return True
 
-def get_show_in_systray(event, account, contact):
+def get_show_in_systray(event, account, contact, type_=None):
 	'''Return True if this event must be shown in systray, else False'''
 	num = get_advanced_notification(event, account, contact)
-	if num != None:
+	if num is not None:
 		if gajim.config.get_per('notifications', str(num), 'systray') == 'yes':
 			return True
 		if gajim.config.get_per('notifications', str(num), 'systray') == 'no':
 			return False
+	if type_ == 'printed_gc_msg' and not gajim.config.get(
+	'notify_on_all_muc_messages'):
+		# it's not an highlighted message, don't show in systray
+		return False
 	return gajim.config.get('trayicon_notification_on_events')
 
 def get_advanced_notification(event, account, contact):
@@ -111,8 +133,8 @@ def get_advanced_notification(event, account, contact):
 		num += 1
 		notif = gajim.config.get_per('notifications', str(num))
 
-def notify(event, jid, account, parameters, advanced_notif_num = None):
-	'''Check what type of notifications we want, depending on basic 
+def notify(event, jid, account, parameters, advanced_notif_num=None):
+	'''Check what type of notifications we want, depending on basic
 	and the advanced configuration of notifications and do these notifications;
 	advanced_notif_num holds the number of the first (top most) advanced
 	notification'''
@@ -151,15 +173,27 @@ def notify(event, jid, account, parameters, advanced_notif_num = None):
 		message_type = parameters[0]
 		is_first_message = parameters[1]
 		nickname = parameters[2]
-		message = parameters[3]
+		if gajim.config.get('notification_preview_message'):
+			message = parameters[3]
+			if message.startswith('/me ') or message.startswith('/me\n'):
+				message = '* ' + nickname + message[3:]
+		else:
+			# We don't want message preview, do_preview = False
+			message = ''
+		focused = parameters[4]
 		if helpers.allow_showing_notification(account, 'notify_on_new_message',
 		advanced_notif_num, is_first_message):
 			do_popup = True
 		if is_first_message and helpers.allow_sound_notification(
 		'first_message_received', advanced_notif_num):
 			do_sound = True
-		elif not is_first_message and helpers.allow_sound_notification(
-		'next_message_received', advanced_notif_num):
+		elif not is_first_message and focused and \
+		helpers.allow_sound_notification('next_message_received_focused',
+		advanced_notif_num):
+			do_sound = True
+		elif not is_first_message and not focused and \
+		helpers.allow_sound_notification('next_message_received_unfocused',
+		advanced_notif_num):
 			do_sound = True
 	else:
 		print '*Event not implemeted yet*'
@@ -167,29 +201,29 @@ def notify(event, jid, account, parameters, advanced_notif_num = None):
 	if advanced_notif_num is not None and gajim.config.get_per('notifications',
 	str(advanced_notif_num), 'run_command'):
 		do_cmd = True
-			
-	# Do the wanted notifications	
+
+	# Do the wanted notifications
 	if do_popup:
 		if event in ('contact_connected', 'contact_disconnected',
 		'status_change'): # Common code for popup for these three events
 			if event == 'contact_disconnected':
 				show_image = 'offline.png'
-				suffix = '_notif_size_bw.png'
+				suffix = '_notif_size_bw'
 			else: #Status Change or Connected
 				# FIXME: for status change,
-				# we don't always 'online.png', but we 
+				# we don't always 'online.png', but we
 				# first need 48x48 for all status
 				show_image = 'online.png'
-				suffix = '_notif_size_colored.png'	
+				suffix = '_notif_size_colored'
 			transport_name = gajim.get_transport_name_from_jid(jid)
 			img = None
 			if transport_name:
-				img = os.path.join(gajim.DATA_DIR, 'iconsets',
-					'transports', transport_name, '48x48', show_image) 
+				img = os.path.join(helpers.get_transport_path(transport_name),
+					'48x48', show_image)
 			if not img or not os.path.isfile(img):
 				iconset = gajim.config.get('iconset')
-				img = os.path.join(gajim.DATA_DIR, 'iconsets',
-						iconset, '48x48', show_image)
+				img = os.path.join(helpers.get_iconset_path(iconset), '48x48',
+					show_image)
 			path = gtkgui_helpers.get_path_to_generic_or_avatar(img,
 				jid = jid, suffix = suffix)
 			if event == 'status_change':
@@ -199,9 +233,9 @@ def notify(event, jid, account, parameters, advanced_notif_num = None):
 					{'nick': gajim.get_name_from_jid(account, jid),\
 					'status': helpers.get_uf_show(gajim.SHOW_LIST[new_show])}
 				if status_message:
-					text =  text + " : " + status_message
+					text = text + " : " + status_message
 				popup(_('Contact Changed Status'), jid, account,
-					path_to_image = path, title = title, text = text)
+					path_to_image=path, title=title, text=text)
 			elif event == 'contact_connected':
 				title = _('%(nickname)s Signed In') % \
 					{'nickname': gajim.get_name_from_jid(account, jid)}
@@ -209,7 +243,7 @@ def notify(event, jid, account, parameters, advanced_notif_num = None):
 				if status_message:
 					text = status_message
 				popup(_('Contact Signed In'), jid, account,
-					path_to_image = path, title = title, text = text)
+					path_to_image=path, title=title, text=text)
 			elif event == 'contact_disconnected':
 				title = _('%(nickname)s Signed Out') % \
 					{'nickname': gajim.get_name_from_jid(account, jid)}
@@ -217,7 +251,7 @@ def notify(event, jid, account, parameters, advanced_notif_num = None):
 				if status_message:
 					text = status_message
 				popup(_('Contact Signed Out'), jid, account,
-					path_to_image = path, title = title, text = text)
+					path_to_image=path, title=title, text=text)
 		elif event == 'new_message':
 			if message_type == 'normal': # single message
 				event_type = _('New Single Message')
@@ -232,8 +266,12 @@ def notify(event, jid, account, parameters, advanced_notif_num = None):
 				img = os.path.join(gajim.DATA_DIR, 'pixmaps', 'events',
 					'priv_msg_recv.png')
 				title = _('New Private Message from group chat %s') % room_name
-				text = _('%(nickname)s: %(message)s') % {'nickname': nickname,
-					'message': message}
+				if message:
+					text = _('%(nickname)s: %(message)s') % {'nickname': nickname,
+						'message': message}
+				else:
+					text = _('Messaged by %(nickname)s') % {'nickname': nickname}
+
 			else: # chat message
 				event_type = _('New Message')
 				img = os.path.join(gajim.DATA_DIR, 'pixmaps', 'events',
@@ -243,7 +281,7 @@ def notify(event, jid, account, parameters, advanced_notif_num = None):
 				text = message
 			path = gtkgui_helpers.get_path_to_generic_or_avatar(img)
 			popup(event_type, jid, account, message_type,
-				path_to_image = path, title = title, text = text)
+				path_to_image=path, title=title, text=text)
 
 	if do_sound:
 		snd_file = None
@@ -258,8 +296,10 @@ def notify(event, jid, account, parameters, advanced_notif_num = None):
 				pass # do not set snd_event
 			elif is_first_message:
 				snd_event = 'first_message_received'
+			elif focused:
+				snd_event = 'next_message_received_focused'
 			else:
-				snd_event = 'next_message_received'
+				snd_event = 'next_message_received_unfocused'
 		elif event in ('contact_connected', 'contact_disconnected'):
 			snd_event = event
 		if snd_file:
@@ -272,21 +312,33 @@ def notify(event, jid, account, parameters, advanced_notif_num = None):
 			'command')
 		try:
 			helpers.exec_command(command)
-		except:
+		except Exception:
 			pass
 
-def popup(event_type, jid, account, msg_type = '', path_to_image = None,
-	title = None, text = None):
+def popup(event_type, jid, account, msg_type='', path_to_image=None,
+	title=None, text=None):
 	'''Notifies a user of an event. It first tries to a valid implementation of
 	the Desktop Notification Specification. If that fails, then we fall back to
 	the older style PopupNotificationWindow method.'''
-	text = gtkgui_helpers.escape_for_pango_markup(text)
-	title = gtkgui_helpers.escape_for_pango_markup(title)
 
+	# default image
+	if not path_to_image:
+		path_to_image = os.path.abspath(
+			os.path.join(gajim.DATA_DIR, 'pixmaps', 'events',
+				'chat_msg_recv.png')) # img to display
+
+	# Try Growl first, as we might have D-Bus and notification daemon running
+	# on OS X for some reason.
+	if USER_HAS_GROWL:
+		osx.growler.notify(event_type, jid, account, msg_type, path_to_image,
+			title, text)
+		return
+
+	# Try to show our popup via D-Bus and notification daemon
 	if gajim.config.get('use_notif_daemon') and dbus_support.supported:
 		try:
 			DesktopNotification(event_type, jid, account, msg_type,
-				path_to_image, title, text)
+				path_to_image, title, gobject.markup_escape_text(text))
 			return	# sucessfully did D-Bus Notification procedure!
 		except dbus.DBusException, e:
 			# Connection to D-Bus failed
@@ -294,28 +346,31 @@ def popup(event_type, jid, account, msg_type = '', path_to_image = None,
 		except TypeError, e:
 			# This means that we sent the message incorrectly
 			gajim.log.debug(str(e))
-	# we failed to speak to notification daemon via D-Bus
-	if USER_HAS_PYNOTIFY: # try via libnotify
-		if not text:
-			text = gajim.get_name_from_jid(account, jid) # default value of text
+
+	# Ok, that failed. Let's try pynotify, which also uses notification daemon
+	if gajim.config.get('use_notif_daemon') and USER_HAS_PYNOTIFY:
+		if not text and event_type == 'new_message':
+			# empty text for new_message means do_preview = False
+			# -> default value for text
+			_text = gobject.markup_escape_text(
+				gajim.get_name_from_jid(account, jid))
+		else:
+			_text = gobject.markup_escape_text(text)
+
 		if not title:
-			title = event_type
-		# default image
-		if not path_to_image:
-			path_to_image = os.path.abspath(
-				os.path.join(gajim.DATA_DIR, 'pixmaps', 'events',
-					'chat_msg_recv.png')) # img to display
-		
-		
-		notification = pynotify.Notification(title, text)
+			_title = ''
+		else:
+			_title = title
+
+		notification = pynotify.Notification(_title, _text)
 		timeout = gajim.config.get('notification_timeout') * 1000 # make it ms
 		notification.set_timeout(timeout)
-		
+
 		notification.set_category(event_type)
 		notification.set_data('event_type', event_type)
 		notification.set_data('jid', jid)
 		notification.set_data('account', account)
-		notification.set_data('msg_type', event_type)
+		notification.set_data('msg_type', msg_type)
 		notification.set_property('icon-name', path_to_image)
 		notification.add_action('default', 'Default Action',
 			on_pynotify_notification_clicked)
@@ -327,7 +382,7 @@ def popup(event_type, jid, account, msg_type = '', path_to_image = None,
 			# Connection to notification-daemon failed, see #2893
 			gajim.log.debug(str(e))
 
-	# go old style
+	# Either nothing succeeded or the user wants old-style notifications
 	instance = dialogs.PopupNotificationWindow(event_type, jid, account,
 		msg_type, path_to_image, title, text)
 	gajim.interface.roster.popup_notification_windows.append(instance)
@@ -356,12 +411,12 @@ class NotificationResponseManager:
 		self.interface.connect_to_signal('ActionInvoked', self.on_action_invoked)
 		self.interface.connect_to_signal('NotificationClosed', self.on_closed)
 
-	def on_action_invoked(self, id, reason):
-		self.received.append((id, time.time(), reason))
-		if self.pending.has_key(id):
-			notification = self.pending[id]
-			notification.on_action_invoked(id, reason)
-			del self.pending[id]
+	def on_action_invoked(self, id_, reason):
+		self.received.append((id_, time.time(), reason))
+		if id_ in self.pending:
+			notification = self.pending[id_]
+			notification.on_action_invoked(id_, reason)
+			del self.pending[id_]
 		if len(self.received) > 20:
 			curt = time.time()
 			for rec in self.received:
@@ -369,21 +424,21 @@ class NotificationResponseManager:
 				if diff > 10:
 					self.received.remove(rec)
 
-	def on_closed(self, id, reason = None):
-		if self.pending.has_key(id):
-			del self.pending[id]
+	def on_closed(self, id_, reason=None):
+		if id_ in self.pending:
+			del self.pending[id_]
 
-	def add_pending(self, id, object):
+	def add_pending(self, id_, object_):
 		# Check to make sure that we handle an event immediately if we're adding
 		# an id that's already been triggered
 		for rec in self.received:
-			if rec[0] == id:
-				object.on_action_invoked(id, rec[2])
+			if rec[0] == id_:
+				object_.on_action_invoked(id_, rec[2])
 				self.received.remove(rec)
 				return
-		if id not in self.pending:
+		if id_ not in self.pending:
 			# Add it
-			self.pending[id] = object
+			self.pending[id_] = object_
 		else:
 			# We've triggered an event that has a duplicate ID!
 			gajim.log.debug('Duplicate ID of notification. Can\'t handle this.')
@@ -393,8 +448,8 @@ notification_response_manager = NotificationResponseManager()
 class DesktopNotification:
 	'''A DesktopNotification that interfaces with D-Bus via the Desktop
 	Notification specification'''
-	def __init__(self, event_type, jid, account, msg_type = '',
-		path_to_image = None, title = None, text = None):
+	def __init__(self, event_type, jid, account, msg_type='',
+		path_to_image=None, title=None, text=None):
 		self.path_to_image = path_to_image
 		self.event_type = event_type
 		self.title = title
@@ -405,8 +460,9 @@ class DesktopNotification:
 		self.jid = jid
 		self.msg_type = msg_type
 
-		if not text:
-			# default value of text
+		# default value of text
+		if not text and event_type == 'new_message':
+			# empty text for new_message means do_preview = False
 			self.text = gajim.get_name_from_jid(account, jid)
 
 		if not title:
@@ -456,14 +512,14 @@ class DesktopNotification:
 				self.notif.Notify(
 					dbus.String(_('Gajim')),
 					dbus.String(self.path_to_image),
-					dbus.UInt32(0), 
-					ntype, 
+					dbus.UInt32(0),
+					ntype,
 					dbus.Byte(0),
 					dbus.String(self.title),
 					dbus.String(self.text),
 					[dbus.String(self.path_to_image)],
-					{'default': 0}, 
-					[''], 
+					{'default': 0},
+					[''],
 					True,
 					dbus.UInt32(timeout),
 					reply_handler=self.attach_by_id,
@@ -471,36 +527,50 @@ class DesktopNotification:
 			except AttributeError:
 				version = [0, 3, 1] # we're actually dealing with the newer version
 		if version > [0, 3]:
-			if version >= [0, 3, 2]:
+			if gajim.interface.systray_enabled and \
+			gajim.config.get('attach_notifications_to_systray'):
+				x, y = gajim.interface.systray.img_tray.window.get_origin()
+				x_, y_, width, height, depth = \
+					gajim.interface.systray.img_tray.window.get_geometry()
+				pos_x = x + (width / 2)
+				pos_y = y + (height / 2)
+				hints = {'x': pos_x, 'y': pos_y}
+			else:
 				hints = {}
+			if version >= [0, 3, 2]:
 				hints['urgency'] = dbus.Byte(0) # Low Urgency
 				hints['category'] = dbus.String(ntype)
+				# it seems notification-daemon doesn't like empty text 
+				if self.text:
+					text = self.text
+				else:
+					text = ' '
 				self.notif.Notify(
 					dbus.String(_('Gajim')),
 					dbus.UInt32(0), # this notification does not replace other
 					dbus.String(self.path_to_image),
 					dbus.String(self.title),
-					dbus.String(self.text), 
+					dbus.String(text),
 					( dbus.String('default'), dbus.String(self.event_type) ),
-					hints, 
+					hints,
 					dbus.UInt32(timeout*1000),
 					reply_handler=self.attach_by_id,
 					error_handler=self.notify_another_way)
 			else:
 				self.notif.Notify(
 					dbus.String(_('Gajim')),
-					dbus.String(self.path_to_image), 
+					dbus.String(self.path_to_image),
 					dbus.UInt32(0),
 					dbus.String(self.title),
 					dbus.String(self.text),
 					dbus.String(''),
-					{},
+					hints,
 					dbus.UInt32(timeout*1000),
 					reply_handler=self.attach_by_id,
 					error_handler=self.notify_another_way)
 
-	def attach_by_id(self, id):
-		self.id = id
+	def attach_by_id(self, id_):
+		self.id = id_
 		notification_response_manager.attach_to_interface()
 		notification_response_manager.add_pending(self.id, self)
 
@@ -508,16 +578,17 @@ class DesktopNotification:
 		gajim.log.debug(str(e))
 		gajim.log.debug('Need to implement a new way of falling back')
 
-	def on_action_invoked(self, id, reason):
+	def on_action_invoked(self, id_, reason):
 		if self.notif is None:
 			return
-		self.notif.CloseNotification(dbus.UInt32(id))
+		self.notif.CloseNotification(dbus.UInt32(id_))
 		self.notif = None
-		if not self.msg_type:
-			self.msg_type = 'chat'
+
 		gajim.interface.handle_event(self.account, self.jid, self.msg_type)
 
-	def version_reply_handler(self, name, vendor, version, spec_version = None):
+	def version_reply_handler(self, name, vendor, version, spec_version=None):
+		if spec_version:
+			version = spec_version
 		version_list = version.split('.')
 		self.version = []
 		while len(version_list):
@@ -536,3 +607,5 @@ class DesktopNotification:
 	def version_error_handler_3_x_try(self, e):
 		self.version = self.default_version
 		self.attempt_notify()
+
+# vim: se ts=3:
